@@ -25,6 +25,8 @@
 #include <pgm/pgm.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+static gboolean iconified = FALSE;
+
 static gboolean
 drawable_pressed_cb (PgmDrawable *drawable,
                      gfloat x,
@@ -204,6 +206,23 @@ drawable_left_cb (PgmDrawable *drawable,
   pgm_drawable_set_opacity (drawable, 150);
 }
 
+static void
+state_event_cb (PgmViewport *viewport,
+                PgmEventState *event,
+                gpointer data)
+{
+  if (event->state_mask & PGM_VIEWPORT_ICONIFIED)
+    {
+      g_print ("%s iconified\n", GST_OBJECT_NAME (viewport));
+      iconified = TRUE;
+    }
+  else
+    {
+      g_print ("%s deiconified\n", GST_OBJECT_NAME (viewport));
+      iconified = FALSE;
+    }
+}
+
 static void
 keypress_event_cb (PgmViewport *viewport,
                    PgmEventKey *event,
@@ -221,6 +240,12 @@ keypress_event_cb (PgmViewport *viewport,
           pgm_viewport_set_fullscreen (viewport, fullscreen);
           break;
 
+          /* Iconify */
+        case PGM_i:
+          iconified = !iconified;
+          pgm_viewport_set_iconified (viewport, iconified);
+          break;
+
           /* Quit */
         case PGM_q:
         case PGM_Escape:
@@ -371,6 +396,8 @@ main (int argc,
                     G_CALLBACK (keypress_event_cb), NULL);
   g_signal_connect (G_OBJECT (viewport), "delete-event",
                     G_CALLBACK (pgm_main_quit), NULL);
+  g_signal_connect (G_OBJECT (viewport), "state-event",
+                    G_CALLBACK (state_event_cb), NULL);
   pgm_viewport_show (viewport);
   pgm_main ();
 
