@@ -162,18 +162,21 @@ main (int argc,
 
   /* Text definitions */
   txt[X] = pgm_text_new ("Press 'x': OFF");
+  pgm_text_set_font_height (PGM_TEXT (txt[X]), 20.0f);
   pgm_text_set_weight (PGM_TEXT (txt[X]), PGM_TEXT_WEIGHT_BOLD);
   pgm_drawable_set_size (txt[X], 300, 20);
   pgm_drawable_set_position (txt[X], 10, 0, 0);
   pgm_drawable_set_bg_color (txt[X], 0, 0, 0, 0);
   pgm_drawable_show (txt[X]);
   txt[Y] = pgm_text_new ("Press 'y': OFF");
+  pgm_text_set_font_height (PGM_TEXT (txt[Y]), 20.0f);
   pgm_text_set_weight (PGM_TEXT (txt[Y]), PGM_TEXT_WEIGHT_BOLD);
   pgm_drawable_set_size (txt[Y], 300, 20);
   pgm_drawable_set_position (txt[Y], 10, 20, 0);
   pgm_drawable_set_bg_color (txt[Y], 0, 0, 0, 0);
   pgm_drawable_show (txt[Y]);
   txt[Z] = pgm_text_new ("Press 'z': OFF");
+  pgm_text_set_font_height (PGM_TEXT (txt[Z]), 20.0f);
   pgm_text_set_weight (PGM_TEXT (txt[Z]), PGM_TEXT_WEIGHT_BOLD);
   pgm_drawable_set_size (txt[Z], 300, 20);
   pgm_drawable_set_position (txt[Z], 10, 40, 0);
