@@ -24,6 +24,7 @@
 
 #include <pgm/pgm.h>
 
+static PgmDrawable *img = NULL;
 static gboolean pressed = FALSE;
 static gfloat last_x = 0;
 static gfloat last_y = 0;
@@ -74,12 +75,50 @@ motion_notify_cb (PgmViewport *viewport,
     }
 }
 
+static void
+keypress_event_cb (PgmViewport *viewport,
+                   PgmEventKey *event,
+                   gpointer data)
+{
+  static gboolean fullscreen = FALSE;
+  static gboolean visible = TRUE;
+
+  if (event->type == PGM_KEY_PRESS)
+    {
+      switch (event->keyval)
+        {
+          /* Fullscreen */
+        case PGM_f:
+          fullscreen = !fullscreen;
+          pgm_viewport_set_fullscreen (viewport, fullscreen);
+          break;
+
+          /* Show/Hide */
+        case PGM_v:
+          visible = !visible;
+          if (visible)
+            pgm_drawable_show (img);
+          else
+            pgm_drawable_hide (img);
+          break;
+
+          /* Quit */
+        case PGM_q:
+        case PGM_Escape:
+          pgm_main_quit ();
+          break;
+
+        default:
+          break;
+        }
+    }
+}
+
 int
 main (int argc, char *argv[])
 {
   PgmViewport *viewport = NULL;
   PgmCanvas *canvas;
-  PgmDrawable *img;
   GstElement *pipeline, *image_sink;
   gulong formats;
 
@@ -147,6 +186,8 @@ main (int argc, char *argv[])
                     G_CALLBACK (button_release_cb), NULL);
   g_signal_connect (G_OBJECT (viewport), "motion-notify-event",
                     G_CALLBACK (motion_notify_cb), img);
+  g_signal_connect (G_OBJECT (viewport), "key-press-event",
+                    G_CALLBACK (keypress_event_cb), NULL);
   pgm_viewport_show (viewport);
   pgm_main ();
 
