@@ -139,7 +139,7 @@ main (int argc,
       char *argv[])
 {
   PgmViewport *viewport = NULL;
-  PgmDrawable *pattern, *text;
+  PgmDrawable *text;
   PgmCanvas *canvas;
 
   /* Init */
@@ -155,16 +155,6 @@ main (int argc,
   pgm_viewport_set_title (viewport, "Viewport projection");
   pgm_viewport_set_size (viewport, 640, 480);
 
-  /* Pattern creation */
-  pattern = pgm_image_new_from_file ("examples/pictures/pattern.png", 0);
-  pgm_image_set_layout (PGM_IMAGE (pattern), PGM_IMAGE_ZOOMED);
-  pgm_image_set_alignment (PGM_IMAGE (pattern), PGM_IMAGE_TOP_LEFT);
-  pgm_drawable_set_size (pattern, 640.0f, 480.0f);
-  pgm_drawable_set_position (pattern, 0.0f, 0.0f, 0.0f);
-  pgm_drawable_set_bg_color (pattern, 255, 255, 255, 0);
-  pgm_drawable_set_fg_color (pattern, 20, 20, 20, 255);
-  pgm_drawable_show (pattern);
-
   /* Text creation */
   text = pgm_text_new ("<span face='MgOpen Modata'>"
                        "<u><b>Rotation</b></u>\n"
@@ -178,7 +168,7 @@ main (int argc,
                        "'6': horizontal flip\n"
                        "'7': vertical flip\n"
                        "</span>");
-  pgm_text_set_font_height (PGM_TEXT (text), 0.085f);
+  pgm_text_set_font_height (PGM_TEXT (text), 25.0f);
   pgm_drawable_set_size (text, 600.0f, 400.0f);
   pgm_drawable_set_position (text, 10.0f, 10.0f, 0.0f);
   pgm_drawable_set_bg_color (text, 255, 255, 255, 0);
@@ -189,7 +179,7 @@ main (int argc,
   canvas = pgm_canvas_new ();
   pgm_canvas_set_size (canvas, 640.0f, 480.0f);
   pgm_viewport_set_canvas (viewport, canvas);
-  pgm_canvas_add_many (canvas, PGM_DRAWABLE_MIDDLE, pattern, text, NULL);
+  pgm_canvas_add (canvas, PGM_DRAWABLE_MIDDLE, text);
 
   /* Main loop */
   g_signal_connect (G_OBJECT (viewport), "delete-event",
