@@ -180,6 +180,57 @@ need_ordering_update (GList *list,
   return TRUE;
 }
 
+/* Update the position and the size if the grid aligned property is set */
+static void
+update_grid_aligned_drawable (PgmDrawable *drawable)
+{
+  gboolean aligned;
+  gfloat x, y, z, w, h;
+
+  GST_OBJECT_LOCK (drawable);
+  aligned = drawable->grid_aligned;
+  x = drawable->orig_x;
+  y = drawable->orig_y;
+  z = drawable->z;
+  w = drawable->orig_width;
+  h = drawable->orig_height;
+  GST_OBJECT_UNLOCK (drawable);
+
+  if (aligned)
+    {
+      pgm_drawable_set_position (drawable, x, y, z);
+      pgm_drawable_set_size (drawable, w, h);
+    }
+}
+
+/* Calls update_grid_aligned_drawable on a canvas layer */
+static void
+update_grid_aligned_drawables (PgmCanvas *canvas,
+                               GList *layer)
+{
+  GList *copy = NULL, *walk;
+
+  /* Copy the layer taking ref on drawables with the canvas lock */
+  GST_OBJECT_LOCK (canvas);
+  walk = layer;
+  while (walk)
+    {
+      copy = g_list_prepend (copy, g_object_ref (walk->data));
+      walk = walk->next;
+    }
+   GST_OBJECT_UNLOCK (canvas);
+
+   /* Call update_grid_aligned_drawables on each drawable */
+   walk = copy;
+   while (walk)
+     {
+       update_grid_aligned_drawable ((PgmDrawable*) walk->data);
+       g_object_unref (walk->data);
+       walk = walk->next;
+     }
+   g_list_free (copy);
+}
+
 /* GObject stuff */
 
 G_DEFINE_TYPE (PgmCanvas, pgm_canvas, GST_TYPE_OBJECT);
@@ -288,13 +339,7 @@ pgm_canvas_init (PgmCanvas *canvas)
 {
   guint i;
 
-  /* Public data */
-  canvas->far_layer = NULL;
-  canvas->middle_layer = NULL;
-  canvas->near_layer = NULL;
-  canvas->width = 4.0f;
-  canvas->height = 3.0f;
-  canvas->pixel_formats = 0;
+  /* canvas->entered_emission_stopper = NULL; */
 
   /* Create and insert all the format in the counter hash initialized at 0 */
   canvas->format_counter = g_hash_table_new (NULL, NULL);
@@ -302,9 +347,16 @@ pgm_canvas_init (PgmCanvas *canvas)
     g_hash_table_insert (canvas->format_counter,
                          GINT_TO_POINTER (1 << i),
                          GINT_TO_POINTER (0));
+  /* canvas->nb_formats = 0; */
 
-  /* No pixel formats are applied yet */
-  canvas->nb_formats = 0;
+  /* canvas->far_layer = NULL; */
+  /* canvas->middle_layer = NULL; */
+  /* canvas->near_layer = NULL; */
+  canvas->width = 4.0f;
+  canvas->height = 3.0f;
+  canvas->pixel_offset_x = 0.005f; /* 4/800 */
+  canvas->pixel_offset_y = 0.005f; /* 3/600 */
+  /* canvas->pixel_formats = 0; */
 }
 
 /* Public methods */
@@ -463,6 +515,8 @@ pgm_canvas_add (PgmCanvas *canvas,
 
   GST_OBJECT_UNLOCK (canvas);
 
+  update_grid_aligned_drawable (drawable);
+
   g_signal_emit (G_OBJECT (canvas), pgm_canvas_signals[DRAWABLE_ADDED], 0,
                  drawable, layer, order);
 
@@ -915,6 +969,34 @@ pgm_canvas_get_pixel_formats (PgmCanvas *canvas,
   return PGM_ERROR_OK;
 }
 
+/**
+ * pgm_canvas_get_pixel_offsets:
+ * @canvas: a #PgmCanvas object.
+ * @x: a pointer to a #gfloat where the x pixel offset is going to be stored.
+ * @y: a pointer to a #gfloat where the y pixel offset is going to be stored.
+ *
+ * Gets the x and y ratios corresponding to the offsets in canvas coordinates
+ * between two pixels at z=0.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_canvas_get_pixel_offsets (PgmCanvas *canvas,
+                              gfloat *x,
+                              gfloat *y)
+{
+  g_return_val_if_fail (PGM_IS_CANVAS (canvas), PGM_ERROR_X);
+  g_return_val_if_fail (x != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (y != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (canvas);
+  *x = canvas->pixel_offset_x;
+  *y = canvas->pixel_offset_y;
+  GST_OBJECT_UNLOCK (canvas);
+
+  return PGM_ERROR_OK;
+}
+
 /* Protected methods */
 
 /**
@@ -987,6 +1069,21 @@ _pgm_canvas_update_order (PgmCanvas *canvas,
   GST_OBJECT_UNLOCK (canvas);
 }
 
+/**
+ * _pgm_canvas_update_grid_aligned_drawables:
+ * @canvas: a #PgmCanvas object.
+ *
+ * This function is only used internally by pgm_viewport_update_projection() to
+ * realign drawables with the grid aligned property set.
+ */
+void
+_pgm_canvas_update_grid_aligned_drawables (PgmCanvas *canvas)
+{
+  update_grid_aligned_drawables (canvas, canvas->near_layer);
+  update_grid_aligned_drawables (canvas, canvas->middle_layer);
+  update_grid_aligned_drawables (canvas, canvas->far_layer);
+}
+
 /**
  * _pgm_canvas_add_pixel_formats:
  * @canvas: a #PgmCanvas object.
