@@ -23,15 +23,6 @@
  *            Julien Moutte <julien@fluendo.com>
  */
 
-/**
- * SECTION:pgmcommon
- * @short_description: Various enums and macros used for common tasks.
- *
- * Various enumerations and macros used by Pigment for common tasks.
- *
- * Last reviewed on 2007-04-12 (0.1.5)
- */
-
 #ifndef __PGM_COMMON_H__
 #define __PGM_COMMON_H__
 
@@ -40,8 +31,6 @@
 
 G_BEGIN_DECLS
 
-#define PGM_PADDING 4
-
 /**
  * PGM_DEGREES_TO_RADIANS:
  * @theta: an angle in degrees.
@@ -106,6 +95,25 @@ typedef enum {
   PGM_ERROR_OK
 } PgmError;
 
+/**
+ * pgm_lrintf:
+ * @x: the input floating-point number.
+ *
+ * Round @x to nearest integer using the current FPU rounding mode. On platforms
+ * other than Windows where the C99 lrintf function is not available, the
+ * function reverts to a float-to-int cast which uses the truncate FPU rounding
+ * mode.
+ *
+ * Returns: the rounded nearest integer.
+ */
+#if HAVE_LRINTF
+#define pgm_lrintf(x) lrintf (x)
+#elif defined (_MSC_VER)
+gint32 pgm_lrintf (gfloat x);
+#else
+#define pgm_lrintf(x) (gint32) (x)
+#endif
+
 G_END_DECLS
 
 #endif /* __PGM_COMMON_H__ */
