@@ -639,19 +639,24 @@ static void
 pgm_drawable_init (PgmDrawable *drawable)
 {
   /* Layering */
+  drawable->transformation_matrix = pgm_mat4x4_new_identity ();
+  /* drawable->angle_x = 0.0f; */
+  /* drawable->angle_y = 0.0f; */
+  /* drawable->angle_z = 0.0f; */
   drawable->layer = PGM_DRAWABLE_UNBOUND;
   /* Position */
-  drawable->x = 0.0f;
-  drawable->y = 0.0f;
-  drawable->z = 0.0f;
+  /* drawable->x = 0.0f; */
+  /* drawable->y = 0.0f; */
+  /* drawable->z = 0.0f; */
+  /* drawable->orig_x = 0.0f; */
+  /* drawable->orig_y = 0.0f; */
   /* Size */
   drawable->width = 1.0f;
   drawable->height = 1.0f;
+  drawable->orig_width = 1.0f;
+  drawable->orig_height = 1.0f;
+  drawable->grid_aligned = FALSE;
   /* Transformation */
-  drawable->transformation_matrix = pgm_mat4x4_new_identity ();
-  drawable->angle_x = 0.0f;
-  drawable->angle_y = 0.0f;
-  drawable->angle_z = 0.0f;
   drawable->scale = 1.0f;
   /* Colors */
   drawable->fg_r = 255;
@@ -665,19 +670,19 @@ pgm_drawable_init (PgmDrawable *drawable)
   /* Opacity */
   drawable->opacity = 255;
   /* Private data */
-  pgm_vec3_set_from_scalars (&drawable->press_point, 0.0f, 0.0f, 0.0f);
-  pgm_vec3_set_from_scalars (&drawable->viewport_press_point, 0.0f, 0.0f, 0.0f);
-  drawable->drag_mask = 0;
-  drawable->drag_distance = 5;
-  drawable->press_mask = 0;
-  drawable->pressure = 0;
-  drawable->release_time = 0;
-  drawable->release_button = 0;
-  drawable->mouse_over = FALSE;
-  drawable->flags = PGM_DRAWABLE_PLANE_VECTORS | PGM_DRAWABLE_ORIGIN;
-  pgm_vec3_set_from_scalars (&drawable->origin, 0.0f, 0.0f, 0.0f);
+  /* pgm_vec3_set_from_scalars (&drawable->origin, 0.0f, 0.0f, 0.0f); */
   pgm_vec3_set_from_scalars (&drawable->plane_vector_i, 1.0f, 0.0f, 0.0f);
   pgm_vec3_set_from_scalars (&drawable->plane_vector_j, 0.0f, 1.0f, 0.0f);
+  /* pgm_vec3_set_from_scalars (&drawable->press_point, 0.0f, 0.0f, 0.0f); */
+  /* pgm_vec3_set_from_scalars (&drawable->viewport_press_point, 0.0f, 0.0f, 0.0f); */
+  /* drawable->pressure = 0; */
+  /* drawable->release_time = 0; */
+  /* drawable->mouse_over = FALSE; */
+  /* drawable->drag_mask = 0; */
+  /* drawable->press_mask = 0; */
+  /* drawable->release_button = 0; */
+  drawable->drag_distance = 5;
+  drawable->flags = PGM_DRAWABLE_PLANE_VECTORS | PGM_DRAWABLE_ORIGIN;
 }
 
 /* Public methods */
@@ -791,30 +796,50 @@ pgm_drawable_set_size (PgmDrawable *drawable,
 {
   PgmError ret = PGM_ERROR_OK;
   PgmDrawableClass *klass;
+  PgmCanvas *canvas;
 
   g_return_val_if_fail (PGM_IS_DRAWABLE (drawable), PGM_ERROR_X);
 
   GST_OBJECT_LOCK (drawable);
 
-  drawable->width = MAX (0.0f, width);
-  drawable->height = MAX (0.0f, height);
+  drawable->orig_width = width = MAX (0.0f, width);
+  drawable->orig_height = height = MAX (0.0f, height);
 
-  /* Update the transformation matrix if the user added rotations */
-  if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
-    update_transformation_matrix (drawable);
+  if (drawable->grid_aligned &&
+      (canvas = PGM_CANVAS (GST_OBJECT_PARENT (drawable))))
+    {
+      width = floorf
+        (width / canvas->pixel_offset_x) * canvas->pixel_offset_x;
+      height = floorf
+        (height / canvas->pixel_offset_y) * canvas->pixel_offset_y;
+    }
+
+  if (width != drawable->width || height != drawable->height)
+    {
+      drawable->width = width;
+      drawable->height = height;
+
+      /* Update the transformation matrix if the user added transforms */
+      if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
+        update_transformation_matrix (drawable);
+
+      GST_OBJECT_UNLOCK (drawable);
+
+      klass = PGM_DRAWABLE_GET_CLASS (drawable);
+
+      if (klass->set_size)
+        ret = klass->set_size (drawable, width, height);
+
+      if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
+        _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_TRANSFORMATION_MATRIX);
+
+      _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_SIZE);
+
+      return ret;
+    }
 
   GST_OBJECT_UNLOCK (drawable);
 
-  klass = PGM_DRAWABLE_GET_CLASS (drawable);
-
-  if (klass->set_size)
-    ret = klass->set_size (drawable, width, height);
-
-  if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
-    _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_TRANSFORMATION_MATRIX);
-
-  _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_SIZE);
-
   return ret;
 }
 
@@ -843,8 +868,8 @@ pgm_drawable_get_size (PgmDrawable *drawable,
 
   GST_OBJECT_LOCK (drawable);
 
-  *width = drawable->width;
-  *height = drawable->height;
+  *width = drawable->orig_width;
+  *height = drawable->orig_height;
 
   GST_OBJECT_UNLOCK (drawable);
 
@@ -873,48 +898,56 @@ pgm_drawable_set_position (PgmDrawable *drawable,
   PgmError ret = PGM_ERROR_OK;
   gboolean reordered = FALSE;
   PgmDrawableClass *klass;
+  PgmCanvas *canvas;
 
   g_return_val_if_fail (PGM_IS_DRAWABLE (drawable), PGM_ERROR_X);
 
   GST_OBJECT_LOCK (drawable);
 
-  /* Update the position */
-  drawable->x = x;
-  drawable->y = y;
-  if (drawable->z != z)
+  drawable->flags &= ~PGM_DRAWABLE_ORIGIN;
+  drawable->orig_x = x;
+  drawable->orig_y = y;
+  canvas = PGM_CANVAS (GST_OBJECT_PARENT (drawable));
+
+  if (drawable->grid_aligned && canvas != NULL)
     {
-      drawable->z = z;
-      reordered = TRUE;
+      x = floorf (x / canvas->pixel_offset_x) * canvas->pixel_offset_x;
+      y = floorf (y / canvas->pixel_offset_y) * canvas->pixel_offset_y;
     }
 
-  drawable->flags &= ~PGM_DRAWABLE_ORIGIN;
+  reordered = (z != drawable->z);
 
-  /* Update the transformation matrix if the user added rotations */
-  if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
-    update_transformation_matrix (drawable);
+  if (x != drawable->x || y != drawable->y || reordered)
+    {
+      drawable->x = x;
+      drawable->y = y;
+      drawable->z = z;
+
+      /* Update the transformation matrix if the user added rotations */
+      if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
+        update_transformation_matrix (drawable);
+
+      GST_OBJECT_UNLOCK (drawable);
+
+      klass = PGM_DRAWABLE_GET_CLASS (drawable);
+
+      if (klass->set_position)
+        ret = klass->set_position (drawable, x, y, z);
+
+      /* Reorder the drawable inside the canvas in the case that z has changed */
+      if (reordered && canvas != NULL)
+        _pgm_canvas_update_order (canvas, drawable);
+
+      if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
+        _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_TRANSFORMATION_MATRIX);
+
+      _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_POSITION);
+
+      return ret;
+    }
 
   GST_OBJECT_UNLOCK (drawable);
 
-  klass = PGM_DRAWABLE_GET_CLASS (drawable);
-
-  if (klass->set_position)
-    ret = klass->set_position (drawable, x, y, z);
-
-  /* Reorder the drawable inside the canvas in the case that z has changed */
-  if (reordered)
-    {
-      GstObject *canvas;
-
-      canvas = GST_OBJECT_PARENT (drawable);
-      if (canvas)
-        _pgm_canvas_update_order (PGM_CANVAS (canvas), drawable);
-    }
-
-  if (drawable->flags & (PGM_DRAWABLE_ROTATED | PGM_DRAWABLE_SCALED))
-    _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_TRANSFORMATION_MATRIX);
-
-  _pgm_drawable_emit_changed (drawable, PGM_DRAWABLE_POSITION);
-
   return ret;
 }
 
@@ -947,8 +980,8 @@ pgm_drawable_get_position (PgmDrawable *drawable,
 
   GST_OBJECT_LOCK (drawable);
 
-  *x = drawable->x;
-  *y = drawable->y;
+  *x = drawable->orig_x;
+  *y = drawable->orig_y;
   *z = drawable->z;
 
   GST_OBJECT_UNLOCK (drawable);
@@ -1463,6 +1496,77 @@ pgm_drawable_get_opacity (PgmDrawable *drawable,
   return PGM_ERROR_OK;
 }
 
+/**
+ * pgm_drawable_set_grid_aligned:
+ * @drawable: a #PgmDrawable object.
+ * @aligned: the grid aligned boolean. %FALSE by default.
+ *
+ * Sets whether or not @drawable must be automatically aligned to the pixel
+ * boundaries in the viewport.
+ *
+ * MT safe.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_drawable_set_grid_aligned (PgmDrawable *drawable,
+                               gboolean aligned)
+{
+  gfloat x, y, z, w, h;
+
+  g_return_val_if_fail (PGM_IS_DRAWABLE (drawable), PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (drawable);
+
+  if (aligned == drawable->grid_aligned)
+    {
+      GST_OBJECT_UNLOCK (drawable);
+      return PGM_ERROR_OK;
+    }
+
+  drawable->grid_aligned = aligned;
+
+  x = drawable->orig_x;
+  y = drawable->orig_y;
+  z = drawable->z;
+  w = drawable->orig_width;
+  h = drawable->orig_height;
+
+  GST_OBJECT_UNLOCK (drawable);
+
+  pgm_drawable_set_position (drawable, x, y, z);
+  pgm_drawable_set_size (drawable, w, h);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_drawable_get_grid_aligned:
+ * @drawable: a #PgmDrawable object.
+ * @aligned: a pointer to a #gboolean where the grid aligned boolean is going to
+ * be stored.
+ *
+ * Gets whether or not @drawable must be automatically aligned to the pixel
+ * boundaries in the viewport.
+ *
+ * MT safe.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_drawable_get_grid_aligned (PgmDrawable *drawable,
+                               gboolean *aligned)
+{
+  g_return_val_if_fail (PGM_IS_DRAWABLE (drawable), PGM_ERROR_X);
+  g_return_val_if_fail (aligned != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (drawable);
+  *aligned = drawable->grid_aligned;
+  GST_OBJECT_UNLOCK (drawable);
+
+  return PGM_ERROR_OK;
+}
+
 /**
  * pgm_drawable_set_drag_distance:
  * @drawable: a #PgmDrawable object.
