@@ -84,9 +84,14 @@ static const GEnumValue _pgm_drawable_property_values[] = {
   { PGM_TEXT_STYLE, "PGM_TEXT_STYLE", "text-style" },
   { PGM_TEXT_VARIANT, "PGM_TEXT_VARIANT", "text-variant" },
   { PGM_TEXT_WEIGHT, "PGM_TEXT_WEIGHT", "text-weight" },
+  { PGM_TEXT_MULTILINE, "PGM_TEXT_MULTILINE", "text-multiline" },
   { PGM_TEXT_LINE_SPACING, "PGM_TEXT_LINE_SPACING", "text-line-spacing" },
+  { PGM_TEXT_COLOR, "PGM_TEXT_COLOR", "text-color" },
   { PGM_TEXT_OUTLINE_COLOR, "PGM_TEXT_OUTLINE_COLOR", "text-outline-color" },
   { PGM_TEXT_OUTLINE_WIDTH, "PGM_TEXT_OUTLINE_WIDTH", "text-outline-width" },
+  { PGM_TEXT_CAST_SHADOW, "PGM_TEXT_CAST_SHADOW", "text-cast-shadow" },
+  { PGM_TEXT_SHADOW_POSITION, "PGM_TEXT_SHADOW_POSITION", "text-shadow-position" },
+  { PGM_TEXT_SHADOW_COLOR, "PGM_TEXT_SHADOW_COLOR", "text-shadow-color" },
   { 0, NULL, NULL }
 };
 
@@ -531,6 +536,29 @@ pgm_text_weight_get_type (void)
   return type;
 }
 
+static const GFlagsValue _pgm_text_shadow_position_values[] = {
+  { PGM_TEXT_SHADOW_NORTH, "PGM_TEXT_SHADOW_NORTH", "north" },
+  { PGM_TEXT_SHADOW_SOUTH, "PGM_TEXT_SHADOW_SOUTH", "south" },
+  { PGM_TEXT_SHADOW_WEST, "PGM_TEXT_SHADOW_WEST", "west" },
+  { PGM_TEXT_SHADOW_EAST, "PGM_TEXT_SHADOW_EAST", "east" },
+  { PGM_TEXT_SHADOW_NORTH_WEST, "PGM_TEXT_SHADOW_NORTH_WEST", "north-west" },
+  { PGM_TEXT_SHADOW_NORTH_EAST, "PGM_TEXT_SHADOW_NORTH_EAST", "north-east" },
+  { PGM_TEXT_SHADOW_SOUTH_WEST, "PGM_TEXT_SHADOW_SOUTH_WEST", "south-west" },
+  { PGM_TEXT_SHADOW_SOUTH_EAST, "PGM_TEXT_SHADOW_SOUTH_EAST", "south-east" },
+  { 0, NULL, NULL }
+};
+
+GType
+pgm_text_shadow_position_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    type = g_flags_register_static ("PgmTextShadowPosition", _pgm_text_shadow_position_values);
+
+  return type;
+}
+
 
 /* enumerations from "./pgmviewport.h" */
 #include "./pgmviewport.h"
