@@ -39,6 +39,9 @@ typedef struct _PgmEventWin32Message PgmEventWin32Message;
 typedef union  _PgmEvent             PgmEvent;
 
 #include <glib-object.h>
+#ifdef WIN32
+#include <Windows.h>
+#endif /* WIN32 */
 
 G_BEGIN_DECLS
 
@@ -69,7 +72,7 @@ G_BEGIN_DECLS
  * @PGM_STATE: the state of a viewport has changed.
  * @PGM_DELETE: the window manager has requested that the toplevel window be
  * destroyed, usually when the user clicks on a special icon in the title bar.
- * @PGM_WIN32_MESSAGE: a Win32 message has been received.
+ * @PGM_WIN32_MESSAGE: a Win32 message has been received (Only used on Windows).
  *
  * Specifies the type of the event.
  */
@@ -318,6 +321,7 @@ struct _PgmEventState {
   PgmViewportState state_mask;
 };
 
+#ifdef WIN32
 /**
  * PgmEventWin32Message:
  * @type: the type of the event.
@@ -334,10 +338,11 @@ struct _PgmEventWin32Message {
   PgmEventType type;
   guint8       source;
   guint32      time;
-  guint        message;
-  guint        wparam;
-  glong        lparam;
+  UINT         message;
+  WPARAM       wparam;
+  LPARAM       lparam;
 };
+#endif /* WIN32 */
 
 /**
  * PgmEvent:
@@ -391,7 +396,9 @@ union _PgmEvent {
   PgmEventConfigure    configure;
   PgmEventDnd          dnd;
   PgmEventState        state;
+#ifdef WIN32
   PgmEventWin32Message win32_message;
+#endif /* WIN32 */
 };
 
 GType     pgm_event_get_type    (void);
