@@ -296,9 +296,11 @@ union _PgmImageData {
 
 /**
  * PgmImage:
+ * @master: the image #PgmImage master.
+ * @slaves: the image list of #PgmImage slaves.
+ * @mapping_matrix: the image mapping matrix.
  * @storage_type: the currently stored image type.
  * @data: the image data depending on the type.
- * @mapping_matrix: the image mapping matrix.
  * @layout: the image layout.
  * @align: the image alignment.
  * @interp: the image interpolation.
@@ -315,8 +317,6 @@ union _PgmImageData {
  * @border_outer_g: the image border outer green color.
  * @border_outer_b: the image border outer blue color.
  * @border_outer_a: the image border outer alpha color.
- * @master: the image #PgmImage master.
- * @slaves: the image list of #PgmImage slaves.
  *
  * The #PgmImage structure.
  */
@@ -325,12 +325,17 @@ struct _PgmImage {
   PgmDrawable parent;
 
   /*< public >*/ /* with LOCK */
+  /* Slavery handling */
+  PgmImage *master;
+  GList    *slaves;
+
+  PgmMat4x4 *mapping_matrix;
+
   /* Image storage */
   PgmImageStorageType storage_type;
   PgmImageData data;
 
   /* Image displaying properties */
-  PgmMat4x4          *mapping_matrix;
   PgmImageLayoutType  layout;
   PgmImageAlignment   align;
   PgmImageInterpType  interp;
@@ -349,19 +354,12 @@ struct _PgmImage {
   guchar border_outer_b;
   guchar border_outer_a;
 
-  /* Slavery handling */
-  PgmImage *master;
-  GList    *slaves;
-
   /*< private >*/
   /* Current loader id */
-  guint loader_id;
+  guint16 loader_id;
 
   /* Management bit mask */
   guint8 flags;
-
-  /* Padding */
-  gpointer _pgm_reserved[PGM_PADDING];
 };
 
 struct _PgmImageClass {
@@ -380,9 +378,6 @@ struct _PgmImageClass {
 
   void (*un_cloned)   (PgmImage *image,
                        guint n_clones);
-
-  /* Padding */
-  gpointer _pgm_reserved[PGM_PADDING];
 };
 
 /* Public functions */
