@@ -364,10 +364,13 @@ push_navigation_event (PgmImageSink *image_sink,
     {
     case PGM_BUTTON_LEFT:
       gst_button = 1;
+      break;
     case PGM_BUTTON_MIDDLE:
       gst_button = 2;
+      break;
     case PGM_BUTTON_RIGHT:
       gst_button = 3;
+      break;
     default:
       gst_button = 0;
     }
@@ -516,9 +519,10 @@ pgm_image_sink_show (GstBaseSink *bsink,
       pgm_image_set_from_gst_buffer (sink->image, sink->format, sink->width,
                                      sink->height, sink->stride, buf);
       /* And specify the pixel-aspect-ratio */
-      pgm_image_set_aspect_ratio (sink->image, (guint) ((gfloat) sink->width *
-                                  ((gfloat) sink->par_n / (gfloat) sink->par_d)),
-                                  (guint) sink->height);
+      pgm_image_set_aspect_ratio
+        (sink->image, pgm_lrintf
+         ((gfloat) sink->width * ((gfloat) sink->par_n / (gfloat) sink->par_d)),
+         sink->height);
     }
 
   GST_OBJECT_UNLOCK (sink);
@@ -608,6 +612,7 @@ pgm_image_sink_setcaps (GstBaseSink *bsink,
   ret = update_format_from_caps (sink, structure);
 
  beach:
+  gst_caps_unref (intersection);
   return ret;
 }
 
@@ -836,12 +841,6 @@ pgm_image_sink_class_init (PgmImageSinkClass *klass)
 static void
 pgm_image_sink_init (PgmImageSink *sink)
 {
-  sink->image = NULL;
-  sink->our_caps = NULL;
-  sink->motion_handler = 0;
-  sink->pressed_handler = 0;
-  sink->released_handler = 0;
-  sink->events = 0;
 }
 
 GType
