@@ -108,28 +108,28 @@ pgm_marshal_BOOLEAN__BOXED (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* POINTER:BOXED (./pgmmarshal.list:32) */
-extern void pgm_marshal_POINTER__BOXED (GClosure     *closure,
-                                        GValue       *return_value,
-                                        guint         n_param_values,
-                                        const GValue *param_values,
-                                        gpointer      invocation_hint,
-                                        gpointer      marshal_data);
+/* LONG:BOXED (./pgmmarshal.list:32) */
+extern void pgm_marshal_LONG__BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data);
 void
-pgm_marshal_POINTER__BOXED (GClosure     *closure,
-                            GValue       *return_value G_GNUC_UNUSED,
-                            guint         n_param_values,
-                            const GValue *param_values,
-                            gpointer      invocation_hint G_GNUC_UNUSED,
-                            gpointer      marshal_data)
+pgm_marshal_LONG__BOXED (GClosure     *closure,
+                         GValue       *return_value G_GNUC_UNUSED,
+                         guint         n_param_values,
+                         const GValue *param_values,
+                         gpointer      invocation_hint G_GNUC_UNUSED,
+                         gpointer      marshal_data)
 {
-  typedef gpointer (*GMarshalFunc_POINTER__BOXED) (gpointer     data1,
-                                                   gpointer     arg_1,
-                                                   gpointer     data2);
-  register GMarshalFunc_POINTER__BOXED callback;
+  typedef glong (*GMarshalFunc_LONG__BOXED) (gpointer     data1,
+                                             gpointer     arg_1,
+                                             gpointer     data2);
+  register GMarshalFunc_LONG__BOXED callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
-  gpointer v_return;
+  glong v_return;
 
   g_return_if_fail (return_value != NULL);
   g_return_if_fail (n_param_values == 2);
@@ -144,13 +144,13 @@ pgm_marshal_POINTER__BOXED (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_POINTER__BOXED) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_LONG__BOXED) (marshal_data ? marshal_data : cc->callback);
 
   v_return = callback (data1,
                        g_marshal_value_peek_boxed (param_values + 1),
                        data2);
 
-  g_value_set_pointer (return_value, v_return);
+  g_value_set_long (return_value, v_return);
 }
 
 /* VOID:OBJECT,UINT (./pgmmarshal.list:33) */
