@@ -26,13 +26,13 @@ extern void pgm_marshal_BOOLEAN__BOXED (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* POINTER:BOXED (./pgmmarshal.list:32) */
-extern void pgm_marshal_POINTER__BOXED (GClosure     *closure,
-                                        GValue       *return_value,
-                                        guint         n_param_values,
-                                        const GValue *param_values,
-                                        gpointer      invocation_hint,
-                                        gpointer      marshal_data);
+/* LONG:BOXED (./pgmmarshal.list:32) */
+extern void pgm_marshal_LONG__BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data);
 
 /* VOID:OBJECT,UINT (./pgmmarshal.list:33) */
 extern void pgm_marshal_VOID__OBJECT_UINT (GClosure     *closure,
