@@ -90,32 +90,39 @@ pgm_text_class_init (PgmTextClass *klass)
 static void
 pgm_text_init (PgmText* text)
 {
-  /* Labels */
+  /* Public data */
   text->label = g_strdup ("");
-
-  /* Font properties */
   text->font_family = g_strdup ("Sans");
   text->gravity = PGM_TEXT_GRAVITY_AUTO;
   text->stretch = PGM_TEXT_STRETCH_NORMAL;
-  text->style = PGM_TEXT_STYLE_NORMAL;
-  text->variant = PGM_TEXT_VARIANT_NORMAL;
+  /* text->style = PGM_TEXT_STYLE_NORMAL; */
+  /* text->variant = PGM_TEXT_VARIANT_NORMAL; */
   text->weight = PGM_TEXT_WEIGHT_NORMAL;
   text->height = 1.0f;
-
-  /* Text adjustment */
-  text->ellipsize = PGM_TEXT_ELLIPSIZE_NONE;
-  text->alignment = PGM_TEXT_ALIGN_LEFT;
-  text->wrap = PGM_TEXT_WRAP_WORD;
-  text->line_spacing = 0.0f;
-  text->justify = FALSE;
-
-  /* Outline parameters */
-  text->outline_r = 0;
-  text->outline_g = 0;
-  text->outline_b = 0;
-  text->outline_a = 0;
+  /* text->ellipsize = PGM_TEXT_ELLIPSIZE_NONE; */
+  /* text->alignment = PGM_TEXT_ALIGN_LEFT; */
+  /* text->wrap = PGM_TEXT_WRAP_WORD; */
+  /* text->line_spacing = 0.0f; */
+  /* text->justify = FALSE; */
+  text->multiline = TRUE;
+  /* text->cast_shadow = FALSE; */
+  text->shadow_position = PGM_TEXT_SHADOW_NORTH;
   text->outline_width = 0.0f;
-  text->use_markup = FALSE;
+  text->r = 255;
+  text->g = 255;
+  text->b = 255;
+  text->a = 255;
+  /* text->shadow_r = 0; */
+  /* text->shadow_g = 0; */
+  /* text->shadow_b = 0; */
+  text->shadow_a = 127;
+  /* text->outline_r = 0; */
+  /* text->outline_g = 0; */
+  /* text->outline_b = 0; */
+  /* text->outline_a = 0; */
+
+  /* Private data */
+  /* text->use_markup = FALSE; */
 }
 
 /* Public methods */
@@ -133,14 +140,15 @@ pgm_text_init (PgmText* text)
 PgmDrawable *
 pgm_text_new (const gchar *markup)
 {
-  PgmText *text;
+  PgmDrawable *drawable;
 
-  text = g_object_new (PGM_TYPE_TEXT, NULL);
-  GST_DEBUG_OBJECT (text, "created new text");
+  drawable = g_object_new (PGM_TYPE_TEXT, NULL);
+  GST_DEBUG_OBJECT (drawable, "created new text");
 
-  pgm_text_set_markup (text, markup);
+  pgm_text_set_markup (PGM_TEXT (drawable), markup);
+  pgm_drawable_set_grid_aligned (drawable, TRUE);
 
-  return PGM_DRAWABLE (text);
+  return drawable;
 }
 
 /**
@@ -305,13 +313,10 @@ pgm_text_get_font_family (PgmText *text,
 /**
  * pgm_text_set_font_height:
  * @text: a #PgmText object.
- * @font_height: the text height in canvas-coordinates.
+ * @font_height: the text height in canvas coordinates.
  *
- * Sets the height of @text to @font_height in proportion to the height of the
- * drawable. The default height is 1.0f, which fills the drawable in height.
- * @font_height will be the maximum height of a line of text (i.e. the height of
- * something like "Čq", measured from the bottom of the 'q' to the top of the
- * accent on the 'Č').
+ * Sets the height of @text to @font_height in canvas coordinates. The default
+ * height is 1.0f.
  *
  * MT safe.
  *
@@ -931,6 +936,149 @@ pgm_text_get_line_spacing (PgmText *text,
   return PGM_ERROR_OK;
 }
 
+/**
+ * pgm_text_set_multiline:
+ * @text: a #PgmText object.
+ * @multiline: the multilining state.
+ *
+ * Sets the multilining state of @text to @multiline. When multilining is
+ * disabled, the text is displayed on one line and vertically centered in the
+ * drawable. The default value is #TRUE.
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_set_multiline (PgmText *text,
+                        gboolean multiline)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  if (multiline == text->multiline)
+    {
+      GST_OBJECT_UNLOCK (text);
+      return PGM_ERROR_OK;
+    }
+  text->multiline = multiline;
+
+  GST_OBJECT_UNLOCK (text);
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (text), PGM_TEXT_MULTILINE);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_get_multiline:
+ * @text: a #PgmText object.
+ * @multiline: a pointer to a #gboolean where the multilining state is going to
+ * be stored.
+ *
+ * Retrieves the multilining state of @text in @multiline.
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_get_multiline (PgmText *text,
+                        gboolean *multiline)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+  g_return_val_if_fail (multiline != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  *multiline = text->multiline;
+
+  GST_OBJECT_UNLOCK (text);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_set_color:
+ * @text: a #PgmText object.
+ * @red: the text red color component.
+ * @green: the text green color component.
+ * @blue: the text blue color component.
+ * @alpha: the text alpha color component.
+ *
+ * Sets the color of @text to (@red, @green, @blue, @alpha).
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_set_color (PgmText *text,
+                    guchar red,
+                    guchar green,
+                    guchar blue,
+                    guchar alpha)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  text->r = red;
+  text->g = green;
+  text->b = blue;
+  text->a = alpha;
+
+  GST_OBJECT_UNLOCK (text);
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (text), PGM_TEXT_COLOR);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_get_color:
+ * @text: a #PgmText object.
+ * @red: a pointer to a #guchar where the red color component is going to be
+ * stored.
+ * @green: a pointer to a #guchar where the green color component is going to be
+ * stored.
+ * @blue: a pointer to a #guchar where the blue color component is going to be
+ * stored.
+ * @alpha: a pointer to a #guchar where the alpha color component is going to be
+ * stored.
+ *
+ * Retrieves the color of @text in (@red, @green, @blue, @alpha).
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_get_color (PgmText *text,
+                    guchar *red,
+                    guchar *green,
+                    guchar *blue,
+                    guchar *alpha)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+  g_return_val_if_fail (red != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (green != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (blue != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (alpha != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  *red = text->r;
+  *green = text->g;
+  *blue = text->b;
+  *alpha = text->a;
+
+  GST_OBJECT_UNLOCK (text);
+
+  return PGM_ERROR_OK;
+}
+
 /**
  * pgm_text_set_outline_color:
  * @text: a #PgmText object.
@@ -1067,3 +1215,203 @@ pgm_text_get_outline_width (PgmText *text,
 
   return PGM_ERROR_OK;
 }
+
+/**
+ * pgm_text_set_cast_shadow:
+ * @text: a #PgmText object.
+ * @cast_shadow: the shadow casting state.
+ *
+ * Sets the shadow casting state of @text to @cast_shadow. When shadow casting
+ * is enabled, the text is displayed with a shadow behind it. The default value
+ * is #FALSE.
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_set_cast_shadow (PgmText *text,
+                          gboolean cast_shadow)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  if (cast_shadow == text->cast_shadow)
+    {
+      GST_OBJECT_UNLOCK (text);
+      return PGM_ERROR_OK;
+    }
+  text->cast_shadow = cast_shadow;
+
+  GST_OBJECT_UNLOCK (text);
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (text), PGM_TEXT_CAST_SHADOW);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_get_cast_shadow:
+ * @text: a #PgmText object.
+ * @cast_shadow: a pointer to a #gboolean where the shadow casting state is
+ * going to be stored.
+ *
+ * Retrieves the shadow casting state of @text in @cast_shadow.
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_get_cast_shadow (PgmText *text,
+                          gboolean *cast_shadow)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+  g_return_val_if_fail (cast_shadow != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  *cast_shadow = text->cast_shadow;
+
+  GST_OBJECT_UNLOCK (text);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_set_shadow_position:
+ * @text: a #PgmText object.
+ * @position: the text shadow position.
+ *
+ * Sets the shadow position of @text to @position. The default value is
+ * #PGM_TEXT_SHADOW_TOP.
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_set_shadow_position (PgmText *text,
+                              PgmTextShadowPosition position)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  text->shadow_position = position;
+
+  GST_OBJECT_UNLOCK (text);
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (text), PGM_TEXT_SHADOW_POSITION);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_get_shadow_position:
+ * @text: a #PgmText object.
+ * @position: a pointer to a #PgmTextShadowPosition where the text shadow
+ * position is going to be stored.
+ *
+ * Retrieves the shadow position of @text in @position.
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_get_shadow_position (PgmText *text,
+                              PgmTextShadowPosition *position)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+  g_return_val_if_fail (position != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  *position = text->shadow_position;
+
+  GST_OBJECT_UNLOCK (text);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_set_shadow_color:
+ * @text: a #PgmText object.
+ * @red: the text shadow red color component.
+ * @green: the text shadow green color component.
+ * @blue: the text shadow blue color component.
+ * @alpha: the text shadow alpha color component.
+ *
+ * Sets the shadow color of @text to (@red, @green, @blue, @alpha).
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_set_shadow_color (PgmText *text,
+                           guchar red,
+                           guchar green,
+                           guchar blue,
+                           guchar alpha)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  text->shadow_r = red;
+  text->shadow_g = green;
+  text->shadow_b = blue;
+  text->shadow_a = alpha;
+
+  GST_OBJECT_UNLOCK (text);
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (text), PGM_TEXT_SHADOW_COLOR);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_text_get_shadow_color:
+ * @text: a #PgmText object.
+ * @red: a pointer to a #guchar where the shadow red color component is going
+ * to be stored.
+ * @green: a pointer to a #guchar where the shadow green color component is
+ * going to be stored.
+ * @blue: a pointer to a #guchar where the shadow blue color component is going
+ * to be stored.
+ * @alpha: a pointer to a #guchar where the shadow alpha color component is
+ * going to be stored.
+ *
+ * Retrieves the shadow color of @text in (@red, @green, @blue, @alpha).
+ *
+ * MT safe.
+ *
+ * Returns: A #PgmError indicating success/failure.
+ */
+PgmError
+pgm_text_get_shadow_color (PgmText *text,
+                           guchar *red,
+                           guchar *green,
+                           guchar *blue,
+                           guchar *alpha)
+{
+  g_return_val_if_fail (PGM_IS_TEXT (text), PGM_ERROR_X);
+  g_return_val_if_fail (red != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (green != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (blue != NULL, PGM_ERROR_X);
+  g_return_val_if_fail (alpha != NULL, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (text);
+
+  *red = text->shadow_r;
+  *green = text->shadow_g;
+  *blue = text->shadow_b;
+  *alpha = text->shadow_a;
+
+  GST_OBJECT_UNLOCK (text);
+
+  return PGM_ERROR_OK;
+}
