@@ -126,9 +126,9 @@ typedef enum {
  *        GObject fields (see #183) */
 typedef struct __PgmIOWatch _PgmIOWatch;
 struct __PgmIOWatch {
-  gint        fd[2];
   GIOChannel *out;
   GIOChannel *in;
+  gint        fd[2];
   gint        tag;
 };
 
@@ -136,7 +136,9 @@ struct __PgmIOWatch {
  * PgmViewport:
  * @factory: the factory used to instantiate the #PgmViewport.
  * @canvas: the attached canvas.
+ * @message_filter: the filter of message events.
  * @title: the viewport title.
+ * @icon: the current icon.
  * @cursor: the current cursor.
  * @fullscreen: the fullscreen state.
  * @visible: the visible state.
@@ -144,7 +146,6 @@ struct __PgmIOWatch {
  * @iconified: the iconified state.
  * @opacity: the opacity.
  * @alpha_blending: the alpha blending state.
- * @icon: the current icon.
  * @width: the viewport width.
  * @height: the viewport height.
  * @width_mm: the viewport screen width in millimeters.
@@ -162,7 +163,6 @@ struct __PgmIOWatch {
  * @canvas_ratio: the attached canvas ratio with pixel-aspect-ratio correction
  * applied.
  * @caps_mask: the viewport mask of supported capacities.
- * @message_filter: the filter of message events.
  *
  * The #PgmViewport structure.
  */
@@ -173,14 +173,15 @@ struct _PgmViewport {
   PgmViewportFactory *factory;
   PgmCanvas          *canvas;
 
+  GList *message_filter;
+
   gchar             *title;
-  PgmViewportCursor  cursor;
   GdkPixbuf         *icon;
+  PgmViewportCursor  cursor;
   gboolean           fullscreen;
   gboolean           visible;
   gboolean           decorated;
   gboolean           iconified;
-  guchar             opacity;
   gboolean           alpha_blending;
 
   gint width, height;
@@ -199,20 +200,18 @@ struct _PgmViewport {
 
   gulong caps_mask;
 
-  GList *message_filter;
+  guchar opacity;
 
   /*< private >*/
-  /* Event handling */
+  /* Watch sources */
   _PgmIOWatch *event_watch;
-  gboolean     event_watch_up;
   GMutex      *event_lock;
   GList       *event_list;
-
-  /* Pixel handling */
   _PgmIOWatch *pixel_watch;
-  gboolean     pixel_watch_up;
   GMutex      *pixel_lock;
   GList       *pixel_list;
+  gboolean     pixel_watch_up;
+  gboolean     event_watch_up;
 
   /* Canvas rotation/reflection */
   gfloat projection_hflip, projection_vflip;
@@ -220,9 +219,6 @@ struct _PgmViewport {
 
   /* Signal handler id */
   gulong canvas_size_handler;
-
-  /* Padding */
-  gpointer _pgm_reserved[PGM_PADDING];
 };
 
 /**
@@ -267,6 +263,7 @@ struct _PgmViewport {
  * @get_embedding_id: the get_embedding_id virtual method.
  * @get_pixel_formats: the get_pixel_formats virtual method.
  * @get_caps_mask: the get_caps_mask virtual method.
+ * @get_max_texture_size: the get_max_texture_size virtual method.
  * @get_frame_rate: the get_frame_rate virtual method.
  * @set_message_filter: the set_message_filter virtual method.
  * @read_pixels: the read_pixels virtual method.
@@ -401,6 +398,9 @@ struct _PgmViewportClass {
   PgmError (*get_caps_mask)         (PgmViewport *viewport,
                                      gulong *caps_mask);
 
+  PgmError (*get_max_texture_size)  (PgmViewport *viewport,
+                                     guint32 *max_texture_size);
+
   PgmError (*get_frame_rate)        (PgmViewport *viewport,
                                      guint *frame_rate);
 
@@ -416,10 +416,6 @@ struct _PgmViewportClass {
 
   PgmError (*set_drag_status)       (PgmViewport *viewport,
                                      gboolean accept);
-
-  /*< private >*/
-  /* Padding */
-  gpointer _pgm_reserved[PGM_PADDING];
 };
 
 GType        pgm_viewport_get_type              (void);
@@ -542,6 +538,9 @@ PgmError     pgm_viewport_get_pixel_formats     (PgmViewport *viewport,
 PgmError     pgm_viewport_get_caps_mask         (PgmViewport *viewport,
                                                  gulong *caps_mask);
 
+PgmError     pgm_viewport_get_max_texture_size  (PgmViewport *viewport,
+                                                 guint32 *max_texture_size);
+
 PgmError     pgm_viewport_get_frame_rate        (PgmViewport *viewport,
                                                  guint *frame_rate);
 
