@@ -926,7 +926,7 @@ pgm_agl_backend_has_alpha_component (PgmBackend *backend)
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmAglBackend, pgm_agl_backend, PGM_TYPE_BACKEND);
+PGM_DEFINE_DYNAMIC_TYPE (PgmAglBackend, pgm_agl_backend, PGM_TYPE_BACKEND)
 
 void pgm_agl_backend_register (GTypeModule *module)
 {
