@@ -92,6 +92,13 @@ struct _PgmBackendClass {
   gboolean (*set_visibility)          (PgmBackend *backend,
                                        gboolean visible);
 
+  /* Iconify/Deiconify (minimize/restore) the window */
+  gboolean (*set_iconified)           (PgmBackend *backend,
+                                       gboolean iconified);
+
+  /* Show, raise on top and give the keyboard focus to window */
+  void     (*focus)                   (PgmBackend *backend);
+
   /* Retrieve the size of the physical screen in millimeters */
   void     (*get_screen_size_mm)      (PgmBackend *backend,
                                        gint *width,
@@ -208,6 +215,11 @@ gboolean    pgm_backend_set_fullscreen          (PgmBackend *backend,
 gboolean    pgm_backend_set_visibility          (PgmBackend *backend,
                                                  gboolean visible);
 
+gboolean    pgm_backend_set_iconified           (PgmBackend *backend,
+                                                 gboolean iconified);
+
+void        pgm_backend_focus                   (PgmBackend *backend);
+
 void        pgm_backend_get_screen_size_mm      (PgmBackend *backend,
                                                  gint *width,
                                                  gint *height);
