@@ -287,6 +287,8 @@ init_opengl_states (PgmContext *context)
   /* Set up standard blending with a different alpha function to correctly
    * support compositing with the desktop when the alpha clearing color is
    * available (ARGB visual) */
+  /* XXX A premuliplied alpha blending should be used instead, but it would
+   *     require API and implementation changes in a lot files */
   if (context->feature_mask & PGM_GL_FEAT_BLEND_FUNC_SEPARATE)
     gl->blend_func_separate (PGM_GL_SRC_ALPHA, PGM_GL_ONE_MINUS_SRC_ALPHA,
                              PGM_GL_SRC_ALPHA, PGM_GL_ONE);
@@ -842,36 +844,6 @@ do_read_pixels (PgmContext *context,
   rectangle = NULL;
 }
 
-/* Quit the rendering thread */
-static void
-do_quit (PgmContext *context,
-         gpointer data)
-{
-  GSource *source;
-
-  /* Remove sources. Do not use g_source_remove() since this only works for
-   * sources attached to the default GMainContext. See bug #261. */
-  if (context->immediate_tag)
-    {
-      source = g_main_context_find_source_by_id (context->render_context,
-                                                 context->immediate_tag);
-      if (source)
-        g_source_destroy (source);
-    }
-  if (context->update_tag)
-    {
-      source = g_main_context_find_source_by_id (context->render_context,
-                                                 context->update_tag);
-      if (source)
-        g_source_destroy (source);
-    }
-
-  /* Join the rendering thread */
-  g_main_loop_quit (context->render_loop);
-  g_main_loop_unref (context->render_loop);
-  g_main_context_unref (context->render_context);
-}
-
 /* Texture generation task handler */
 static void
 do_gen_texture (PgmContext *context,
@@ -951,20 +923,72 @@ do_intel945gm_black_magic (PgmContext *context)
 }
 #endif /* HAVE_WGL_H */
 
-/* Traverse the layers and draw the drawables */
+/* Copy a layer */
+static GList*
+copy_layer (GList *layer)
+{
+  GList *new_layer = NULL;
+
+  if (layer)
+    {
+      GList *last;
+
+      new_layer = g_slice_new (GList);
+      new_layer->data = g_object_ref (layer->data);
+      new_layer->prev = NULL;
+      last = new_layer;
+      layer = layer->next;
+
+      while (layer)
+        {
+          last->next = g_slice_new (GList);
+          last->next->prev = last;
+          last = last->next;
+          last->data = g_object_ref (layer->data);
+          layer = layer->next;
+        }
+
+      last->next = NULL;
+    }
+
+  return new_layer;
+}
+
+/* Render a layer */
+static void
+render_layer (GList *layer)
+{
+  GList *walk = layer;
+
+  while (walk)
+    {
+      PgmGlDrawable *gldrawable = (PgmGlDrawable*) walk->data;
+      pgm_gl_drawable_draw (gldrawable);
+      g_object_unref (gldrawable);
+      walk = walk->next;
+    }
+
+  g_list_free (layer);
+}
+
+/* Traverse the layers and draw each drawable */
 static void
 render (PgmContext *context)
 {
+  GList *near_layer, *middle_layer, *far_layer;
   PgmGlViewport *glviewport = context->glviewport;
 
+  /* Get copies of the layers */
   g_mutex_lock (glviewport->layer_lock);
-
-  g_list_foreach (glviewport->far_layer, (GFunc) pgm_gl_drawable_draw, NULL);
-  g_list_foreach (glviewport->middle_layer,
-                  (GFunc) pgm_gl_drawable_draw, NULL);
-  g_list_foreach (glviewport->near_layer, (GFunc) pgm_gl_drawable_draw, NULL);
-
+  near_layer = copy_layer (glviewport->near_layer);
+  middle_layer = copy_layer (glviewport->middle_layer);
+  far_layer = copy_layer (glviewport->far_layer);
   g_mutex_unlock (glviewport->layer_lock);
+
+  /* Then render each drawable of each layer */
+  render_layer (far_layer);
+  render_layer (middle_layer);
+  render_layer (near_layer);
 }
 
 /* Push a task in a queue removing a similar one if any */
@@ -1115,32 +1139,38 @@ update_cb (gpointer data)
   /* User defined update callback */
   pgm_viewport_emit_update_pass (PGM_VIEWPORT (context->glviewport));
 
-  /* Flush update queue */
-  pgm_gl_viewport_flush_update_queue (context->glviewport);
+  /* Check if the viewport has not been disposed in the update pass signal */
+  if (G_LIKELY (context->glviewport))
+    {
+      /* Flush update queue */
+      pgm_gl_viewport_flush_update_queue (context->glviewport);
 
-  /* Flush rendering queues */
-  flush_task_queue (context, &context->immediate_task);
-  flush_task_queue (context, &context->deferred_task);
+      /* Flush rendering queues */
+      flush_task_queue (context, &context->immediate_task);
+      flush_task_queue (context, &context->deferred_task);
 
-  /* Render the scene */
-  render (context);
+      /* Render the scene */
+      render (context);
 
 #ifdef HAVE_WGL_H
-  /* The driver for Intel 945GM on Windows needs black magic. See the
-   * do_intel945gm_black_magic function documentation. */
-  if (context->intel945gm_chipset)
-    do_intel945gm_black_magic (context);
+      /* The driver for Intel 945GM on Windows needs black magic. See the
+       * do_intel945gm_black_magic function documentation. */
+      if (context->intel945gm_chipset)
+        do_intel945gm_black_magic (context);
 #endif /* HAVE_WGL_H */
 
-  /* Wait for VSync and swap buffers */
-  pgm_backend_wait_for_vblank (context->backend);
-  pgm_backend_swap_buffers (context->backend);
-  context->gl->clear (PGM_GL_COLOR_BUFFER_BIT);
+      /* Wait for VSync and swap buffers */
+      pgm_backend_wait_for_vblank (context->backend);
+      pgm_backend_swap_buffers (context->backend);
+      context->gl->clear (PGM_GL_COLOR_BUFFER_BIT);
 
-  /* Handle frame rate */
-  compute_frame_rate (context);
+      /* Handle frame rate */
+      compute_frame_rate (context);
 
-  return TRUE;
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
 /* Update timeout used to remove the update idle after 1 second of inactivity */
@@ -1196,14 +1226,22 @@ create_io_channels (gint *fd,
       return;
     }
 
+#ifdef WIN32
+  *in = g_io_channel_win32_new_fd (fd[1]);
+#else
   *in = g_io_channel_unix_new (fd[1]);
+#endif /* WIN32 */
   if (!*in)
     {
       GST_ERROR ("cannot create the input channel");
       return;
     }
 
+#ifdef WIN32
+  *out = g_io_channel_win32_new_fd (fd[0]);
+#else
   *out = g_io_channel_unix_new (fd[0]);
+#endif /* WIN32 */
   if (!*out)
     {
       GST_ERROR ("cannot create the output channel");
@@ -1271,6 +1309,7 @@ render_loop (gpointer data)
   PgmContext *context = (PgmContext *) data;
   PgmContextProcAddress *gl = NULL;
   const gchar *env_var = NULL;
+  GSource *source;
   gint i = 0;
 
   /* Instantiate correct backend depending on the system at compile-time */
@@ -1417,12 +1456,22 @@ render_loop (gpointer data)
   g_cond_signal (context->init_cond);
   g_mutex_unlock (context->init_mutex);
 
-  /* And enter the main loop */
+  /* Enter the main loop */
   g_main_loop_run (context->render_loop);
 
-  /* Flush the queues */
-  flush_task_queue (context, &context->immediate_task);
-  flush_task_queue (context, &context->deferred_task);
+  /* Dispose immediate source */
+  source = g_main_context_find_source_by_id (context->render_context,
+                                             context->immediate_tag);
+  if (source)
+    g_source_destroy (source);
+  delete_io_channels (context->immediate_fd, &context->immediate_in,
+                      &context->immediate_out);
+  context->immediate_in = NULL;
+  context->immediate_out = NULL;
+
+  /* Free the task queues */
+  free_task_queue (&context->deferred_task);
+  free_task_queue (&context->immediate_task);
 
   /* Delete the ARB fragment programs if supported */
   if (context->feature_mask & PGM_GL_FEAT_FRAGMENT_PROGRAM)
@@ -1431,7 +1480,16 @@ render_loop (gpointer data)
   /* Clean up rendering thread related data */
   pgm_backend_destroy_window (context->backend);
   gst_object_unref (context->backend);
-  context->backend = NULL;
+
+  /* Signal application thread that the rendering thread is now finalized,
+   * no need to signal if the context is disposed in the rendering thread */
+  if (context->fini_signal)
+    {
+      g_mutex_lock (context->fini_mutex);
+      context->finalized = TRUE;
+      g_cond_signal (context->fini_cond);
+      g_mutex_unlock (context->fini_mutex);
+    }
 
   return NULL;
 
@@ -1468,6 +1526,12 @@ init_context (PgmContext *context)
   context->init_cond = g_cond_new ();
   context->initialized = FALSE;
 
+  /* Finalization lock */
+  context->fini_mutex = g_mutex_new ();
+  context->fini_cond = g_cond_new ();
+  context->fini_signal = FALSE;
+  context->finalized = FALSE;
+
   /* Immediate task source */
   context->immediate_fd[0] = -1;
   context->immediate_fd[1] = -1;
@@ -1499,7 +1563,8 @@ init_context (PgmContext *context)
 #endif /* HAVE_GL_WGL_H */
 
   /* Create rendering thread */
-  context->render_thread = g_thread_create (render_loop, context, TRUE, &error);
+  context->render_thread =
+    g_thread_create (render_loop, context, FALSE, &error);
   if (error != NULL)
     {
       GST_ERROR ("couldn't create rendering thread: %s", error->message);
@@ -1507,10 +1572,12 @@ init_context (PgmContext *context)
     }
 
   /* Various tasks function binding */
-  context->task_func[PGM_CONTEXT_PROJECTION] = GST_DEBUG_FUNCPTR (do_projection);
+  context->task_func[PGM_CONTEXT_PROJECTION] =
+    GST_DEBUG_FUNCPTR (do_projection);
   context->task_func[PGM_CONTEXT_SIZE] = GST_DEBUG_FUNCPTR (do_size);
   context->task_func[PGM_CONTEXT_TITLE] = GST_DEBUG_FUNCPTR (do_title);
-  context->task_func[PGM_CONTEXT_DECORATION] = GST_DEBUG_FUNCPTR (do_decoration);
+  context->task_func[PGM_CONTEXT_DECORATION] =
+    GST_DEBUG_FUNCPTR (do_decoration);
   context->task_func[PGM_CONTEXT_FULLSCREEN] =
     GST_DEBUG_FUNCPTR (do_fullscreen);
   context->task_func[PGM_CONTEXT_VISIBILITY] =
@@ -1543,7 +1610,6 @@ init_context (PgmContext *context)
     GST_DEBUG_FUNCPTR (do_update_texture);
   context->task_func[PGM_CONTEXT_FREE_TEXTURE] =
     GST_DEBUG_FUNCPTR (do_free_texture);
-  context->task_func[PGM_CONTEXT_QUIT] = GST_DEBUG_FUNCPTR (do_quit);
 
   /* Wait for the rendering thread initialization completion */
   g_mutex_lock (context->init_mutex);
@@ -1554,33 +1620,6 @@ init_context (PgmContext *context)
   return TRUE;
 }
 
-/* Dispose context */
-static void
-dispose_context (PgmContext *context)
-{
-  /* Join the rendering thread */
-  g_thread_join (context->render_thread);
-
-  /* Dispose immediate source data */
-  delete_io_channels (context->immediate_fd, &context->immediate_in,
-                      &context->immediate_out);
-
-  /* Free the task queues */
-  free_task_queue (&context->deferred_task);
-  free_task_queue (&context->immediate_task);
-
-  /* Auto-update lock */
-  g_mutex_free (context->update_mutex);
-
-  /* Initialization lock */
-  g_mutex_free (context->init_mutex);
-  g_cond_free (context->init_cond);
-  context->initialized = FALSE;
-
-  /* Structure access */
-  g_mutex_free (context->mutex);
-}
-
 /* Public functions */
 
 PgmContext *
@@ -1608,9 +1647,31 @@ pgm_context_free (PgmContext *context)
 {
   g_return_if_fail (context != NULL);
 
-  dispose_context (context);
-  context = NULL;
+  /* Store whether the context disposition happens in the rendering thread */
+  context->fini_signal = g_thread_self () != context->render_thread;
 
+  /* Quit the rendering loop */
+  g_main_loop_quit (context->render_loop);
+
+  /* Wait for the rendering thread finalization completion if the context
+   * disposition happens in the rendering thread */
+  if (context->fini_signal)
+    {
+      g_mutex_lock (context->fini_mutex);
+      if (!context->finalized)
+        g_cond_wait (context->fini_cond, context->fini_mutex);
+      g_mutex_unlock (context->fini_mutex);
+    }
+
+  /* Free the context */
+  g_main_loop_unref (context->render_loop);
+  g_main_context_unref (context->render_context);
+  g_mutex_free (context->fini_mutex);
+  g_cond_free (context->fini_cond);
+  g_mutex_free (context->update_mutex);
+  g_mutex_free (context->init_mutex);
+  g_cond_free (context->init_cond);
+  g_mutex_free (context->mutex);
   g_slice_free (PgmContext, context);
 }
 
