@@ -68,7 +68,6 @@ typedef enum {
   PGM_CONTEXT_MESSAGE_FILTER, /* Update message filter */
   PGM_CONTEXT_DRAG_STATUS,    /* Update drag status */
   PGM_CONTEXT_READ_PIXELS,    /* Read frame buffer */
-  PGM_CONTEXT_QUIT,           /* Clean up and quit the rendering thread */
   /* Texture related */
   PGM_CONTEXT_GEN_TEXTURE,    /* Generate texture */
   PGM_CONTEXT_CLEAN_TEXTURE,  /* Clean up texture */
@@ -114,6 +113,12 @@ struct _PgmContext {
   GCond    *init_cond;
   gboolean  initialized;
 
+  /* Finalization lock */
+  GMutex   *fini_mutex;
+  GCond    *fini_cond;
+  gboolean  fini_signal;
+  gboolean  finalized;
+
   /* Physical screen informations */
   gint screen_width_mm;
   gint screen_height_mm;
