@@ -115,22 +115,23 @@ G_BEGIN_DECLS
 #define PGM_GL_TEXTURE_BINDING_2D        0x8069
 #define PGM_GL_TEXTURE_BINDING_RECTANGLE 0x84F6
 
-#define PGM_GL_TEXTURE_ENV        0x2300
-#define PGM_GL_TEXTURE_ENV_MODE   0x2200
-#define PGM_GL_TEXTURE_2D         0x0DE1
-#define PGM_GL_PROXY_TEXTURE_2D   0x8064
-#define PGM_GL_TEXTURE_WRAP_S     0x2802
-#define PGM_GL_TEXTURE_WRAP_T     0x2803
-#define PGM_GL_TEXTURE_MAG_FILTER 0x2800
-#define PGM_GL_TEXTURE_MIN_FILTER 0x2801
-#define PGM_GL_TEXTURE_ENV_COLOR  0x2201
-#define PGM_GL_TEXTURE_GEN_S      0x0C60
-#define PGM_GL_TEXTURE_GEN_T      0x0C61
-#define PGM_GL_TEXTURE_GEN_MODE   0x2500
-#define PGM_GL_EYE_LINEAR         0x2400
-#define PGM_GL_EYE_PLANE          0x2502
-#define PGM_GL_S                  0x2000
-#define PGM_GL_T                  0x2001
+#define PGM_GL_TEXTURE_ENV          0x2300
+#define PGM_GL_TEXTURE_ENV_MODE     0x2200
+#define PGM_GL_TEXTURE_2D           0x0DE1
+#define PGM_GL_PROXY_TEXTURE_2D     0x8064
+#define PGM_GL_TEXTURE_WRAP_S       0x2802
+#define PGM_GL_TEXTURE_WRAP_T       0x2803
+#define PGM_GL_TEXTURE_MAG_FILTER   0x2800
+#define PGM_GL_TEXTURE_MIN_FILTER   0x2801
+#define PGM_GL_TEXTURE_ENV_COLOR    0x2201
+#define PGM_GL_TEXTURE_GEN_S        0x0C60
+#define PGM_GL_TEXTURE_GEN_T        0x0C61
+#define PGM_GL_TEXTURE_GEN_MODE     0x2500
+#define PGM_GL_TEXTURE_BORDER_COLOR 0x1004
+#define PGM_GL_EYE_LINEAR           0x2400
+#define PGM_GL_EYE_PLANE            0x2502
+#define PGM_GL_S                    0x2000
+#define PGM_GL_T                    0x2001
 
 #define PGM_GL_MODULATE           0x2100
 #define PGM_GL_NEAREST            0x2600
@@ -504,6 +505,8 @@ typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_tex_image_2d)
    PgmGlEnum type, const PgmGlVoid *pixels);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_tex_parameter_i)
   (PgmGlEnum target, PgmGlEnum pname, PgmGlInt param);
+typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_tex_parameter_fv)
+  (PgmGlEnum target, PgmGlEnum pname, PgmGlFloat *params);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_copy_tex_sub_image_2d)
   (PgmGlEnum target, PgmGlInt level, PgmGlInt xoffset, PgmGlInt yoffset,
    PgmGlInt x, PgmGlInt y, PgmGlSizei width, PgmGlSizei height);
