@@ -287,8 +287,8 @@ set_image_standard_vertices (PgmGlImage *glimage)
   gfloat border_width, border_height;
 
   /* Get the border sizes avoiding overlapped borders */
-  border_width = MIN (glimage->border_width, gldrawable->width / 2.0f);
-  border_height = MIN (glimage->border_width, gldrawable->height / 2.0f);
+  border_width = MIN (glimage->border_width, gldrawable->width * 0.5f);
+  border_height = MIN (glimage->border_width, gldrawable->height * 0.5f);
 
   /* Copy the drawable vertices applying the border offset */
   img_vertex[0] = drb_vertex[0] + border_width;
@@ -482,13 +482,18 @@ update_vertices_position (PgmGlImage *glimage)
   y_offset = gldrawable->y - glimage->last_y;
   z_offset = gldrawable->z - glimage->last_z;
 
-  /* Add the offsets to the current vertices */
-  for (i = 0; i < 12; i += 3)
-    {
-      img_vertex[i] += x_offset;
-      img_vertex[i+1] += y_offset;
-      img_vertex[i+2] += z_offset;
-    }
+  img_vertex[0] += x_offset;
+  img_vertex[1] += y_offset;
+  img_vertex[2] += z_offset;
+  img_vertex[3] += x_offset;
+  img_vertex[4] = img_vertex[1];
+  img_vertex[5] = img_vertex[2];
+  img_vertex[6] = img_vertex[3];
+  img_vertex[7] += y_offset;
+  img_vertex[8] = img_vertex[5];
+  img_vertex[9] = img_vertex[0];
+  img_vertex[10] = img_vertex[7];
+  img_vertex[11] = img_vertex[8];
 
   /* Add the offsets to the border vertices, if necessary */
   if (glimage->border_width > 0.0f)
@@ -558,8 +563,6 @@ pgm_gl_image_draw (PgmGlDrawable *gldrawable)
   PgmGlImage *glimage = PGM_GL_IMAGE (gldrawable);
   PgmContextProcAddress *gl;
 
-  GST_LOG_OBJECT (gldrawable, "draw");
-
   /* Avoid drawing if it's not visible */
   if (glimage->empty || (glimage->fg_color[3] == 0.0f))
     return;
@@ -581,7 +584,11 @@ pgm_gl_image_draw (PgmGlDrawable *gldrawable)
 
   gl->disable_client_state (PGM_GL_TEXTURE_COORD_ARRAY);
 
-  if (glimage->border_width > 0.0f)
+  if (glimage->border_width <= 0.0f)
+    {
+      gl->disable_client_state (PGM_GL_VERTEX_ARRAY);
+    }
+  else
     {
       /* Upload vertices and colors for the border */
       gl->enable_client_state (PGM_GL_COLOR_ARRAY);
@@ -591,15 +598,13 @@ pgm_gl_image_draw (PgmGlDrawable *gldrawable)
       /* Draw the border */
       gl->draw_arrays (PGM_GL_QUAD_STRIP, 0, 10);
       gl->disable_client_state (PGM_GL_COLOR_ARRAY);
+      gl->disable_client_state (PGM_GL_VERTEX_ARRAY);
     }
-
-  gl->disable_client_state (PGM_GL_VERTEX_ARRAY);
 }
 
 static void
 pgm_gl_image_regenerate (PgmGlDrawable *gldrawable)
 {
-  GST_LOG_OBJECT (gldrawable, "regenerate");
 }
 
 static void
@@ -607,8 +612,6 @@ pgm_gl_image_update_projection (PgmGlDrawable *gldrawable)
 {
   PgmGlImage *glimage = PGM_GL_IMAGE (gldrawable);
 
-  GST_LOG_OBJECT (gldrawable, "update_projection");
-
   update_image_ratio (glimage);
   update_layout (glimage);
 }
@@ -618,8 +621,6 @@ pgm_gl_image_set_size (PgmGlDrawable *gldrawable)
 {
   PgmGlImage *glimage = PGM_GL_IMAGE (gldrawable);
 
-  GST_DEBUG_OBJECT (gldrawable, "set_size");
-
   update_drawable_ratio (glimage);
   update_layout (glimage);
 }
@@ -629,8 +630,6 @@ pgm_gl_image_set_position (PgmGlDrawable *gldrawable)
 {
   PgmGlImage *glimage = PGM_GL_IMAGE (gldrawable);
 
-  GST_DEBUG_OBJECT (gldrawable, "set_position");
-
   update_vertices_position (glimage);
 }
 
@@ -640,8 +639,6 @@ pgm_gl_image_set_fg_color (PgmGlDrawable *gldrawable)
   PgmGlImage *glimage = PGM_GL_IMAGE (gldrawable);
   PgmDrawable *drawable;
 
-  GST_DEBUG_OBJECT (gldrawable, "set_fg_color");
-
   drawable = gldrawable->drawable;
 
   GST_OBJECT_LOCK (drawable);
@@ -662,8 +659,6 @@ pgm_gl_image_set_opacity (PgmGlDrawable *gldrawable)
   gfloat fg_opacity, border_opacity;
   guint i;
 
-  GST_DEBUG_OBJECT (gldrawable, "set_opacity");
-
   /* Get the opacities */
   GST_OBJECT_LOCK (drawable);
   fg_opacity = drawable->fg_a * drawable->opacity * SQR_INV_255;
@@ -685,8 +680,6 @@ pgm_gl_image_sync (PgmGlDrawable *gldrawable)
   PgmImage *image = PGM_IMAGE (gldrawable->drawable);
   PgmImageStorageType type;
 
-  GST_LOG_OBJECT (gldrawable, "sync");
-
   /* Synchronize various properties */
   pgm_gl_image_set_fg_color (gldrawable);
   update_interp (glimage);
@@ -734,7 +727,7 @@ pgm_gl_image_sync (PgmGlDrawable *gldrawable)
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlImage, pgm_gl_image, PGM_TYPE_GL_DRAWABLE);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlImage, pgm_gl_image, PGM_TYPE_GL_DRAWABLE)
 
 void
 pgm_gl_image_register (GTypeModule *module)
@@ -750,13 +743,10 @@ pgm_gl_image_dispose (GObject *object)
   PgmGlViewport *glviewport = gldrawable->glviewport;
   PgmGlImage *glimage = PGM_GL_IMAGE (object);
   PgmImage *master = PGM_IMAGE (gldrawable->drawable);
+  PgmContextTask *task;
   PgmGlImage *slave;
   GList *walk;
 
-  PgmContextTask *task;
-
-  GST_DEBUG_OBJECT (glimage, "dispose");
-
   /* Clean up the slaves */
   GST_OBJECT_LOCK (master);
   walk = master->slaves;
@@ -827,10 +817,8 @@ pgm_gl_image_class_finalize (PgmGlImageClass *klass)
 static void
 pgm_gl_image_init (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "init");
-
   glimage->empty = TRUE;
-  glimage->border_width = 0.0f;
+  /* glimage->border_width = 0.0f; */
 }
 
 /* Public methods */
@@ -847,8 +835,6 @@ pgm_gl_image_new (PgmDrawable *drawable,
 
   glimage = g_object_new (PGM_TYPE_GL_IMAGE, NULL);
 
-  GST_DEBUG_OBJECT (glimage, "created new glimage");
-
   glimage->native_texture = pgm_texture_new (glviewport->context);
   glimage->texture = glimage->native_texture;
 
@@ -888,8 +874,6 @@ pgm_gl_image_clear (PgmGlImage *glimage)
   PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (glimage);
   PgmContextTask *task = NULL;
 
-  GST_DEBUG_OBJECT (glimage, "clear");
-
   if (glimage->empty)
     return;
 
@@ -917,8 +901,6 @@ pgm_gl_image_set_from_file (PgmGlImage *glimage)
   PgmImage *image = PGM_IMAGE (gldrawable->drawable);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_from_file");
-
   GST_OBJECT_LOCK (image);
 
   if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_FILE
@@ -960,8 +942,6 @@ pgm_gl_image_set_from_buffer (PgmGlImage *glimage)
   PgmImage *image = PGM_IMAGE (gldrawable->drawable);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_from_buffer");
-
   GST_OBJECT_LOCK (image);
 
   if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_BUFFER
@@ -974,7 +954,7 @@ pgm_gl_image_set_from_buffer (PgmGlImage *glimage)
   pgm_texture_set_buffer (glimage->texture, image->data.buffer.buffer,
                           image->data.buffer.format, image->data.buffer.width,
                           image->data.buffer.height, image->data.buffer.size,
-                          image->data.buffer.stride);
+                          image->data.buffer.stride, FALSE);
   pgm_texture_set_matrix (glimage->texture, image->mapping_matrix);
 
   GST_OBJECT_UNLOCK (image);
@@ -998,8 +978,6 @@ pgm_gl_image_set_from_gst_buffer (PgmGlImage *glimage)
   PgmImage *image = PGM_IMAGE (gldrawable->drawable);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_from_gst_buffer");
-
   /* It's the first time the glimage receives a GstBuffer, let's set the
    * texture buffer with its different informations */
   if (G_UNLIKELY (glimage->empty))
@@ -1060,8 +1038,6 @@ pgm_gl_image_set_from_pixbuf (PgmGlImage *glimage)
   PgmImage *image = PGM_IMAGE (gldrawable->drawable);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_from_pixbuf");
-
   GST_OBJECT_LOCK (image);
 
   if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_PIXBUF
@@ -1094,8 +1070,6 @@ pgm_gl_image_set_from_system_buffer (PgmGlImage *glimage)
   PgmImage *image = PGM_IMAGE (gldrawable->drawable);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_from_system_buffer");
-
   GST_OBJECT_LOCK (image);
 
   if (!(gldrawable->glviewport->context->feature_mask
@@ -1150,8 +1124,6 @@ pgm_gl_image_set_from_image (PgmGlImage *glimage)
   PgmGlViewport *glviewport = gldrawable->glviewport;
   PgmGlImage *master;
 
-  GST_DEBUG_OBJECT (glimage, "set_from_image");
-
   /* Retrieve the master glimage */
   GST_OBJECT_LOCK (image);
 
@@ -1179,16 +1151,12 @@ pgm_gl_image_set_from_image (PgmGlImage *glimage)
 void
 pgm_gl_image_set_mapping_matrix (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_mapping_matrix");
-
   update_mapping_matrix (glimage);
 }
 
 void
 pgm_gl_image_set_alignment (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_alignment");
-
   update_alignment (glimage);
   update_layout (glimage);
 }
@@ -1196,8 +1164,6 @@ pgm_gl_image_set_alignment (PgmGlImage *glimage)
 void
 pgm_gl_image_set_layout (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_layout");
-
   update_layout (glimage);
 }
 
@@ -1207,8 +1173,6 @@ pgm_gl_image_set_interp (PgmGlImage *glimage)
   PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (glimage);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_interp");
-
   update_interp (glimage);
 
   task = pgm_context_task_new (PGM_CONTEXT_UPDATE_TEXTURE, glimage->texture);
@@ -1221,8 +1185,6 @@ pgm_gl_image_set_wrapping (PgmGlImage *glimage)
   PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (glimage);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glimage, "set_wrapping");
-
   update_wrapping (glimage);
 
   task = pgm_context_task_new (PGM_CONTEXT_UPDATE_TEXTURE, glimage->texture);
@@ -1232,8 +1194,6 @@ pgm_gl_image_set_wrapping (PgmGlImage *glimage)
 void
 pgm_gl_image_set_aspect_ratio (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_aspect_ratio");
-
   update_image_ratio (glimage);
   update_layout (glimage);
 }
@@ -1241,8 +1201,6 @@ pgm_gl_image_set_aspect_ratio (PgmGlImage *glimage)
 void
 pgm_gl_image_set_border_width (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_border_width");
-
   update_border_width (glimage);
   update_layout (glimage);
 }
@@ -1250,21 +1208,16 @@ pgm_gl_image_set_border_width (PgmGlImage *glimage)
 void
 pgm_gl_image_set_border_inner_color (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_border_inner_color");
-
   update_border_inner_color (glimage);
 }
 
 void
 pgm_gl_image_set_border_outer_color (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "set_border_outer_color");
-
   update_border_outer_color (glimage);
 }
 
 void
 pgm_gl_image_system_buffer_changed (PgmGlImage *glimage)
 {
-  GST_DEBUG_OBJECT (glimage, "system_buffer_changed");
 }
