@@ -66,10 +66,10 @@
 
 /* These are the A,R,G,B component byte masks for a native endian ARGB value
  * (such as what is given by Cairo) */
-#define ARGB_A_MASK 0xFF << ARGB_A_OFFSET
-#define ARGB_R_MASK 0xFF << ARGB_R_OFFSET
-#define ARGB_G_MASK 0xFF << ARGB_G_OFFSET
-#define ARGB_B_MASK 0xFF << ARGB_B_OFFSET
+#define ARGB_A_MASK 0xFF000000
+#define ARGB_R_MASK 0x00FF0000
+#define ARGB_G_MASK 0x0000FF00
+#define ARGB_B_MASK 0x000000FF
 
 GST_DEBUG_CATEGORY_STATIC (pgm_gl_text_debug);
 #define GST_CAT_DEFAULT pgm_gl_text_debug
@@ -84,14 +84,8 @@ get_projected_height (PgmGlDrawable *gldrawable,
                       gfloat canvas_height)
 {
   PgmGlViewport *glviewport = gldrawable->glviewport;
-  PgmViewport *viewport = PGM_VIEWPORT (glviewport);
-  gfloat height;
 
-  GST_OBJECT_LOCK (viewport);
-  height = (canvas_height * glviewport->projected_h) / glviewport->canvas_h;
-  GST_OBJECT_UNLOCK (viewport);
-
-  return height;
+  return (canvas_height * glviewport->projected_h) / glviewport->canvas_h;
 }
 
 /* Gets the projected width of the drawable on the viewport at z=0 */
@@ -100,62 +94,8 @@ get_projected_width (PgmGlDrawable *gldrawable,
                      gfloat canvas_width)
 {
   PgmGlViewport *glviewport = gldrawable->glviewport;
-  PgmViewport *viewport = PGM_VIEWPORT (glviewport);
-  gfloat width;
 
-  GST_OBJECT_LOCK (viewport);
-  width = (canvas_width * glviewport->projected_w) / glviewport->canvas_w;
-  GST_OBJECT_UNLOCK (viewport);
-
-  return width;
-}
-
-/* Sets the position of the text taking care of aligning the vertices to the
- * pixel boundaries. It ensures clean text rendering without any artifact
- * implied by the sub-pixels rasterization process. */
-static void
-set_position (PgmGlDrawable *gldrawable)
-{
-  PgmGlViewport *glviewport = gldrawable->glviewport;
-  PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
-  gfloat offset_x = glviewport->pixel_offset_x;
-  gfloat offset_y = glviewport->pixel_offset_y;
-  gfloat bound_width = floorf (gldrawable->width / offset_x) * offset_x;
-  gfloat bound_height = floorf (gldrawable->height / offset_y) * offset_y;
-  gfloat bound_x = floorf (gldrawable->x / offset_x) * offset_x;
-  gfloat bound_y = floorf (gldrawable->y / offset_y) * offset_y;
-
-  gltext->vertex[0] = bound_x;
-  gltext->vertex[1] = bound_y;
-  gltext->vertex[2] = gldrawable->z;
-  gltext->vertex[3] = bound_x + bound_width;
-  gltext->vertex[4] = bound_y;
-  gltext->vertex[5] = gldrawable->z;
-  gltext->vertex[6] = gltext->vertex[3];
-  gltext->vertex[7] = bound_y + bound_height;
-  gltext->vertex[8] = gldrawable->z;
-  gltext->vertex[9] = bound_x;
-  gltext->vertex[10] = gltext->vertex[7];
-  gltext->vertex[11] = gldrawable->z;
-}
-
-/* Sets the size of the text taking care of aligning the vertices to the
- * pixel boundaries. It ensures clean text rendering without any artifact
- * implied by the sub-pixels rasterization process. */
-static void
-set_size (PgmGlDrawable *gldrawable)
-{
-  PgmGlViewport *glviewport = gldrawable->glviewport;
-  PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
-  gfloat offset_x = glviewport->pixel_offset_x;
-  gfloat offset_y = glviewport->pixel_offset_y;
-  gfloat bound_width = floorf (gldrawable->width / offset_x) * offset_x;
-  gfloat bound_height = floorf (gldrawable->height / offset_y) * offset_y;
-
-  gltext->vertex[3] = gldrawable->bg_vertex[0] + bound_width;
-  gltext->vertex[6] = gltext->vertex[3];
-  gltext->vertex[7] = gldrawable->bg_vertex[1] + bound_height;
-  gltext->vertex[10] = gltext->vertex[7];
+  return (canvas_width * glviewport->projected_w) / glviewport->canvas_w;
 }
 
 /* Defines texture coordinates */
@@ -163,13 +103,40 @@ static void
 set_coordinates (PgmGlText *gltext)
 {
   PgmTexture *texture = gltext->texture;
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
+  PgmText *text = PGM_TEXT (gldrawable->drawable);
+  gfloat ratio = ((gfloat) texture->height / texture->height_pot);
+  gfloat offset;
+
+  if (gltext->multiline == TRUE)
+    offset = 0.0f;
+  else
+    offset = (1.0f - (text->height / gldrawable->height)) * ratio * -0.5f;
 
   gltext->coord[2] = (PgmGlFloat) texture->width / texture->width_pot;
   gltext->coord[4] = gltext->coord[2];
-  gltext->coord[5] = (PgmGlFloat) texture->height / texture->height_pot;
+  gltext->coord[1] = offset;
+  gltext->coord[3] = gltext->coord[1];
+  gltext->coord[5] = (PgmGlFloat) ratio + offset;
   gltext->coord[7] = gltext->coord[5];
 }
 
+/* Stores color depending on PgmText color normalizing components to the range
+ * [0.0; 1.0] */
+static void
+set_color (PgmGlText *gltext)
+{
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
+  PgmText *text = PGM_TEXT (gldrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+  gltext->color[0] = text->r * INV_255;
+  gltext->color[1] = text->g * INV_255;
+  gltext->color[2] = text->b * INV_255;
+  gltext->color[3] = text->a * INV_255;
+  GST_OBJECT_UNLOCK (text);
+}
+
 /* Stores outline width depending on PgmText outline width converting it
  * from canvas coordinates to viewport coordinates */
 static void
@@ -197,6 +164,60 @@ set_outline_color (PgmGlText *gltext)
   GST_OBJECT_UNLOCK (text);
 }
 
+/* Stores the shadow casting state */
+static void
+set_cast_shadow (PgmGlText *gltext)
+{
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
+  PgmText *text = PGM_TEXT (gldrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+  gltext->cast_shadow = text->cast_shadow;
+  GST_OBJECT_UNLOCK (text);
+}
+
+/* Stores the shadow position */
+static void
+set_shadow_position (PgmGlText *gltext)
+{
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
+  PgmText *text = PGM_TEXT (gldrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+
+  if (text->shadow_position & PGM_TEXT_SHADOW_NORTH)
+    gltext->shadow_longitude_factor = -1.0f;
+  else if (text->shadow_position & PGM_TEXT_SHADOW_SOUTH)
+    gltext->shadow_longitude_factor = 1.0f;
+  else
+    gltext->shadow_longitude_factor = 0.0f;
+
+  if (text->shadow_position & PGM_TEXT_SHADOW_WEST)
+    gltext->shadow_latitude_factor = -1.0f;
+  else if (text->shadow_position & PGM_TEXT_SHADOW_EAST)
+    gltext->shadow_latitude_factor = 1.0f;
+  else
+    gltext->shadow_latitude_factor = 0.0f;
+
+  GST_OBJECT_UNLOCK (text);
+}
+
+/* Stores shadow color depending on PgmText color normalizing components to the
+ * range [0.0; 1.0] */
+static void
+set_shadow_color (PgmGlText *gltext)
+{
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
+  PgmText *text = PGM_TEXT (gldrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+  gltext->shadow_color[0] = text->shadow_r * INV_255;
+  gltext->shadow_color[1] = text->shadow_g * INV_255;
+  gltext->shadow_color[2] = text->shadow_b * INV_255;
+  gltext->shadow_color[3] = text->shadow_a * INV_255;
+  GST_OBJECT_UNLOCK (text);
+}
+
 /* Defines Pango spacing depending on PgmText line spacing converting it
  * from canvas coordinates to viewport coordinates */
 static void
@@ -207,8 +228,8 @@ set_line_spacing (PgmGlText *gltext)
   gfloat line_spacing;
 
   line_spacing = get_projected_height (gldrawable, text->line_spacing);
-  pango_layout_set_spacing (gltext->layout, (gint)
-                            (line_spacing * PANGO_SCALE));
+  pango_layout_set_spacing
+    (gltext->layout, pgm_lrintf (line_spacing * PANGO_SCALE));
 }
 
 /* Calculates the maximum total height of a line of text for the current font
@@ -259,8 +280,8 @@ set_font_height (PgmGlText *gltext)
   /* All the heights here are in device unit. */
   gfloat requested_height, wanted_height, max_height, font_ratio;
 
+  /* text->height is a value in canvas space */
   requested_height = get_projected_height (gldrawable, text->height);
-  requested_height *= gldrawable->height;
 
   /* The "pango height" is something like the height of an M, but we want to set
    * the maximum height of a line, so we need to calculate the ratio first. */
@@ -273,6 +294,10 @@ set_font_height (PgmGlText *gltext)
   pango_font_description_set_absolute_size (gltext->desc,
                                             wanted_height * PANGO_SCALE);
   pango_layout_set_font_description (gltext->layout, gltext->desc);
+
+  /* Precompute and store the offset to apply to the shadow based on the font
+   * wanted height, the idea is to increase the offset with the font height. */
+  gltext->shadow_offset = sqrtf (wanted_height / 16.0f);
 }
 
 /* Defines Pango font family depending on PgmText font family */
@@ -310,18 +335,63 @@ static void
 set_width (PgmGlText *gltext)
 {
   PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
-  PgmViewport *viewport = PGM_VIEWPORT (gldrawable->glviewport);
-  gfloat non_square_ratio;
-  gfloat width;
 
-  width = get_projected_width (gldrawable, gldrawable->width);
+  /* Multiline mode */
+  if (gltext->multiline == TRUE)
+    {
+      PgmViewport *viewport = PGM_VIEWPORT (gldrawable->glviewport);
+      gfloat non_square_ratio, size;
 
-  GST_OBJECT_LOCK (viewport);
-  non_square_ratio = 1.0f / viewport->pixel_aspect_ratio;
-  GST_OBJECT_UNLOCK (viewport);
+      /* Set the correct width */
+      size = get_projected_width (gldrawable, gldrawable->width);
+      GST_OBJECT_LOCK (viewport);
+      non_square_ratio = 1.0f / viewport->pixel_aspect_ratio;
+      GST_OBJECT_UNLOCK (viewport);
+      pango_layout_set_width
+        (gltext->layout, pgm_lrintf (size * PANGO_SCALE * non_square_ratio));
 
-  pango_layout_set_width (gltext->layout, (guint)
-                          (width * PANGO_SCALE * non_square_ratio));
+      /* Set the correct height */
+      size = get_projected_height (gldrawable, gldrawable->height);
+      pango_layout_set_height (gltext->layout, pgm_lrintf (size * PANGO_SCALE));
+    }
+
+  /* Monoline mode */
+  else
+    {
+      PgmText *text = PGM_TEXT (gldrawable->drawable);
+
+      /* If the text needs to be ellipsized, set the correct width */
+      if (text->ellipsize != PGM_TEXT_ELLIPSIZE_NONE)
+        {
+          PgmViewport *viewport = PGM_VIEWPORT (gldrawable->glviewport);
+          gfloat non_square_ratio, width;
+
+          /* Set the correct width */
+          width = get_projected_width (gldrawable, gldrawable->width);
+          GST_OBJECT_LOCK (viewport);
+          non_square_ratio = 1.0f / viewport->pixel_aspect_ratio;
+          GST_OBJECT_UNLOCK (viewport);
+          pango_layout_set_width
+            (gltext->layout,
+             pgm_lrintf (width * PANGO_SCALE * non_square_ratio));
+        }
+      /* Otherwise disable the width */
+      else
+        pango_layout_set_width (gltext->layout, -1);
+
+      /* Disable the height */
+      pango_layout_set_height (gltext->layout, -1);
+    }
+}
+
+/* Stores the multiline state */
+static void
+set_multiline (PgmGlText *gltext)
+{
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
+  PgmText *text = PGM_TEXT (gldrawable->drawable);
+
+  gltext->multiline = text->multiline;
 }
 
 /* Defines Pango justification depending on PgmText justification */
@@ -579,27 +649,33 @@ update_properties (PgmGlText *gltext)
   set_alignment (gltext);
   set_ellipsize (gltext);
   set_justify (gltext);
+  set_multiline (gltext);
   set_line_spacing (gltext);
   set_width (gltext);
+  set_coordinates (gltext);
   set_wrap (gltext);
   set_gravity (gltext);
   set_label (gltext);
+  set_color (gltext);
   set_outline_width (gltext);
   set_outline_color (gltext);
+  set_cast_shadow (gltext);
+  set_shadow_position (gltext);
+  set_shadow_color (gltext);
   set_font_height (gltext);
 }
 
-/* Cairo uses premultiplied-alpha to represent pixels (mostly for compositing).
- * This function unpremultiplies the alpha of cairo's native endian ARGB buffer
- * and converts it to BGRA in memory order. */
+/* Cairo uses a premultiplied alpha blending function, pigment's not. This
+ * function unpremultiplies the alpha of cairo's native endian ARGB buffer and
+ * converts it to BGRA in memory order. */
 static void
 unpremultiply_alpha (guint32 *buffer,
-                     guint width,
-                     guint height)
+                     guint32 width,
+                     guint32 height)
 {
-  guint size = width * height;
-  guint8 a, r, g, b;
-  guint i;
+  guint32 size = width * height;
+  guint32 a, r, g, b;
+  guint32 i;
 
   for (i = 0; i < size; i++)
     {
@@ -643,10 +719,15 @@ create_pixmap (PgmGlText *gltext)
   PgmContextTask *task;
   gfloat sx, sy;
 
-  gltext->width =
-    MAX (1, (guint) get_projected_width (gldrawable, gldrawable->width));
-  gltext->height =
-    MAX (1, (guint) get_projected_height (gldrawable, gldrawable->height));
+  /* It's important here to round the floating-point value returned by
+   * get_projected_* using the default "to nearest" rounding mode of the FPU and
+   * not the "truncate" mode implied by the float-to-int cast. It's faster and
+   * allows to get the exact rasterized size required to have a direct texel to
+   * pixel mapping. */
+  gltext->width = MAX
+    (1, pgm_lrintf (get_projected_width (gldrawable, gldrawable->width)));
+  gltext->height = MAX
+    (1, pgm_lrintf (get_projected_height (gldrawable, gldrawable->height)));
 
   GST_OBJECT_LOCK (viewport);
   if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
@@ -677,7 +758,8 @@ create_pixmap (PgmGlText *gltext)
   gltext->pango_ctx = pango_layout_get_context (gltext->layout);
 
   pgm_texture_set_buffer (gltext->texture, gltext->buffer, PGM_IMAGE_BGRA,
-                          gltext->width, gltext->height, gltext->size, 0);
+                          gltext->width, gltext->height, gltext->size, 0,
+                          TRUE);
   set_coordinates (gltext);
 
   /* And request a texture generation */
@@ -716,14 +798,36 @@ update_pixmap (PgmGlText *gltext)
   PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (gltext);
   PgmContextTask *task;
 
-  /* Make context fully transparent */
+  /* Clear the surface */
   cairo_set_operator (gltext->cairo_ctx, CAIRO_OPERATOR_SOURCE);
   cairo_set_source_rgba (gltext->cairo_ctx, 0.0f, 0.0f, 0.0f, 0.0f);
   cairo_paint (gltext->cairo_ctx);
 
-  /* Then draw the text */
-  cairo_set_source_rgba (gltext->cairo_ctx, 1.0f, 1.0f, 1.0f, 1.0f);
-  pango_cairo_show_layout (gltext->cairo_ctx, gltext->layout);
+  /* Draw the text without shadow */
+  if (!gltext->cast_shadow)
+    {
+      cairo_set_source_rgba (gltext->cairo_ctx, gltext->color[0],
+                             gltext->color[1], gltext->color[2],
+                             gltext->color[3]);
+      pango_cairo_show_layout (gltext->cairo_ctx, gltext->layout);
+    }
+  /* Draw the text with a shadow */
+  else
+    {
+      gdouble tx = gltext->shadow_offset * gltext->shadow_latitude_factor;
+      gdouble ty = gltext->shadow_offset * gltext->shadow_longitude_factor;
+
+      cairo_set_source_rgba (gltext->cairo_ctx, gltext->shadow_color[0],
+                             gltext->shadow_color[1], gltext->shadow_color[2],
+                             gltext->shadow_color[3]);
+      cairo_translate (gltext->cairo_ctx, tx, ty);
+      pango_cairo_show_layout (gltext->cairo_ctx, gltext->layout);
+      cairo_translate (gltext->cairo_ctx, -tx, -ty);
+      cairo_set_source_rgba (gltext->cairo_ctx, gltext->color[0],
+                             gltext->color[1], gltext->color[2],
+                             gltext->color[3]);
+      pango_cairo_show_layout (gltext->cairo_ctx, gltext->layout);
+    }
 
   /* And the stroke */
   if (gltext->outline_width > 0.0f)
@@ -752,7 +856,8 @@ update_pixmap (PgmGlText *gltext)
 
   /* Update texture */
   pgm_texture_set_buffer (gltext->texture, gltext->buffer, PGM_IMAGE_BGRA,
-                          gltext->width, gltext->height, gltext->size, 0);
+                          gltext->width, gltext->height, gltext->size, 0,
+                          TRUE);
 
   /* Request texture upload */
   task = pgm_context_task_new (PGM_CONTEXT_UPLOAD_TEXTURE, gltext->texture);
@@ -767,7 +872,6 @@ regenerate_text (PgmGlText *gltext)
   create_pixmap (gltext);
   update_properties (gltext);
   update_pixmap (gltext);
-  set_position (PGM_GL_DRAWABLE (gltext));
 }
 
 /* Update a text by simply generating the new one in the current allocated
@@ -792,8 +896,6 @@ pgm_gl_text_draw (PgmGlDrawable *gldrawable)
 {
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
 
-  GST_LOG_OBJECT (gldrawable, "draw");
-
   /* Only draw if it's visible */
   if (gltext->fg_color[3] != 0.0f)
     {
@@ -817,8 +919,6 @@ pgm_gl_text_regenerate (PgmGlDrawable *gldrawable)
 {
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
 
-  GST_DEBUG_OBJECT (gldrawable, "regenerate");
-
   if (gltext->size_updated)
     {
       regenerate_text (gltext);
@@ -831,8 +931,6 @@ pgm_gl_text_update_projection (PgmGlDrawable *gldrawable)
 {
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
 
-  GST_DEBUG_OBJECT (gldrawable, "update_projection");
-
   gltext->size_updated = TRUE;
 }
 
@@ -840,10 +938,10 @@ static void
 pgm_gl_text_set_size (PgmGlDrawable *gldrawable)
 {
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
+  guint32 i;
 
-  GST_DEBUG_OBJECT (gldrawable, "set_size");
-
-  set_size (gldrawable);
+  for (i = 0; i < 12; i++)
+    gltext->vertex[i] = gldrawable->bg_vertex[i];
 
   gltext->size_updated = TRUE;
 }
@@ -851,9 +949,11 @@ pgm_gl_text_set_size (PgmGlDrawable *gldrawable)
 static void
 pgm_gl_text_set_position (PgmGlDrawable *gldrawable)
 {
-  GST_DEBUG_OBJECT (gldrawable, "set_position");
+  PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
+  guint32 i;
 
-  set_position (gldrawable);
+  for (i = 0; i < 12; i++)
+    gltext->vertex[i] = gldrawable->bg_vertex[i];
 }
 
 static void
@@ -862,8 +962,6 @@ pgm_gl_text_set_fg_color (PgmGlDrawable *gldrawable)
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
   PgmDrawable *drawable;
 
-  GST_DEBUG_OBJECT (gldrawable, "set_fg_color");
-
   drawable = gldrawable->drawable;
 
   GST_OBJECT_LOCK (drawable);
@@ -880,8 +978,6 @@ pgm_gl_text_set_opacity (PgmGlDrawable *gldrawable)
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
   PgmDrawable *drawable;
 
-  GST_DEBUG_OBJECT (gldrawable, "set_opacity");
-
   drawable = gldrawable->drawable;
 
   GST_OBJECT_LOCK (drawable);
@@ -894,8 +990,6 @@ pgm_gl_text_sync (PgmGlDrawable *gldrawable)
 {
   PgmGlText *gltext = PGM_GL_TEXT (gldrawable);
 
-  GST_DEBUG_OBJECT (gldrawable, "sync");
-
   create_pixmap (gltext);
   update_properties (gltext);
 
@@ -908,7 +1002,7 @@ pgm_gl_text_sync (PgmGlDrawable *gldrawable)
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlText, pgm_gl_text, PGM_TYPE_GL_DRAWABLE);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlText, pgm_gl_text, PGM_TYPE_GL_DRAWABLE)
 
 void
 pgm_gl_text_register (GTypeModule *module)
@@ -923,8 +1017,6 @@ pgm_gl_text_dispose (GObject *object)
   PgmGlText *gltext = PGM_GL_TEXT (object);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (gltext, "dispose");
-
   free_pixmap (gltext);
 
   pango_font_description_free (gltext->desc);
@@ -981,15 +1073,14 @@ pgm_gl_text_class_finalize (PgmGlTextClass *klass)
 static void
 pgm_gl_text_init (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "init");
-
   gltext->font_options = cairo_font_options_create ();
   cairo_font_options_set_antialias (gltext->font_options,
                                     CAIRO_ANTIALIAS_SUBPIXEL);
 
+  gltext->multiline = TRUE;
   gltext->desc = pango_font_description_new ();
 
-  gltext->size_updated = FALSE;
+  /* gltext->size_updated = FALSE; */
 }
 
 /* Public methods */
@@ -1003,10 +1094,10 @@ pgm_gl_text_new (PgmDrawable *drawable,
 
   gltext = g_object_new (PGM_TYPE_GL_TEXT, NULL);
 
-  GST_DEBUG_OBJECT (gltext, "created new gltext");
-
   gltext->texture = pgm_texture_new (glviewport->context);
   gltext->texture->filter = PGM_GL_NEAREST;
+  gltext->texture->wrap_s = PGM_GL_CLAMP_TO_BORDER;
+  gltext->texture->wrap_t = PGM_GL_CLAMP_TO_BORDER;
 
   gldrawable = PGM_GL_DRAWABLE (gltext);
   gldrawable->drawable = gst_object_ref (drawable);
@@ -1020,8 +1111,6 @@ pgm_gl_text_new (PgmDrawable *drawable,
 void
 pgm_gl_text_set_label (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_label");
-
   set_label (gltext);
   update_text (gltext);
 }
@@ -1029,8 +1118,6 @@ pgm_gl_text_set_label (PgmGlText *gltext)
 void
 pgm_gl_text_set_markup (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_markup");
-
   set_label (gltext);
   update_text (gltext);
 }
@@ -1038,8 +1125,6 @@ pgm_gl_text_set_markup (PgmGlText *gltext)
 void
 pgm_gl_text_set_font_family (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_font_family");
-
   set_font_family (gltext);
   update_text (gltext);
 }
@@ -1047,8 +1132,6 @@ pgm_gl_text_set_font_family (PgmGlText *gltext)
 void
 pgm_gl_text_set_font_height (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_font_height");
-
   set_font_height (gltext);
   update_text (gltext);
 }
@@ -1056,17 +1139,15 @@ pgm_gl_text_set_font_height (PgmGlText *gltext)
 void
 pgm_gl_text_set_ellipsize (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_ellipsize");
-
   set_ellipsize (gltext);
+  set_width (gltext);
+  set_coordinates (gltext);
   update_text (gltext);
 }
 
 void
 pgm_gl_text_set_justify (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_justify");
-
   set_justify (gltext);
   update_text (gltext);
 }
@@ -1074,8 +1155,6 @@ pgm_gl_text_set_justify (PgmGlText *gltext)
 void
 pgm_gl_text_set_alignment (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_alignment");
-
   set_alignment (gltext);
   update_text (gltext);
 }
@@ -1083,8 +1162,6 @@ pgm_gl_text_set_alignment (PgmGlText *gltext)
 void
 pgm_gl_text_set_wrap (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_wrap");
-
   set_wrap (gltext);
   update_text (gltext);
 }
@@ -1092,8 +1169,6 @@ pgm_gl_text_set_wrap (PgmGlText *gltext)
 void
 pgm_gl_text_set_gravity (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_gravity");
-
   set_gravity (gltext);
   update_text (gltext);
 }
@@ -1101,8 +1176,6 @@ pgm_gl_text_set_gravity (PgmGlText *gltext)
 void
 pgm_gl_text_set_stretch (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_stretch");
-
   set_stretch (gltext);
   update_text (gltext);
 }
@@ -1110,8 +1183,6 @@ pgm_gl_text_set_stretch (PgmGlText *gltext)
 void
 pgm_gl_text_set_style (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_style");
-
   set_style (gltext);
   update_text (gltext);
 }
@@ -1119,8 +1190,6 @@ pgm_gl_text_set_style (PgmGlText *gltext)
 void
 pgm_gl_text_set_variant (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_variant");
-
   set_variant (gltext);
   update_text (gltext);
 }
@@ -1128,26 +1197,36 @@ pgm_gl_text_set_variant (PgmGlText *gltext)
 void
 pgm_gl_text_set_weight (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_weight");
-
   set_weight (gltext);
   update_text (gltext);
 }
 
+void
+pgm_gl_text_set_multiline (PgmGlText *gltext)
+{
+  set_multiline (gltext);
+  set_width (gltext);
+  set_coordinates (gltext);
+  update_text (gltext);
+}
+
 void
 pgm_gl_text_set_line_spacing (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_line_spacing");
-
   set_line_spacing (gltext);
   update_text (gltext);
 }
 
+void
+pgm_gl_text_set_color (PgmGlText *gltext)
+{
+  set_color (gltext);
+  update_text (gltext);
+}
+
 void
 pgm_gl_text_set_outline_color (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_outline_color");
-
   set_outline_color (gltext);
   update_text (gltext);
 }
@@ -1155,8 +1234,27 @@ pgm_gl_text_set_outline_color (PgmGlText *gltext)
 void
 pgm_gl_text_set_outline_width (PgmGlText *gltext)
 {
-  GST_DEBUG_OBJECT (gltext, "set_outline_width");
-
   set_outline_width (gltext);
   update_text (gltext);
 }
+
+void
+pgm_gl_text_set_cast_shadow (PgmGlText *gltext)
+{
+  set_cast_shadow (gltext);
+  update_text (gltext);
+}
+
+void
+pgm_gl_text_set_shadow_position (PgmGlText *gltext)
+{
+  set_shadow_position (gltext);
+  update_text (gltext);
+}
+
+void
+pgm_gl_text_set_shadow_color (PgmGlText *gltext)
+{
+  set_shadow_color (gltext);
+  update_text (gltext);
+}
