@@ -52,10 +52,11 @@ struct _PgmGlText {
   PgmGlDrawable parent;
 
   /* Pixmap buffer */
-  guint   width, height;
-  guint   size;
   guchar *buffer;
 
+  /* Texture handling */
+  PgmTexture *texture;
+
   /* Pango rendering */
   PangoContext         *pango_ctx;
   PangoFontDescription *desc;
@@ -66,15 +67,28 @@ struct _PgmGlText {
   cairo_surface_t      *surface;
   cairo_font_options_t *font_options;
 
+  /* Pixmap size */
+  guint width, height, size;
+
   /* Regeneration boolean */
   gboolean size_updated;
 
+  /* Multi lines */
+  gboolean multiline;
+
+  /* Color */
+  gfloat color[4];
+
   /* Outline conversions */
   gfloat outline_color[4];
   gfloat outline_width;
 
-  /* Texture handling */
-  PgmTexture *texture;
+  /* Shadow */
+  gboolean cast_shadow;
+  gfloat shadow_color[4];
+  gfloat shadow_offset;
+  gfloat shadow_latitude_factor;
+  gfloat shadow_longitude_factor;
 
   /* Vertex arrays */
   PgmGlFloat fg_color[4];
@@ -86,29 +100,34 @@ struct _PgmGlTextClass {
   PgmGlDrawableClass parent_class;
 };
 
-GType          pgm_gl_text_get_type          (void) G_GNUC_CONST;
+GType          pgm_gl_text_get_type            (void) G_GNUC_CONST;
 
-PgmGlDrawable *pgm_gl_text_new               (PgmDrawable *drawable,
-                                              PgmGlViewport *glviewport);
+PgmGlDrawable *pgm_gl_text_new                 (PgmDrawable *drawable,
+                                                PgmGlViewport *glviewport);
 
-void           pgm_gl_text_register          (GTypeModule *module);
+void           pgm_gl_text_register            (GTypeModule *module);
 
-void           pgm_gl_text_set_label         (PgmGlText *gltext);
-void           pgm_gl_text_set_markup        (PgmGlText *gltext);
-void           pgm_gl_text_set_font_family   (PgmGlText *gltext);
-void           pgm_gl_text_set_font_height   (PgmGlText *gltext);
-void           pgm_gl_text_set_ellipsize     (PgmGlText *gltext);
-void           pgm_gl_text_set_justify       (PgmGlText *gltext);
-void           pgm_gl_text_set_alignment     (PgmGlText *gltext);
-void           pgm_gl_text_set_wrap          (PgmGlText *gltext);
-void           pgm_gl_text_set_gravity       (PgmGlText *gltext);
-void           pgm_gl_text_set_stretch       (PgmGlText *gltext);
-void           pgm_gl_text_set_style         (PgmGlText *gltext);
-void           pgm_gl_text_set_variant       (PgmGlText *gltext);
-void           pgm_gl_text_set_weight        (PgmGlText *gltext);
-void           pgm_gl_text_set_line_spacing  (PgmGlText *gltext);
-void           pgm_gl_text_set_outline_color (PgmGlText *gltext);
-void           pgm_gl_text_set_outline_width (PgmGlText *gltext);
+void           pgm_gl_text_set_label           (PgmGlText *gltext);
+void           pgm_gl_text_set_markup          (PgmGlText *gltext);
+void           pgm_gl_text_set_font_family     (PgmGlText *gltext);
+void           pgm_gl_text_set_font_height     (PgmGlText *gltext);
+void           pgm_gl_text_set_ellipsize       (PgmGlText *gltext);
+void           pgm_gl_text_set_justify         (PgmGlText *gltext);
+void           pgm_gl_text_set_alignment       (PgmGlText *gltext);
+void           pgm_gl_text_set_wrap            (PgmGlText *gltext);
+void           pgm_gl_text_set_gravity         (PgmGlText *gltext);
+void           pgm_gl_text_set_stretch         (PgmGlText *gltext);
+void           pgm_gl_text_set_style           (PgmGlText *gltext);
+void           pgm_gl_text_set_variant         (PgmGlText *gltext);
+void           pgm_gl_text_set_weight          (PgmGlText *gltext);
+void           pgm_gl_text_set_multiline       (PgmGlText *gltext);
+void           pgm_gl_text_set_line_spacing    (PgmGlText *gltext);
+void           pgm_gl_text_set_color           (PgmGlText *gltext);
+void           pgm_gl_text_set_outline_color   (PgmGlText *gltext);
+void           pgm_gl_text_set_outline_width   (PgmGlText *gltext);
+void           pgm_gl_text_set_cast_shadow     (PgmGlText *gltext);
+void           pgm_gl_text_set_shadow_position (PgmGlText *gltext);
+void           pgm_gl_text_set_shadow_color    (PgmGlText *gltext);
 
 G_END_DECLS
 
