@@ -133,6 +133,37 @@ static PgmViewportClass *parent_class = NULL;
 
 /* Private functions */
 
+/* Copy a layer taking a ref on each drawable */
+static GList*
+copy_layer (GList *layer)
+{
+  GList *new_layer = NULL;
+
+  if (layer)
+    {
+      GList *last;
+
+      new_layer = g_slice_new (GList);
+      new_layer->data = g_object_ref (layer->data);
+      new_layer->prev = NULL;
+      last = new_layer;
+      layer = layer->next;
+
+      while (layer)
+        {
+          last->next = g_slice_new (GList);
+          last->next->prev = last;
+          last = last->next;
+          last->data = g_object_ref (layer->data);
+          layer = layer->next;
+        }
+
+      last->next = NULL;
+    }
+
+  return new_layer;
+}
+
 /* Create a change task */
 static Task*
 task_change_new (PgmGlDrawable *gldrawable,
@@ -327,6 +358,9 @@ gl_drawable_free (PgmGlViewport *glviewport,
   GSList *walk, *tmp;
   Task *task;
 
+  if (gldrawable == NULL)
+    return;
+
   /* Disconnect from the 'changed' signal and remove it from the hash */
   GST_OBJECT_LOCK (glviewport);
   GST_OBJECT_LOCK (gldrawable->drawable);
@@ -509,22 +543,42 @@ do_task_remove (PgmGlViewport *glviewport,
   task_remove_free (task);
 }
 
+/* Regenerate a layer */
+static void
+regenerate_layer (GList *layer)
+{
+  GList *walk = layer;
+
+  while (walk)
+    {
+      PgmGlDrawable *gldrawable = (PgmGlDrawable*) walk->data;
+      pgm_gl_drawable_regenerate (gldrawable);
+      g_object_unref (gldrawable);
+      walk = walk->next;
+    }
+
+  g_list_free (layer);
+}
+
 /* Complete a regenerate task and free it */
 static void
 do_task_regenerate (PgmGlViewport *glviewport,
                     Task *task)
 {
+  GList *near_layer, *middle_layer, *far_layer;
+
+  /* Get copies of the layers */
   g_mutex_lock (glviewport->layer_lock);
-
-  g_list_foreach (glviewport->near_layer,
-                  (GFunc) pgm_gl_drawable_regenerate, NULL);
-  g_list_foreach (glviewport->middle_layer,
-                  (GFunc) pgm_gl_drawable_regenerate, NULL);
-  g_list_foreach (glviewport->far_layer,
-                  (GFunc) pgm_gl_drawable_regenerate, NULL);
-
+  near_layer = copy_layer (glviewport->near_layer);
+  middle_layer = copy_layer (glviewport->middle_layer);
+  far_layer = copy_layer (glviewport->far_layer);
   g_mutex_unlock (glviewport->layer_lock);
 
+  /* Then regenerate each drawable of each layer */
+  regenerate_layer (far_layer);
+  regenerate_layer (middle_layer);
+  regenerate_layer (near_layer);
+
   task_any_free (task);
 }
 
@@ -537,8 +591,6 @@ changed_cb (PgmDrawable *drawable,
   PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (data);
   PgmGlViewport *glviewport = gldrawable->glviewport;
 
-  GST_LOG_OBJECT (glviewport, "drawable_changed_cb");
-
   if (glviewport)
     {
       Task *task;
@@ -566,8 +618,6 @@ drawable_added_cb (PgmCanvas *canvas,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glviewport, "drawable_added_cb");
-
   /* Add the add task in the update queue */
   task = task_add_new (drawable, layer, order);
   g_mutex_lock (glviewport->update_lock);
@@ -588,8 +638,6 @@ drawable_removed_cb (PgmCanvas *canvas,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glviewport, "drawable_removed_cb");
-
   /* Add the remove task in the update queue */
   task = task_remove_new (drawable, layer);
   g_mutex_lock (glviewport->update_lock);
@@ -611,8 +659,6 @@ drawable_reordered_cb (PgmCanvas *canvas,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glviewport, "drawable_reordered_cb");
-
   /* Add the reorder task in the update queue */
   task = task_reorder_new (drawable, layer, order);
   g_mutex_lock (glviewport->update_lock);
@@ -631,8 +677,6 @@ regenerated_cb (PgmCanvas *canvas,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glviewport, "drawable_regenerated_cb");
-
   /* Add the regenerate task in the update queue */
   task = task_any_new (TASK_REGENERATE);
   g_mutex_lock (glviewport->update_lock);
@@ -710,6 +754,23 @@ sync_gl_drawable_from_canvas (PgmGlViewport *glviewport,
     }
 }
 
+/* Update a layer projection */
+static void
+update_layer_projection (GList *layer)
+{
+  GList *walk = layer;
+
+  while (walk)
+    {
+      PgmGlDrawable *gldrawable = (PgmGlDrawable*) walk->data;
+      pgm_gl_drawable_update_projection (gldrawable);
+      g_object_unref (gldrawable);
+      walk = walk->next;
+    }
+
+  g_list_free (layer);
+}
+
 /* PgmViewport methods */
 
 static PgmError
@@ -719,8 +780,6 @@ pgm_gl_viewport_set_title (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_title");
-
   task = pgm_context_task_new (PGM_CONTEXT_TITLE, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -733,8 +792,6 @@ pgm_gl_viewport_show (PgmViewport *viewport)
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "show");
-
   task = pgm_context_task_new (PGM_CONTEXT_VISIBILITY, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -747,8 +804,6 @@ pgm_gl_viewport_hide (PgmViewport *viewport)
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "hide");
-
   task = pgm_context_task_new (PGM_CONTEXT_VISIBILITY, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -762,8 +817,6 @@ pgm_gl_viewport_set_decorated (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_decorated");
-
   task = pgm_context_task_new (PGM_CONTEXT_DECORATION, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -777,8 +830,6 @@ pgm_gl_viewport_set_cursor (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_cursor");
-
   task = pgm_context_task_new (PGM_CONTEXT_CURSOR, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -792,8 +843,6 @@ pgm_gl_viewport_set_icon (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_icon");
-
   task = pgm_context_task_new (PGM_CONTEXT_ICON, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -807,8 +856,6 @@ pgm_gl_viewport_set_drag_status (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_drag_status");
-
   glviewport->drag_status = accept;
 
   task = pgm_context_task_new (PGM_CONTEXT_DRAG_STATUS, NULL);
@@ -825,8 +872,6 @@ pgm_gl_viewport_set_size (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_size");
-
   task = pgm_context_task_new (PGM_CONTEXT_SIZE, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -840,8 +885,6 @@ pgm_gl_viewport_set_alpha_blending (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_alpha_blending");
-
   task = pgm_context_task_new (PGM_CONTEXT_ALPHA_BLENDING, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -855,8 +898,6 @@ pgm_gl_viewport_set_opacity (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_opacity");
-
   task = pgm_context_task_new (PGM_CONTEXT_OPACITY, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -870,8 +911,6 @@ pgm_gl_viewport_set_fullscreen (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_fullscreen");
-
   task = pgm_context_task_new (PGM_CONTEXT_FULLSCREEN, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -885,8 +924,6 @@ pgm_gl_viewport_set_iconified (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "set_iconified");
-
   task = pgm_context_task_new (PGM_CONTEXT_ICONIFICATION, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -899,8 +936,6 @@ pgm_gl_viewport_focus (PgmViewport *viewport)
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "present");
-
   task = pgm_context_task_new (PGM_CONTEXT_FOCUS, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -914,8 +949,6 @@ pgm_gl_viewport_get_screen_resolution (PgmViewport *viewport,
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glviewport, "get_screen_resolution");
-
   pgm_backend_get_screen_resolution (glviewport->context->backend, width,
                                      height);
 
@@ -929,8 +962,6 @@ pgm_gl_viewport_get_screen_size_mm (PgmViewport *viewport,
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glviewport, "get_screen_size_mm");
-
   pgm_backend_get_screen_size_mm (glviewport->context->backend, width, height);
 
   return PGM_ERROR_OK;
@@ -942,8 +973,6 @@ pgm_gl_viewport_set_canvas (PgmViewport *viewport,
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
 
-  GST_DEBUG_OBJECT (glviewport, "set_canvas");
-
   /* There's a canvas bound, let's clear associated data */
   if (glviewport->canvas)
     {
@@ -1014,8 +1043,6 @@ pgm_gl_viewport_update_projection (PgmViewport *viewport)
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_DEBUG_OBJECT (glviewport, "update_projection");
-
   GST_OBJECT_LOCK (viewport);
 
   /* Store the projected size  */
@@ -1042,10 +1069,6 @@ pgm_gl_viewport_update_projection (PgmViewport *viewport)
 
   GST_OBJECT_UNLOCK (viewport);
 
-  /* Deduct the pixel boundary offsets */
-  glviewport->pixel_offset_x = glviewport->canvas_w / glviewport->projected_w;
-  glviewport->pixel_offset_y = glviewport->canvas_h / glviewport->projected_h;
-
   /* And queue the projection update task */
   task = pgm_context_task_new (PGM_CONTEXT_PROJECTION, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
@@ -1059,8 +1082,6 @@ pgm_gl_viewport_get_embedding_id (PgmViewport *viewport,
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glviewport, "get_embedding_id");
-
   GST_OBJECT_LOCK (glviewport);
   pgm_backend_get_embedding_id (glviewport->context->backend, embedding_id);
   GST_OBJECT_UNLOCK (glviewport);
@@ -1075,8 +1096,6 @@ pgm_gl_viewport_get_pixel_formats (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   const gchar *variable;
 
-  GST_LOG_OBJECT (glviewport, "get_pixel_formats");
-
   GST_OBJECT_LOCK (glviewport);
 
   /* The classic formats supported by all hardware */
@@ -1111,8 +1130,6 @@ pgm_gl_viewport_get_caps_mask (PgmViewport *viewport,
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glviewport, "get_caps_mask");
-
   GST_OBJECT_LOCK (glviewport);
   *caps_mask = glviewport->capacities;
   GST_OBJECT_UNLOCK (glviewport);
@@ -1120,14 +1137,25 @@ pgm_gl_viewport_get_caps_mask (PgmViewport *viewport,
   return PGM_ERROR_OK;
 }
 
+static PgmError
+pgm_gl_viewport_get_max_texture_size (PgmViewport *viewport,
+                                      guint32 *max_texture_size)
+{
+  PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
+
+  GST_OBJECT_LOCK (glviewport);
+  *max_texture_size = glviewport->context->max_texture_2d_size;
+  GST_OBJECT_UNLOCK (glviewport);
+
+  return PGM_ERROR_OK;
+}
+
 static PgmError
 pgm_gl_viewport_get_frame_rate (PgmViewport *viewport,
                                 guint *frame_rate)
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glviewport, "get_frame_rate");
-
   GST_OBJECT_LOCK (glviewport);
   *frame_rate = glviewport->context->fps;
   GST_OBJECT_UNLOCK (glviewport);
@@ -1142,8 +1170,6 @@ pgm_gl_viewport_set_message_filter (PgmViewport *viewport,
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (viewport);
   PgmContextTask *task;
 
-  GST_LOG_OBJECT (glviewport, "set_message_filter");
-
   task = pgm_context_task_new (PGM_CONTEXT_MESSAGE_FILTER, NULL);
   pgm_context_push_immediate_task (glviewport->context, task);
 
@@ -1162,8 +1188,6 @@ pgm_gl_viewport_read_pixels (PgmViewport *viewport,
   PgmGlViewportPixelRectangle *rectangle = NULL;
   PgmContextTask *task = NULL;
 
-  GST_DEBUG_OBJECT (glviewport, "read_pixels");
-
   rectangle = g_slice_new (PgmGlViewportPixelRectangle);
   if (!rectangle)
     return PGM_ERROR_X;
@@ -1182,7 +1206,7 @@ pgm_gl_viewport_read_pixels (PgmViewport *viewport,
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlViewport, pgm_gl_viewport, PGM_TYPE_VIEWPORT);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlViewport, pgm_gl_viewport, PGM_TYPE_VIEWPORT)
 
 void
 pgm_gl_viewport_register (GTypeModule *module)
@@ -1194,9 +1218,6 @@ static void
 pgm_gl_viewport_dispose (GObject *object)
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (object);
-  PgmContextTask *task;
-
-  GST_DEBUG_OBJECT (glviewport, "dispose");
 
   if (glviewport->canvas)
     {
@@ -1224,8 +1245,6 @@ pgm_gl_viewport_dispose (GObject *object)
   glviewport->update_queue = NULL;
   g_mutex_unlock (glviewport->update_lock);
 
-  task = pgm_context_task_new (PGM_CONTEXT_QUIT, NULL);
-  pgm_context_push_immediate_task (glviewport->context, task);
   pgm_context_free (glviewport->context);
 
   g_mutex_free (glviewport->layer_lock);
@@ -1284,6 +1303,8 @@ pgm_gl_viewport_class_init (PgmGlViewportClass *klass)
     GST_DEBUG_FUNCPTR (pgm_gl_viewport_get_pixel_formats);
   viewport_class->get_caps_mask =
     GST_DEBUG_FUNCPTR (pgm_gl_viewport_get_caps_mask);
+  viewport_class->get_max_texture_size =
+    GST_DEBUG_FUNCPTR (pgm_gl_viewport_get_max_texture_size);
   viewport_class->get_frame_rate =
     GST_DEBUG_FUNCPTR (pgm_gl_viewport_get_frame_rate);
   viewport_class->set_message_filter =
@@ -1371,12 +1392,22 @@ pgm_gl_viewport_class_init (PgmGlViewportClass *klass)
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_variant);
   klass->changed_func[PGM_TEXT_WEIGHT] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_weight);
+  klass->changed_func[PGM_TEXT_MULTILINE] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_multiline);
   klass->changed_func[PGM_TEXT_LINE_SPACING] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_line_spacing);
+  klass->changed_func[PGM_TEXT_COLOR] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_color);
   klass->changed_func[PGM_TEXT_OUTLINE_COLOR] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_outline_color);
   klass->changed_func[PGM_TEXT_OUTLINE_WIDTH] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_outline_width);
+  klass->changed_func[PGM_TEXT_CAST_SHADOW] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_cast_shadow);
+  klass->changed_func[PGM_TEXT_SHADOW_POSITION] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_shadow_position);
+  klass->changed_func[PGM_TEXT_SHADOW_COLOR] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_text_set_shadow_color);
 }
 
 static void
@@ -1388,19 +1419,17 @@ pgm_gl_viewport_class_finalize (PgmGlViewportClass *klass)
 static void
 pgm_gl_viewport_init (PgmGlViewport *glviewport)
 {
-  GST_DEBUG_OBJECT (glviewport, "init");
-
   /* Attached canvas */
-  glviewport->canvas = NULL;
+  /* glviewport->canvas = NULL; */
 
   /* Empty lists */
-  glviewport->far_layer = NULL;
-  glviewport->middle_layer = NULL;
-  glviewport->near_layer = NULL;
+  /* glviewport->far_layer = NULL; */
+  /* glviewport->middle_layer = NULL; */
+  /* glviewport->near_layer = NULL; */
   glviewport->layer_lock = g_mutex_new ();
 
   /* Update task queue */
-  glviewport->update_queue = NULL;
+  /* glviewport->update_queue = NULL; */
   glviewport->update_lock = g_mutex_new ();
 
   /* Create the drawable hash using pointers as keys */
@@ -1409,16 +1438,20 @@ pgm_gl_viewport_init (PgmGlViewport *glviewport)
   /* Create context */
   glviewport->context = pgm_context_new (glviewport);
 
+  /* Signal handlers id */
+  /* glviewport->add_handler = 0; */
+  /* glviewport->remove_handler = 0; */
+  /* glviewport->reorder_handler = 0; */
+  /* glviewport->regenerated_handler = 0; */
+
   /* Projection informations */
   glviewport->projected_w = 800;
   glviewport->projected_h = 600;
   glviewport->canvas_w = 4.0f;
   glviewport->canvas_h = 3.0f;
-  glviewport->pixel_offset_x = 0.005f;
-  glviewport->pixel_offset_y = 0.0005f;
 
   /* Fill capacities */
-  glviewport->capacities = 0;
+  /* glviewport->capacities = 0; */
   if (pgm_backend_is_accelerated (glviewport->context->backend))
     glviewport->capacities |= PGM_VIEWPORT_HARDWARE_ACCELERATION;
   if (pgm_backend_is_embeddable (glviewport->context->backend))
@@ -1433,7 +1466,7 @@ pgm_gl_viewport_init (PgmGlViewport *glviewport)
 #endif /* HAVE_GL_GLX_H */
 
   /* Drag status */
-  glviewport->drag_status = FALSE;
+  /* glviewport->drag_status = FALSE; */
 }
 
 /* Public methods */
@@ -1477,17 +1510,19 @@ pgm_gl_viewport_flush_update_queue (PgmGlViewport *glviewport)
 void
 pgm_gl_viewport_update_drawable_projection (PgmGlViewport *glviewport)
 {
+  GList *near_layer, *middle_layer, *far_layer;
+
+  /* Get copies of the layers */
   g_mutex_lock (glviewport->layer_lock);
-
-  /* Adapt position and size of all the drawables */
-  g_list_foreach (glviewport->near_layer,
-                  (GFunc) pgm_gl_drawable_update_projection, NULL);
-  g_list_foreach (glviewport->middle_layer,
-                  (GFunc) pgm_gl_drawable_update_projection, NULL);
-  g_list_foreach (glviewport->far_layer,
-                  (GFunc) pgm_gl_drawable_update_projection, NULL);
-
+  near_layer = copy_layer (glviewport->near_layer);
+  middle_layer = copy_layer (glviewport->middle_layer);
+  far_layer = copy_layer (glviewport->far_layer);
   g_mutex_unlock (glviewport->layer_lock);
+
+  /* Then update the projection of each drawable of each layer */
+  update_layer_projection (far_layer);
+  update_layer_projection (middle_layer);
+  update_layer_projection (near_layer);
 }
 
 /* Connects the GL drawable to the "changed" signal. Should be called at the
