@@ -244,7 +244,7 @@ static inline gpointer
 get_proc_address (PgmGlxBackend *glxbackend,
                   const gchar *proc_name)
 {
-  return glXGetProcAddressARB ((const guchar *) proc_name);
+  return (gpointer) glXGetProcAddressARB ((const guchar *) proc_name);
 }
 
 /* Check whether an extension is supported by the GLX implementation given
@@ -486,7 +486,7 @@ get_framebuffer_configuration (PgmGlxBackend *glxbackend,
 
               if (format->direct.alphaMask > 0)
                 {
-                  GST_INFO_OBJECT (glxbackend, "found an ARGB visual", _visual);
+                  GST_INFO_OBJECT (glxbackend, "found an ARGB visual");
                   glxbackend->feature_mask |= PGM_GLX_FEAT_ARGB_VISUAL;
                   break;
                 }
@@ -632,8 +632,8 @@ setup_vblank (PgmGlxBackend *glxbackend)
    */
   if (glxbackend->feature_mask & PGM_GLX_FEAT_VIDEO_SYNC)
     {
-      pgm_gl_get_string get_string = get_proc_address
-        (glxbackend, "glGetString");
+      pgm_gl_get_string get_string = (pgm_gl_get_string)
+        get_proc_address (glxbackend, "glGetString");
       const gchar *renderer = (const gchar*) get_string (GL_RENDERER);
 
       if (!strncmp ("GeForce 8", renderer, 9)
@@ -910,9 +910,9 @@ handle_xdnd_enter_event (PgmGlxBackend *glxbackend,
   gchar *data_type;
   guint i;
 
-  GST_DEBUG_OBJECT (glxbackend, "initiated xdnd protocol %d with window 0x%x",
+  GST_DEBUG_OBJECT (glxbackend, "initiated xdnd protocol %lu with window 0x%x",
                     (xevent->xclient.data.l[1] & 0xff000000) >> 24,
-                    xevent->xclient.data.l[0]);
+                    (guint) xevent->xclient.data.l[0]);
 
   /* Search for the "text/uri-list" data type in the three available one */
   for (i = 2; i < 5; i++)
@@ -1436,8 +1436,6 @@ dispatch_x_events (PgmBackend *backend)
   PgmGlViewport *glviewport = backend->context->glviewport;
   PgmViewport *viewport = PGM_VIEWPORT (glviewport);
 
-  GST_LOG_OBJECT (glxbackend, "dispatch_x_events");
-
   if (!XPending (glxbackend->dpy))
     return TRUE;
 
@@ -1463,8 +1461,8 @@ dispatch_x_events (PgmBackend *backend)
           /* --- Key press event --- */
         case KeyPress:
           GST_DEBUG_OBJECT (glxbackend, "KeyPress event: (time=%u, state=0x%x, "
-                            "keycode=0x%x)", xevent.xkey.time, xevent.xkey.state,
-                            xevent.xkey.keycode);
+                            "keycode=0x%x)", (guint) xevent.xkey.time,
+                            xevent.xkey.state, xevent.xkey.keycode);
 
           pgmevent = pgm_event_new (PGM_KEY_PRESS);
           translate_key_event (glxbackend, (PgmEventKey *) pgmevent, &xevent);
@@ -1496,7 +1494,8 @@ dispatch_x_events (PgmBackend *backend)
           /* --- Key release event --- */
         case KeyRelease:
           GST_DEBUG_OBJECT (glxbackend, "KeyRelease event: (time=%u, "
-                            "state=0x%x, keycode=0x%x)", xevent.xkey.time,
+                            "state=0x%x, keycode=0x%x)",
+                            (guint) xevent.xkey.time,
                             xevent.xkey.state, xevent.xkey.keycode);
 
           pgmevent = pgm_event_new (PGM_KEY_RELEASE);
@@ -1507,7 +1506,7 @@ dispatch_x_events (PgmBackend *backend)
           /* --- Button press event --- */
         case ButtonPress:
           GST_DEBUG_OBJECT (glxbackend, "ButtonPress event: (time=%u, "
-                            "button=%d x=%d y=%d)", xevent.xbutton.time,
+                            "button=%d x=%d y=%d)", (guint) xevent.xbutton.time,
                             xevent.xbutton.button, xevent.xbutton.x,
                             xevent.xbutton.y);
 
@@ -1539,7 +1538,7 @@ dispatch_x_events (PgmBackend *backend)
           /* --- Button release event --- */
         case ButtonRelease:
           GST_DEBUG_OBJECT (glxbackend, "ButtonRelease event: (time=%u, "
-                            "button=%d x=%d y=%d)", xevent.xbutton.time,
+                            "button=%d x=%d y=%d)", (guint) xevent.xbutton.time,
                             xevent.xbutton.button, xevent.xbutton.x,
                             xevent.xbutton.y);
 
@@ -1556,8 +1555,8 @@ dispatch_x_events (PgmBackend *backend)
           /* --- Motion notify event --- */
         case MotionNotify:
           GST_DEBUG_OBJECT (glxbackend, "MotionNotify event: (time=%u, x=%d, "
-                            "y=%d)", xevent.xmotion.time, xevent.xmotion.x,
-                            xevent.xmotion.y);
+                            "y=%d)", (guint) xevent.xmotion.time,
+                            xevent.xmotion.x, xevent.xmotion.y);
 
           compress_events (glxbackend, &xevent);
           pgmevent = pgm_event_new (PGM_MOTION_NOTIFY);
@@ -1622,7 +1621,8 @@ dispatch_x_events (PgmBackend *backend)
           /* --- Selection notify event --- */
         case SelectionNotify:
           GST_DEBUG_OBJECT (glxbackend, "SelectionNotify event: "
-                            "(requestor=0x%x)", xevent.xselection.requestor);
+                            "(requestor=0x%x)",
+                            (guint) xevent.xselection.requestor);
 
           glxbackend->drag_uri = get_xdnd_uri_list (glxbackend, &xevent);
           if (glxbackend->drag_uri)
@@ -1801,8 +1801,6 @@ pgm_glx_backend_create_window (PgmBackend *backend)
   PgmViewport *viewport;
   gint width, height;
 
-  GST_DEBUG_OBJECT (glxbackend, "create_window");
-
   viewport = PGM_VIEWPORT (backend->context->glviewport);
 
   glxbackend->dpy = XOpenDisplay (NULL);
@@ -1846,7 +1844,7 @@ pgm_glx_backend_create_window (PgmBackend *backend)
       GST_ERROR_OBJECT (glxbackend, "couldn't create window");
       return FALSE;
     }
-  GST_INFO_OBJECT (glxbackend, "created window (id 0x%x)", glxbackend->win);
+  GST_INFO_OBJECT (glxbackend, "created window (id 0x%x)", (guint) glxbackend->win);
 
   /* Event source */
   if (!add_event_source (glxbackend))
@@ -1932,9 +1930,6 @@ pgm_glx_backend_destroy_window (PgmBackend *backend)
 
   if (glxbackend->created)
     {
-      /* Remove event handling source */
-      g_source_remove (glxbackend->event_id);
-
       /* Clean up X related data */
       XFreeCursor (glxbackend->dpy, glxbackend->none_cursor);
       XFree (glxbackend->fbconfig);
@@ -1957,8 +1952,6 @@ pgm_glx_backend_set_title (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_DEBUG_OBJECT (glxbackend, "set_title");
-
   XChangeProperty (glxbackend->dpy, glxbackend->win,
                    glxbackend->atom[ATOM_NET_WM_NAME],
                    glxbackend->atom[ATOM_UTF8_STRING], 8, PropModeReplace,
@@ -1973,8 +1966,6 @@ pgm_glx_backend_set_visibility (PgmBackend *backend,
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
   PgmViewport *viewport = PGM_VIEWPORT (backend->context->glviewport);
 
-  GST_DEBUG_OBJECT (glxbackend, "set_visibility");
-
   if (visible)
     {
       XMapWindow (glxbackend->dpy, glxbackend->win);
@@ -2009,8 +2000,6 @@ pgm_glx_backend_set_iconified (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_DEBUG_OBJECT (glxbackend, "set_iconified");
-
   if (iconified)
     {
       if (glxbackend->mapped)
@@ -2043,8 +2032,6 @@ pgm_glx_backend_focus (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (glxbackend, "focus");
-
   if (!glxbackend->mapped)
     pgm_glx_backend_set_visibility (backend, TRUE);
 
@@ -2062,8 +2049,6 @@ pgm_glx_backend_set_decorated (PgmBackend *backend,
   gint format;
   Atom type;
 
-  GST_DEBUG_OBJECT (glxbackend, "set_decorated");
-
   /* Create the new hints */
   memset (&new_hints, 0, sizeof (new_hints));
   new_hints.flags = MWM_HINTS_DECORATIONS;
@@ -2108,8 +2093,6 @@ pgm_glx_backend_swap_buffers (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (glxbackend, "swap_buffers");
-
   glXSwapBuffers (glxbackend->dpy, glxbackend->win);
 }
 
@@ -2129,8 +2112,6 @@ pgm_glx_backend_set_size (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_DEBUG_OBJECT (glxbackend, "set_size");
-
   XResizeWindow (glxbackend->dpy, glxbackend->win, width, height);
   XSync (glxbackend->dpy, False);
 
@@ -2143,8 +2124,6 @@ pgm_glx_backend_set_fullscreen (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_DEBUG_OBJECT (glxbackend, "set_fullscreen");
-
   set_fullscreen (glxbackend, fullscreen);
   glxbackend->fullscreen = fullscreen;
 
@@ -2158,8 +2137,6 @@ pgm_glx_backend_get_screen_size_mm (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (glxbackend, "get_screen_size_mm");
-
   *width = glxbackend->size_mm_width;
   *height = glxbackend->size_mm_height;
 }
@@ -2169,12 +2146,6 @@ pgm_glx_backend_set_screen_resolution (PgmBackend *backend,
                                        gint width,
                                        gint height)
 {
-  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
-
-  GST_DEBUG_OBJECT (glxbackend, "set_screen_resolution");
-
-  /* FIXME */
-
   return FALSE;
 }
 
@@ -2185,8 +2156,6 @@ pgm_glx_backend_get_screen_resolution (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (glxbackend, "get_screen_resolution");
-
   *width = glxbackend->resolution_width;
   *height = glxbackend->resolution_height;
 }
@@ -2200,8 +2169,6 @@ pgm_glx_backend_build_text_lists (PgmBackend *backend)
   gint first;
   gint last;
 
-  GST_DEBUG_OBJECT (glxbackend, "build_text_lists");
-
   glxbackend->text_lists = gl->gen_lists (256);
   if (!gl->is_list (glxbackend->text_lists))
     {
@@ -2237,8 +2204,6 @@ pgm_glx_backend_destroy_text_lists (PgmBackend *backend)
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
   PgmContextProcAddress *gl = backend->context->gl;
 
-  GST_DEBUG_OBJECT (glxbackend, "destroy_text_lists");
-
   if (gl->is_list (glxbackend->text_lists))
     gl->delete_lists (glxbackend->text_lists, 256);
 
@@ -2257,8 +2222,6 @@ pgm_glx_backend_raster_text (PgmBackend *backend,
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
   PgmContextProcAddress *gl = backend->context->gl;
 
-  GST_DEBUG_OBJECT (glxbackend, "raster_text");
-
   gl->load_identity ();
   gl->bind_texture (PGM_GL_TEXTURE_2D, 0);
   gl->push_attrib (PGM_GL_LIST_BIT);
@@ -2274,8 +2237,6 @@ pgm_glx_backend_wait_for_vblank (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (glxbackend, "wait_for_vblank");
-
   if (glxbackend->vblank_mode == PGM_VBLANK_VIDEO_SYNC)
     {
       PgmGlxBackendProcAddress *glx = glxbackend->glx;
@@ -2296,8 +2257,6 @@ pgm_glx_backend_notify_startup_complete (PgmBackend *backend)
   XClientMessageEvent xclient;
   Window win;
 
-  GST_LOG_OBJECT (glxbackend, "notify_startup_complete");
-
   if (!glxbackend->startup_notification_id)
     return;
 
@@ -2360,8 +2319,6 @@ pgm_glx_backend_set_cursor (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_DEBUG_OBJECT (glxbackend, "set_cursor");
-
   switch (cursor)
     {
     case PGM_VIEWPORT_LEFT_ARROW:
@@ -2396,8 +2353,6 @@ pgm_glx_backend_set_icon (PgmBackend *backend,
   Pixmap icon_pixmap = None;
   Pixmap icon_mask = None;
 
-  GST_DEBUG_OBJECT (glxbackend, "set_icon");
-
   if (icon)
     gdk_pixbuf_xlib_render_pixmap_and_mask (icon, &icon_pixmap, &icon_mask, 128);
   else
@@ -2424,8 +2379,6 @@ pgm_glx_backend_set_drag_status (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (backend, "set_drag_status");
-
   /* Avoid useless computations */
   if (accept == glxbackend->drag_status)
     return;
@@ -2443,16 +2396,12 @@ pgm_glx_backend_is_accelerated (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (backend, "is_accelerated");
-
   return glxbackend->feature_mask & PGM_GLX_FEAT_DIRECT_RENDERING;
 }
 
 static gboolean
 pgm_glx_backend_is_embeddable (PgmBackend *backend)
 {
-  GST_LOG_OBJECT (backend, "is_embeddable");
-
   /* Supported through the XEmbed implementation */
   return TRUE;
 }
@@ -2463,8 +2412,6 @@ pgm_glx_backend_get_embedding_id (PgmBackend *backend,
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (backend, "get_embedding_id");
-
   *embedding_id = glxbackend->win;
 }
 
@@ -2473,8 +2420,6 @@ pgm_glx_backend_has_alpha_component (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (backend, "has_alpha_component");
-
   return glxbackend->feature_mask & PGM_GLX_FEAT_ARGB_VISUAL;
 }
 
@@ -2483,8 +2428,6 @@ pgm_glx_backend_has_system_buffer (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
 
-  GST_LOG_OBJECT (backend, "has_system_buffer");
-
   return glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP;
 }
 
@@ -2505,8 +2448,6 @@ pgm_glx_backend_create_system_buffer_object (PgmBackend *backend,
     None
   };
 
-  GST_LOG_OBJECT (glxbackend, "create_system_buffer_object");
-
   if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
     {
       GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
@@ -2553,8 +2494,6 @@ pgm_glx_backend_destroy_system_buffer_object (PgmBackend *backend,
 
   g_return_if_fail (system_buffer_object);
 
-  GST_LOG_OBJECT (glxbackend, "destroy_system_buffer_object");
-
   if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
     {
       GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
@@ -2576,8 +2515,6 @@ pgm_glx_backend_bind_system_buffer_object (PgmBackend *backend,
 
   g_return_if_fail (system_buffer_object);
 
-  GST_LOG_OBJECT (glxbackend, "bind_system_buffer_object");
-
   if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
     {
       GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
@@ -2598,8 +2535,6 @@ pgm_glx_backend_release_system_buffer_object (PgmBackend *backend,
 
   g_return_if_fail (system_buffer_object);
 
-  GST_LOG_OBJECT (glxbackend, "release_system_buffer_object");
-
   if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
     {
       GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
@@ -2614,7 +2549,7 @@ pgm_glx_backend_release_system_buffer_object (PgmBackend *backend,
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlxBackend, pgm_glx_backend, PGM_TYPE_BACKEND);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlxBackend, pgm_glx_backend, PGM_TYPE_BACKEND)
 
 void
 pgm_glx_backend_register (GTypeModule *module)
@@ -2627,8 +2562,6 @@ pgm_glx_backend_dispose (GObject *object)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (object);
 
-  GST_DEBUG_OBJECT (glxbackend, "dispose");
-
   pgm_glx_backend_destroy_text_lists (PGM_BACKEND (glxbackend));
 
   if (glxbackend->created)
@@ -2731,8 +2664,6 @@ pgm_glx_backend_class_finalize (PgmGlxBackendClass *klass)
 static void
 pgm_glx_backend_init (PgmGlxBackend *glxbackend)
 {
-  GST_DEBUG_OBJECT (glxbackend, "init");
-
   glxbackend->dpy = NULL;
   glxbackend->glx = NULL;
   glxbackend->visual = NULL;
