@@ -109,22 +109,6 @@ static const gchar *per_plane_i420_yv12_rgb_string =
   "MAD color.rgb, { 0, -0.391, 2.018 }, tmp.rrra, color;"
   "MAD color.rgb, { 1.596, -0.813, 0 }, tmp.ggga, color;";
 
-/* Program dedicated to the DRI R300 driver which doesn't take in account the
- * writemask during a texture lookup. A patch has been pushed the 7 Oct 2007
- * in the Mesa git repository:
- * http://gitweb.freedesktop.org/?p=mesa/mesa.git;a=commit;h=32699696e31234c8d4e4b08f255ba2134ec12db5
- */
-static const gchar *per_plane_i420_yv12_rgb_r300_string =
-  "TEMP color, cb, cr;"
-  "TEX color, position, texture[0], 2D;"
-  "MAD color, color, 1.164, -0.073;"
-  "TEX cb, position, texture[%s], 2D;"
-  "TEX cr, position, texture[%s], 2D;"
-  "SUB cb.r, cb, 0.5;"
-  "SUB cr.r, cr, 0.5;"
-  "MAD color.rgb, { 0, -0.391, 2.018 }, cb.rrra, color;"
-  "MAD color.rgb, { 1.596, -0.813, 0 }, cr.rrra, color;";
-
 /* Combine the color with the primary vertex color */
 static const gchar *primary_color_combine_string =
   "MUL result.color, fragment.color, color;";
@@ -291,14 +275,6 @@ pgm_program_create (PgmContext *context)
 
   if (context->feature_mask & PGM_GL_FEAT_PER_PLANE_YCBCR_PROGRAM)
     {
-      const gchar *_per_plane_i420_yv12_rgb_string = NULL;
-
-      /* Hack to workaround the bug commented above with the DRI R300 driver */
-      if (strncmp ("Mesa DRI R300", context->renderer, 13) == 0)
-        _per_plane_i420_yv12_rgb_string = per_plane_i420_yv12_rgb_r300_string;
-      else
-        _per_plane_i420_yv12_rgb_string = per_plane_i420_yv12_rgb_string;
-
       /* Program converting from per plane I420 to RGB */
       per_plane_i420_rgb = new_program ();
       if (per_plane_i420_rgb)
@@ -306,7 +282,7 @@ pgm_program_create (PgmContext *context)
           p = buffer;
           p += sprintf (p, PROGRAM_HEADER);
           p += sprintf (p, "%s", per_plane_i420_yv12_rgb_header_string);
-          p += sprintf (p, _per_plane_i420_yv12_rgb_string, "1", "2");
+          p += sprintf (p, per_plane_i420_yv12_rgb_string, "1", "2");
           p += sprintf (p, "%s", primary_color_combine_string);
           sprintf (p, PROGRAM_FOOTER);
 
@@ -327,7 +303,7 @@ pgm_program_create (PgmContext *context)
           p = buffer;
           p += sprintf (p, PROGRAM_HEADER);
           p += sprintf (p, "%s", per_plane_i420_yv12_rgb_header_string);
-          p += sprintf (p, _per_plane_i420_yv12_rgb_string, "2", "1");
+          p += sprintf (p, per_plane_i420_yv12_rgb_string, "2", "1");
           p += sprintf (p, "%s", primary_color_combine_string);
           sprintf (p, PROGRAM_FOOTER);
 
