@@ -453,7 +453,11 @@ free_buffer (PgmTexture *texture)
 
     case PGM_TEXTURE_BUFFER:
       if (texture->data.buffer)
-        texture->data.buffer = NULL;
+        {
+          if (!texture->shared)
+            g_free (texture->data.buffer);
+          texture->data.buffer = NULL;
+        }
       break;
 
     case PGM_TEXTURE_SYSTEM_BUFFER:
@@ -502,6 +506,8 @@ init_texture (PgmTexture *texture)
   /* Indentifiants */
   texture->count = 0;
   texture->id = NULL;
+
+  texture->shared = 0;
 }
 
 /* Destructor */
@@ -600,12 +606,21 @@ pgm_texture_set_buffer (PgmTexture *texture,
                         guint width,
                         guint height,
                         guint size,
-                        guint stride)
+                        guint stride,
+                        gboolean share)
 {
   free_buffer (texture);
-
   texture->storage = PGM_TEXTURE_BUFFER;
-  texture->data.buffer = buffer;
+  if (share)
+    {
+      texture->data.buffer = buffer;
+      texture->shared = 1;
+    }
+  else
+    {
+      texture->data.buffer = g_memdup (buffer, size);
+      texture->shared = 0;
+    }
   texture->width = width;
   texture->height = height;
   texture->stride = stride;
