@@ -101,6 +101,9 @@ struct _PgmTexture {
   guint cbcr_height;
   guint y_stride, cbcr_stride;
   guint cb_offset, cr_offset;
+
+  /* Whether the buffer is shared */
+  guint8 shared;
 };
 
 /* create a new texture */
@@ -116,7 +119,8 @@ void        pgm_texture_set_buffer        (PgmTexture *texture,
                                            guint width,
                                            guint height,
                                            guint size,
-                                           guint stride);
+                                           guint stride,
+                                           gboolean share);
 
 /* Set a system buffer */
 void        pgm_texture_set_system_buffer (PgmTexture *texture,
