@@ -58,7 +58,6 @@
 #include <fcntl.h>    /* _O_BINARY */
 #include <io.h>       /* _pipe, _close */
 #define pipe(handle)  _pipe(handle, 4096, _O_BINARY)
-#define close(handle) _close(handle)
 #endif /* WIN32 */
 
 #include <GLES/gl.h>  /* gl* */
@@ -159,8 +158,12 @@ update_projection (PgmGlesContext *glescontext)
   projection = viewport->projection;
   GST_OBJECT_UNLOCK (viewport);
 
+#if HAVE_GDL
+  gles->viewport (projected_x, projected_y, viewport->width, viewport->height);
+#else
   /* Set up viewport */
   gles->viewport (projected_x, projected_y, projected_width, projected_height);
+#endif
 
   /* Update projection matrix */
   gles->matrix_mode (PGM_GLES_PROJECTION);
@@ -263,6 +266,22 @@ do_projection (PgmGlesContext *glescontext,
   pgm_gles_context_update (glescontext);
 }
 
+/* Screen resolution changes */
+static void
+do_resolution (PgmGlesContext *context,
+               gpointer data)
+{
+  gint width, height;
+
+  GST_OBJECT_LOCK (context->glesviewport);
+  width = PGM_VIEWPORT (context->glesviewport)->width;
+  height = PGM_VIEWPORT (context->glesviewport)->height;
+  GST_OBJECT_UNLOCK (context->glesviewport);
+
+  pgm_gles_backend_set_screen_resolution (context->backend, width, height);
+  pgm_gles_context_update (context);
+}
+
 /* Alpha blending update task handler */
 static void
 do_alpha_blending (PgmGlesContext *glescontext,
@@ -387,22 +406,72 @@ do_free_texture (PgmGlesContext *glescontext,
   pgm_gles_texture_free (PGM_GLES_TEXTURE_CAST (data));
 }
 
-/* Traverse the layers and draw the drawables */
+/* Copy a layer */
+static GList*
+copy_layer (GList *layer)
+{
+  GList *new_layer = NULL;
+
+  if (layer)
+    {
+      GList *last;
+
+      new_layer = g_slice_new (GList);
+      new_layer->data = g_object_ref (layer->data);
+      new_layer->prev = NULL;
+      last = new_layer;
+      layer = layer->next;
+
+      while (layer)
+        {
+          last->next = g_slice_new (GList);
+          last->next->prev = last;
+          last = last->next;
+          last->data = g_object_ref (layer->data);
+          layer = layer->next;
+        }
+
+      last->next = NULL;
+    }
+
+  return new_layer;
+}
+
+/* Render a layer */
+static void
+render_layer (GList *layer)
+{
+  GList *walk = layer;
+
+  while (walk)
+    {
+      PgmGlesDrawable *glesdrawable = (PgmGlesDrawable*) walk->data;
+      pgm_gles_drawable_draw (glesdrawable);
+      g_object_unref (glesdrawable);
+      walk = walk->next;
+    }
+
+  g_list_free (layer);
+}
+
+/* Traverse the layers and draw each drawable */
 static void
 render (PgmGlesContext *glescontext)
 {
+  GList *near_layer, *middle_layer, *far_layer;
   PgmGlesViewport *glesviewport = glescontext->glesviewport;
 
+  /* Get copies of the layers */
   g_mutex_lock (glesviewport->layer_lock);
-
-  g_list_foreach (glesviewport->far_layer,
-                  (GFunc) pgm_gles_drawable_draw, NULL);
-  g_list_foreach (glesviewport->middle_layer,
-                  (GFunc) pgm_gles_drawable_draw, NULL);
-  g_list_foreach (glesviewport->near_layer,
-                  (GFunc) pgm_gles_drawable_draw, NULL);
-
+  near_layer = copy_layer (glesviewport->near_layer);
+  middle_layer = copy_layer (glesviewport->middle_layer);
+  far_layer = copy_layer (glesviewport->far_layer);
   g_mutex_unlock (glesviewport->layer_lock);
+
+  /* Then render each drawable of each layer */
+  render_layer (far_layer);
+  render_layer (middle_layer);
+  render_layer (near_layer);
 }
 
 /* Push a task in a queue removing a similar one if any */
@@ -553,22 +622,28 @@ update_cb (gpointer data)
   /* User defined update callback */
   pgm_viewport_emit_update_pass (PGM_VIEWPORT (glescontext->glesviewport));
 
-  /* Flush update queue */
-  pgm_gles_viewport_flush_update_queue (glescontext->glesviewport);
+  /* Check if the viewport has not been disposed by an update pass handler */
+  if (G_LIKELY (glescontext->glesviewport))
+    {
+      /* Flush update queue */
+      pgm_gles_viewport_flush_update_queue (glescontext->glesviewport);
 
-  /* Flush rendering queues */
-  flush_task_queue (glescontext, &glescontext->immediate_task);
-  flush_task_queue (glescontext, &glescontext->deferred_task);
+      /* Flush rendering queues */
+      flush_task_queue (glescontext, &glescontext->immediate_task);
+      flush_task_queue (glescontext, &glescontext->deferred_task);
 
-  /* Render the scene */
-  render (glescontext);
-  pgm_gles_backend_swap_buffers (glescontext->backend);
-  glescontext->gles->clear (PGM_GLES_COLOR_BUFFER_BIT);
+      /* Render the scene */
+      render (glescontext);
+      pgm_gles_backend_swap_buffers (glescontext->backend);
+      glescontext->gles->clear (PGM_GLES_COLOR_BUFFER_BIT);
 
-  /* Handle frame rate */
-  compute_frame_rate (glescontext);
+      /* Handle frame rate */
+      compute_frame_rate (glescontext);
 
-  return TRUE;
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
 /* Update timeout used to remove the update idle after 1 second of inactivity */
@@ -584,7 +659,7 @@ update_removal_timeout_cb (gpointer data)
 
   /* If the elapsed time since the previous update request rises above one
    * second, remove the auto-update source */
-  if ((current_time.tv_sec - glescontext->update_timestamp.tv_sec) >= 1)
+  if ((current_time.tv_sec - glescontext->update_timestamp.tv_sec) >= 2)
     {
       GSource *source;
 
@@ -623,19 +698,31 @@ create_io_channels (gint *fd,
       return;
     }
 
+#ifdef WIN32
+  *in = g_io_channel_win32_new_fd (fd[1]);
+#else
   *in = g_io_channel_unix_new (fd[1]);
+#endif /* WIN32 */
   if (!*in)
     {
       GST_ERROR ("cannot create the input channel");
       return;
     }
 
+#ifdef WIN32
+  *out = g_io_channel_win32_new_fd (fd[0]);
+#else
   *out = g_io_channel_unix_new (fd[0]);
+#endif /* WIN32 */
   if (!*out)
     {
       GST_ERROR ("cannot create the output channel");
       return;
     }
+
+  /* We don't close the fd ourselves because otherwise it deadlocks on windows */
+  g_io_channel_set_close_on_unref (*in, TRUE);
+  g_io_channel_set_close_on_unref (*out, TRUE);
 }
 
 /* Close a pipe and unref its i/o channels */
@@ -648,14 +735,12 @@ delete_io_channels (gint *fd,
     {
       g_io_channel_unref (*out);
       *out = NULL;
-      close (fd[0]);
     }
 
   if (*in)
     {
       g_io_channel_unref (*in);
       *in = NULL;
-      close (fd[1]);
     }
 }
 
@@ -696,6 +781,7 @@ render_loop (gpointer data)
   PgmGlesContext *glescontext = (PgmGlesContext *) data;
   PgmGlesContextProcAddress *gles = NULL;
   const gchar *env_var = NULL;
+  GSource *source;
   gint i = 0;
 
 #ifndef HAVE_OPENGLES_PC_EMULATION
@@ -766,17 +852,36 @@ render_loop (gpointer data)
   g_cond_signal (glescontext->init_cond);
   g_mutex_unlock (glescontext->init_mutex);
 
-  /* And enter the main loop */
+  /* Enter the main loop */
   g_main_loop_run (glescontext->render_loop);
 
-  /* Flush the queues */
-  flush_task_queue (glescontext, &glescontext->immediate_task);
-  flush_task_queue (glescontext, &glescontext->deferred_task);
+  /* Dispose immediate source data */
+  source = g_main_context_find_source_by_id (glescontext->render_context,
+                                             glescontext->immediate_tag);
+  if (source)
+    g_source_destroy (source);
+  delete_io_channels (glescontext->immediate_fd, &glescontext->immediate_in,
+                      &glescontext->immediate_out);
+  glescontext->immediate_in = NULL;
+  glescontext->immediate_out = NULL;
+
+  /* Free the task queues */
+  free_task_queue (&glescontext->deferred_task);
+  free_task_queue (&glescontext->immediate_task);
 
   /* Clean up rendering thread related data */
   pgm_gles_backend_destroy_window (glescontext->backend);
   gst_object_unref (glescontext->backend);
-  glescontext->backend = NULL;
+
+  /* Signal application thread that the rendering thread is now finalized,
+   * no need to signal if the context is disposed in the rendering thread */
+  if (glescontext->fini_signal)
+    {
+      g_mutex_lock (glescontext->fini_mutex);
+      glescontext->finalized = TRUE;
+      g_cond_signal (glescontext->fini_cond);
+      g_mutex_unlock (glescontext->fini_mutex);
+    }
 
   return NULL;
 }
@@ -800,6 +905,12 @@ init_gles_context (PgmGlesContext *glescontext)
   glescontext->init_cond = g_cond_new ();
   glescontext->initialized = FALSE;
 
+  /* Finalization lock */
+  glescontext->fini_mutex = g_mutex_new ();
+  glescontext->fini_cond = g_cond_new ();
+  glescontext->fini_signal = FALSE;
+  glescontext->finalized = FALSE;
+
   /* Immediate task source */
   glescontext->immediate_fd[0] = -1;
   glescontext->immediate_fd[1] = -1;
@@ -825,8 +936,8 @@ init_gles_context (PgmGlesContext *glescontext)
   glescontext->fps = 0;
 
   /* Create rendering thread */
-  glescontext->render_thread = g_thread_create (render_loop, glescontext,
-                                                TRUE, &error);
+  glescontext->render_thread =
+    g_thread_create (render_loop, glescontext, FALSE, &error);
   if (error != NULL)
     {
       GST_ERROR ("couldn't create rendering thread: %s", error->message);
@@ -842,6 +953,8 @@ init_gles_context (PgmGlesContext *glescontext)
     GST_DEBUG_FUNCPTR (do_alpha_blending);
   glescontext->task_func[PGM_GLES_CONTEXT_READ_PIXELS] =
     GST_DEBUG_FUNCPTR (do_read_pixels);
+  glescontext->task_func[PGM_GLES_CONTEXT_RESOLUTION] =
+    GST_DEBUG_FUNCPTR (do_resolution);
 
   /* Texture related tasks function binding */
   glescontext->task_func[PGM_GLES_CONTEXT_GEN_TEXTURE] =
@@ -864,53 +977,6 @@ init_gles_context (PgmGlesContext *glescontext)
   return TRUE;
 }
 
-/* Dispose context */
-static void
-dispose_gles_context (PgmGlesContext *glescontext)
-{
-  /* Iterate the last events */
-  while (g_main_context_pending (glescontext->render_context))
-    g_main_context_iteration (glescontext->render_context, FALSE);
-
-  /* Remove the immediate source */
-  if (glescontext->immediate_tag)
-    g_source_remove (glescontext->immediate_tag);
-
-  /* Remove the auto-update source */
-  if (glescontext->update_tag)
-    {
-      GSource *source = g_main_context_find_source_by_id
-        (glescontext->render_context, glescontext->update_tag);
-      if (source)
-        g_source_destroy (source);
-    }
-
-  /* Join the rendering thread */
-  g_main_loop_quit (glescontext->render_loop);
-  g_main_loop_unref (glescontext->render_loop);
-  g_main_context_unref (glescontext->render_context);
-  g_thread_join (glescontext->render_thread);
-
-  /* Dispose immediate source data */
-  delete_io_channels (glescontext->immediate_fd, &glescontext->immediate_in,
-                      &glescontext->immediate_out);
-
-  /* Free the task queues */
-  free_task_queue (&glescontext->deferred_task);
-  free_task_queue (&glescontext->immediate_task);
-
-  /* Auto-update lock */
-  g_mutex_free (glescontext->update_mutex);
-
-  /* Initialization lock */
-  g_mutex_free (glescontext->init_mutex);
-  g_cond_free (glescontext->init_cond);
-  glescontext->initialized = FALSE;
-
-  /* Structure access */
-  g_mutex_free (glescontext->mutex);
-}
-
 /* Public functions */
 
 PgmGlesContext *
@@ -938,9 +1004,31 @@ pgm_gles_context_free (PgmGlesContext *glescontext)
 {
   g_return_if_fail (glescontext != NULL);
 
-  dispose_gles_context (glescontext);
-  glescontext = NULL;
+  /* Store whether the context disposition happens in the rendering thread */
+  glescontext->fini_signal = g_thread_self () != glescontext->render_thread;
 
+  /* Quit the rendering loop */
+  g_main_loop_quit (glescontext->render_loop);
+
+  /* Wait for the rendering thread finalization completion if the context
+   * disposition happens in the rendering thread */
+  if (glescontext->fini_signal)
+    {
+      g_mutex_lock (glescontext->fini_mutex);
+      if (!glescontext->finalized)
+        g_cond_wait (glescontext->fini_cond, glescontext->fini_mutex);
+      g_mutex_unlock (glescontext->fini_mutex);
+    }
+
+  /* Free the context */
+  g_main_loop_unref (glescontext->render_loop);
+  g_main_context_unref (glescontext->render_context);
+  g_mutex_free (glescontext->fini_mutex);
+  g_cond_free (glescontext->fini_cond);
+  g_mutex_free (glescontext->update_mutex);
+  g_mutex_free (glescontext->init_mutex);
+  g_cond_free (glescontext->init_cond);
+  g_mutex_free (glescontext->mutex);
   g_slice_free (PgmGlesContext, glescontext);
 }
 
@@ -1003,7 +1091,7 @@ pgm_gles_context_update (PgmGlesContext *glescontext)
       g_source_unref (source);
 
       /* Add the update removal timeout to the rendering GMainContext */
-      source = g_timeout_source_new (500);
+      source = g_timeout_source_new (750);
       g_source_set_callback (source, update_removal_timeout_cb, glescontext,
                              NULL);
       g_source_attach (source, glescontext->render_context);
