@@ -20,16 +20,22 @@
  * Boston, MA 02111-1307, USA.
  *
  * Author: Loïc Molinari <loic@fluendo.com>
- */
-
-/*
- *
+ *         Sergi Alvarez <salvarez@fluendo.com>
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#if HAVE_GDL
+#include <glib/gprintf.h>
+#include "libgdl.h"
+#include "EGL/egl.h"
+#define DEFAULT_WIDTH 1280
+#define DEFAULT_HEIGHT 720
+#endif
+
+#include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 
@@ -312,6 +318,139 @@ remove_event_source (PgmGlesEglBackend *eglbackend)
 #endif /* HAVE_TSLIB */
 
 /* PgmGlesBackend methods */
+#if HAVE_GDL
+#include <signal.h>
+
+/* Global pointer to the last initialization of the GLES EGL backend */
+static PgmGlesEglBackend *gbackend = NULL;
+
+static void
+sigint_handler(int sig)
+{
+  static int sigint_called = 0;
+  g_fprintf (stderr, "\n^C\n");
+  if (sigint_called++)
+    return;
+  if (gbackend != NULL) {
+    eglDestroyContext (gbackend->display, gbackend->context);
+    gbackend->context = NULL;
+    eglDestroySurface (gbackend->display, gbackend->surface);
+    gbackend->surface = NULL;
+    eglMakeCurrent (gbackend->display, EGL_NO_SURFACE, EGL_NO_SURFACE,
+                      EGL_NO_CONTEXT);
+    eglTerminate (gbackend->display);
+    gbackend->display = NULL;
+    gdl_close();
+  }
+  g_fprintf(stderr, "GDL terminated\n");
+  exit(1);
+}
+
+/* GDL get/set tvmode resolution */
+
+static inline gboolean
+gdl_tvmode_get_size (gint * width, gint * height)
+{
+  gboolean ret = FALSE;
+  gdl_display_info_t di;
+  memset (&di, 0, sizeof (di));
+
+  /* Read current hardware state */
+  if (gdl_get_display_info (GDL_DISPLAY_ID_0, &di) != GDL_SUCCESS) {
+    GST_ERROR ("gdl_get_display_info failed");
+    goto beach;
+  }
+
+  *width = di.tvmode.width;
+  *height = di.tvmode.height;
+
+  ret = TRUE;
+beach:
+  return ret;
+}
+
+static int
+initialize_gdl(PgmGlesEglBackend *glesbackend, gdl_plane_id_t plane)
+{
+  PgmGlesBackend *backend = PGM_GLES_BACKEND (glesbackend);
+  PgmGlesViewport *glesviewport = backend->context->glesviewport;
+  gdl_display_info_t di;
+  PgmViewport *viewport = PGM_VIEWPORT (glesviewport);
+  gdl_pixel_format_t pixelFormat = GDL_PF_ARGB_32;
+  gdl_color_space_t colorSpace = GDL_COLOR_SPACE_RGB;
+  gdl_rectangle_t rect;
+  gboolean b_true = TRUE;
+  gdl_ret_t rc = GDL_SUCCESS;
+  gint width, height;
+  int ret = TRUE;
+
+  gdl_init(0);
+  signal(SIGINT, sigint_handler);
+
+  if (!gdl_tvmode_get_size (&width, &height)) {
+    /* Write current hardware state */
+    di.id = GDL_DISPLAY_ID_0;
+    di.gamma = GDL_GAMMA_LINEAR;
+    di.tvmode.refresh = GDL_REFRESH_50;
+    di.tvmode.interlaced = 0;
+    di.tvmode.width = DEFAULT_WIDTH;
+    di.tvmode.height = DEFAULT_HEIGHT;
+    di.color_space = (di.tvmode.width>720) ? 
+      GDL_COLOR_SPACE_BT709 : GDL_COLOR_SPACE_BT601;
+    if (gdl_set_display_info (&di) != GDL_SUCCESS) {
+      GST_ERROR ("gdl_set_display_info failed");
+      goto beach;
+    }
+    gdl_port_set_attr (GDL_PD_ID_HDMI, GDL_PD_ATTR_ID_POWER, &b_true);
+    gdl_port_set_attr (GDL_PD_ID_HDMI, GDL_PD_ATTR_ID_PAR, &b_true);
+
+    if (!gdl_tvmode_get_size (&width, &height))
+      ret = FALSE;
+    if (ret == FALSE) {
+      goto beach;
+    }
+  }
+
+  /* force viewport size to fullscreen */
+  viewport->width = width;
+  viewport->height = height;
+  viewport->projected_width = width;
+  viewport->projected_height = height;
+  glesviewport->projected_w = width;
+  glesviewport->projected_h = height;
+
+  rc = gdl_plane_reset(plane);
+  if (rc == GDL_SUCCESS)
+    rc = gdl_plane_config_begin(plane);
+
+  if (rc == GDL_SUCCESS)
+    rc = gdl_plane_set_attr(GDL_PLANE_SRC_COLOR_SPACE, &colorSpace);
+
+  if (rc == GDL_SUCCESS)
+    rc = gdl_plane_set_attr(GDL_PLANE_PIXEL_FORMAT, &pixelFormat);
+
+  rect.origin.x = 0;
+  rect.origin.y = 0;
+  rect.width = width;
+  rect.height = height;
+  if (rc == GDL_SUCCESS)
+    rc = gdl_plane_set_attr(GDL_PLANE_DST_RECT, &rect);
+
+  if (rc == GDL_SUCCESS)
+    rc = gdl_plane_config_end(GDL_FALSE);
+  else
+    gdl_plane_config_end(GDL_TRUE);
+
+  if (rc != GDL_SUCCESS)
+    {
+      GST_DEBUG_OBJECT (glesbackend,
+        "GDL configuration failed! GDL error code is 0x%x\n", rc);
+      ret = FALSE;
+    }
+beach:
+  return ret;
+}
+#endif
 
 static gboolean
 pgm_gles_egl_backend_create_window (PgmGlesBackend *glesbackend)
@@ -319,35 +458,50 @@ pgm_gles_egl_backend_create_window (PgmGlesBackend *glesbackend)
   PgmGlesEglBackend *eglbackend = PGM_GLES_EGL_BACKEND (glesbackend);
   gint swap_interval = 1;
   const gchar *env_var;
-  EGLConfig config;
-  gint num_configs;
+  EGLConfig config = 0;
+  gint num_configs = 0;
+  gint egl_major, egl_minor;
 
-  gint attrib[] = {
-    EGL_RED_SIZE, 5,
-    EGL_GREEN_SIZE, 6,
-    EGL_BLUE_SIZE, 5,
-    EGL_DEPTH_SIZE, 0,
-    EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
-    EGL_NONE };
+  EGLint attrib[] =
+    {
+        EGL_BUFFER_SIZE,       EGL_DONT_CARE,
+        EGL_DEPTH_SIZE,        0,
+        EGL_RED_SIZE,          8,
+        EGL_GREEN_SIZE,        8,
+        EGL_BLUE_SIZE,         8,
+	EGL_RENDERABLE_TYPE,   EGL_OPENGL_ES_BIT, 
+        EGL_NONE
+    };
 
   g_return_val_if_fail (eglbackend != NULL, FALSE);
 
   GST_DEBUG_OBJECT (eglbackend, "create_window");
 
+#if HAVE_GDL
+  if (initialize_gdl (eglbackend, GDL_PLANE_ID_UPP_C) == FALSE)
+    {
+      GST_ERROR_OBJECT (eglbackend, "Cannot initialize GDL backend");
+      goto error_display;
+    }
+#endif
+
   /* Initialize EGL for the default display */
   eglbackend->display = eglGetDisplay ((NativeDisplayType) EGL_DEFAULT_DISPLAY);
   if (eglbackend->display == EGL_NO_DISPLAY)
     {
       GST_ERROR_OBJECT (eglbackend, "cannot get a display");
-      return FALSE;
+      goto error_display;
     }
-  if (!eglInitialize (eglbackend->display, NULL, NULL))
+
+  if (!eglInitialize (eglbackend->display, &egl_major, &egl_minor))
     {
       get_egl_error ("eglInitialize");
-      return FALSE;
+      goto error;
     }
 
   /* Get and log EGL informations */
+  GST_DEBUG_OBJECT (eglbackend, "EGL init version: %d.%d", egl_major, egl_minor);
+
   eglbackend->vendor = eglQueryString (eglbackend->display, EGL_VENDOR);
   eglbackend->version = eglQueryString (eglbackend->display, EGL_VERSION);
   eglbackend->extensions = eglQueryString (eglbackend->display, EGL_EXTENSIONS);
@@ -358,23 +512,40 @@ pgm_gles_egl_backend_create_window (PgmGlesBackend *glesbackend)
 
   if (!eglChooseConfig (eglbackend->display, attrib, &config, 1, &num_configs))
     {
-      get_egl_error ("eglCreateWindowSurface");
-      return FALSE;
+      get_egl_error ("eglChooseConfig");
+      goto error;
     }
+  GST_DEBUG_OBJECT (eglbackend, "EGL config %d (num=%d)", config, num_configs);
 
   /* Create the context and make it current */
+#if HAVE_GDL
+  eglbackend->surface = eglCreateWindowSurface (eglbackend->display, config,
+      (NativeWindowType)GDL_PLANE_ID_UPP_C, NULL);
+  gbackend = eglbackend;
+
+  if (eglBindAPI(EGL_OPENGL_ES_API) == 0)
+    {
+      GST_DEBUG_OBJECT (eglbackend, "eglBindAPI: %s", eglbackend->extensions);
+      goto error;
+    }
+  eglWaitClient();
+#else
   eglbackend->surface = eglCreateWindowSurface (eglbackend->display, config,
                                                 (NativeWindowType) 0, NULL);
-  if (!get_egl_error ("eglCreateWindowSurface"))
-    return FALSE;
+#endif
+
+  if (eglbackend->surface == EGL_NO_SURFACE \
+    || !get_egl_error ("eglCreateWindowSurface"))
+    goto error;
+
   eglbackend->context = eglCreateContext (eglbackend->display, config,
                                           NULL, NULL);
-  if (!get_egl_error ("eglCreateContext"))
-    return FALSE;
-  eglMakeCurrent (eglbackend->display, eglbackend->surface, eglbackend->surface,
-                  eglbackend->context);
-  if (!get_egl_error ("eglMakeCurrent"))
-    return FALSE;
+
+  if (eglbackend->context == EGL_NO_CONTEXT
+      || !get_egl_error ("eglCreateContext"))
+    {
+      goto error;
+    }
 
 #ifdef HAVE_TSLIB
   /* Touch screen event source */
@@ -383,11 +554,24 @@ pgm_gles_egl_backend_create_window (PgmGlesBackend *glesbackend)
       GST_ERROR_OBJECT (eglbackend, "couldn't add event handling source\n");
       eglMakeCurrent (eglbackend->display, EGL_NO_SURFACE, EGL_NO_SURFACE,
                       EGL_NO_CONTEXT);
-      eglTerminate (eglbackend->display);
-      return FALSE;
+      goto error;
+    }
+#else
+  eglMakeCurrent (eglbackend->display, eglbackend->surface,
+                  eglbackend->surface, eglbackend->context);
+  if (!get_egl_error ("eglMakeCurrent"))
+    {
+      goto error;
     }
 #endif /* HAVE_TSLIB */
 
+#if HAVE_GDL
+  /* Get and store the window size, the resolution in this case is the same */
+  gdl_tvmode_get_size (&eglbackend->resolution_width,
+      &eglbackend->resolution_height);
+  eglbackend->size_mm_width = eglbackend->resolution_width;
+  eglbackend->size_mm_height = eglbackend->resolution_height;
+#else
   /* Get and store the window size, the resolution in this case is the same */
   eglQuerySurface (eglbackend->display, eglbackend->surface, EGL_WIDTH,
                    &eglbackend->size_mm_width);
@@ -395,6 +579,7 @@ pgm_gles_egl_backend_create_window (PgmGlesBackend *glesbackend)
   eglQuerySurface (eglbackend->display, eglbackend->surface, EGL_HEIGHT,
                    &eglbackend->size_mm_height);
   eglbackend->resolution_height = eglbackend->size_mm_height;
+#endif
 
   /* Specify the minimum number of video frame periods per buffer swap, this
    * is actually how vertical synchronization is handled */
@@ -407,6 +592,24 @@ pgm_gles_egl_backend_create_window (PgmGlesBackend *glesbackend)
   eglbackend->created = TRUE;
 
   return TRUE;
+
+error:
+  if (eglbackend->context != EGL_NO_CONTEXT)
+    {
+      eglDestroyContext (eglbackend->display, eglbackend->context);
+    }
+  if (eglbackend->surface != EGL_NO_SURFACE)
+    {
+      eglDestroySurface (eglbackend->display, eglbackend->surface);
+    }
+
+  eglTerminate (eglbackend->display);
+
+error_display:
+#if HAVE_GDL
+  gdl_close();
+#endif
+  return FALSE;
 }
 
 static gboolean
@@ -421,13 +624,25 @@ pgm_gles_egl_backend_destroy_window (PgmGlesBackend *glesbackend)
 #ifdef HAVE_TSLIB
       remove_event_source (eglbackend);
 #endif /* HAVE_TSLIB */
-
+#if 0
+  /* this makes the hardware crash */
+  if (eglbackend->context != EGL_NO_CONTEXT)
+    {
+      eglDestroyContext (eglbackend->display, eglbackend->context);
+    }
+  if (eglbackend->surface != EGL_NO_SURFACE)
+    {
+      eglDestroySurface (eglbackend->display, eglbackend->surface);
+    }
       eglMakeCurrent (eglbackend->display, EGL_NO_SURFACE, EGL_NO_SURFACE,
                       EGL_NO_CONTEXT);
+#endif
       eglTerminate (eglbackend->display);
       eglbackend->created = FALSE;
     }
-
+#if HAVE_GDL
+  gdl_close();
+#endif
   return TRUE;
 }
 
@@ -449,8 +664,6 @@ pgm_gles_egl_backend_swap_buffers (PgmGlesBackend *glesbackend)
 {
   PgmGlesEglBackend *eglbackend = PGM_GLES_EGL_BACKEND (glesbackend);
 
-  GST_LOG_OBJECT (eglbackend, "swap_buffers");
-
   eglSwapBuffers (eglbackend->display, eglbackend->surface);
 }
 
@@ -458,10 +671,6 @@ static gpointer
 pgm_gles_egl_backend_get_proc_address (PgmGlesBackend *glesbackend,
                                        const gchar *proc_name)
 {
-  PgmGlesEglBackend *eglbackend = PGM_GLES_EGL_BACKEND (glesbackend);
-
-  GST_LOG_OBJECT (eglbackend, "get_proc_address");
-
   return eglGetProcAddress (proc_name);
 }
 
@@ -472,21 +681,69 @@ pgm_gles_egl_backend_get_screen_size_mm (PgmGlesBackend *glesbackend,
 {
   PgmGlesEglBackend *eglbackend = PGM_GLES_EGL_BACKEND (glesbackend);
 
-  GST_LOG_OBJECT (eglbackend, "get_screen_size_mm");
-
   *width = eglbackend->size_mm_width;
   *height = eglbackend->size_mm_height;
 }
 
+static void
+pgm_gles_egl_backend_set_screen_resolution (PgmGlesBackend *glesbackend,
+                               gint width, gint height)
+{
+#if HAVE_GDL
+  PgmGlesEglBackend *eglbackend = PGM_GLES_EGL_BACKEND (glesbackend);
+  gdl_rectangle_t rect;                                                    
+  gint num_configs = 0;
+  EGLConfig config = 0;
+  EGLint attrib[] =
+    {
+        EGL_BUFFER_SIZE,       EGL_DONT_CARE,
+        EGL_DEPTH_SIZE,        0,
+        EGL_RED_SIZE,          8,
+        EGL_GREEN_SIZE,        8,
+        EGL_BLUE_SIZE,         8,
+	EGL_RENDERABLE_TYPE,   EGL_OPENGL_ES_BIT, 
+        EGL_NONE
+    };
+
+  eglbackend->size_mm_width =
+  eglbackend->resolution_width = width;
+  eglbackend->size_mm_height =
+  eglbackend->resolution_height = height;
+
+  eglMakeCurrent (eglbackend->display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
+  eglDestroySurface (eglbackend->display, eglbackend->surface);
+  eglbackend->surface = NULL;
+
+  rect.origin.x = 0;
+  rect.origin.y = 0;
+  rect.width = width;
+  rect.height = height;
+  gdl_plane_config_begin (GDL_PLANE_ID_UPP_C);
+  gdl_plane_set_attr (GDL_PLANE_DST_RECT, &rect);
+  gdl_plane_config_end (GDL_FALSE);
+
+  if (!eglChooseConfig (eglbackend->display, attrib, &config, 1, &num_configs))
+    {
+      GST_WARNING_OBJECT (eglbackend, "Cannot get EGL configuration");
+    }
+  eglbackend->surface = eglCreateWindowSurface (eglbackend->display, config,
+    (NativeWindowType)GDL_PLANE_ID_UPP_C, NULL);
+
+  if (eglBindAPI (EGL_OPENGL_ES_API) == 0)
+    {
+      GST_WARNING_OBJECT (eglbackend, "Cannot bind OpenGL-ES from EGL");
+    }
+  eglMakeCurrent (eglbackend->display, eglbackend->surface,
+                  eglbackend->surface, eglbackend->context);
+#endif
+}
+
 static void
 pgm_gles_egl_backend_get_screen_resolution (PgmGlesBackend *glesbackend,
-                                            gint *width,
-                                            gint *height)
+                                            gint *width, gint *height)
 {
   PgmGlesEglBackend *eglbackend = PGM_GLES_EGL_BACKEND (glesbackend);
 
-  GST_LOG_OBJECT (eglbackend, "get_screen_resolution");
-
   *width = eglbackend->resolution_width;
   *height = eglbackend->resolution_height;
 }
@@ -494,7 +751,7 @@ pgm_gles_egl_backend_get_screen_resolution (PgmGlesBackend *glesbackend,
 /* GObject stuff */
 
 PGM_DEFINE_DYNAMIC_TYPE (PgmGlesEglBackend, pgm_gles_egl_backend,
-                         PGM_TYPE_GLES_BACKEND);
+                         PGM_TYPE_GLES_BACKEND)
 
 void
 pgm_gles_egl_backend_register (GTypeModule *module)
@@ -547,6 +804,8 @@ pgm_gles_egl_backend_class_init (PgmGlesEglBackendClass *klass)
     GST_DEBUG_FUNCPTR (pgm_gles_egl_backend_get_screen_size_mm);
   glesbackend_class->get_screen_resolution =
     GST_DEBUG_FUNCPTR (pgm_gles_egl_backend_get_screen_resolution);
+  glesbackend_class->set_screen_resolution =
+    GST_DEBUG_FUNCPTR (pgm_gles_egl_backend_set_screen_resolution);
 }
 
 static void
