@@ -530,7 +530,7 @@ update_vertices_position (PgmGlesImage *glesimage)
 
   /* Add the offsets to the border vertices, if necessary */
   if (glesimage->border_width > 0.0f)
-    for (i = 0; i < 30; i += 3)
+    for (i = 0; i < 36; i += 3)
       {
         img_border[i] += x_offset;
         img_border[i+1] += y_offset;
@@ -600,8 +600,6 @@ pgm_gles_image_draw (PgmGlesDrawable *glesdrawable)
   PgmGlesImage *glesimage = PGM_GLES_IMAGE (glesdrawable);
   PgmGlesContextProcAddress *gles;
 
-  GST_LOG_OBJECT (glesdrawable, "draw");
-
   /* Avoid drawing if it's not visible */
   if (glesimage->empty || (glesimage->fg_color[3] == 0.0f))
     return;
@@ -625,7 +623,12 @@ pgm_gles_image_draw (PgmGlesDrawable *glesdrawable)
 
   gles->disable_client_state (PGM_GLES_TEXTURE_COORD_ARRAY);
 
-  if (glesimage->border_width > 0.0f)
+  if (glesimage->border_width <= 0.0f)
+    {
+      gles->disable_client_state (PGM_GLES_VERTEX_ARRAY);
+      gles->disable_client_state (PGM_GLES_COLOR_ARRAY);
+    }
+  else
     {
       /* Upload vertices and colors for the border */
       gles->vertex_pointer (3, PGM_GLES_FLOAT, 0, glesimage->border_vertex);
@@ -634,16 +637,15 @@ pgm_gles_image_draw (PgmGlesDrawable *glesdrawable)
       /* Draw the border */
       gles->draw_elements (PGM_GLES_TRIANGLES, 24,
                            PGM_GLES_UNSIGNED_SHORT, border_indices);
-    }
 
-  gles->disable_client_state (PGM_GLES_VERTEX_ARRAY);
-  gles->disable_client_state (PGM_GLES_COLOR_ARRAY);
+      gles->disable_client_state (PGM_GLES_VERTEX_ARRAY);
+      gles->disable_client_state (PGM_GLES_COLOR_ARRAY);
+    }
 }
 
 static void
 pgm_gles_image_regenerate (PgmGlesDrawable *glesdrawable)
 {
-  GST_LOG_OBJECT (glesdrawable, "regenerate");
 }
 
 static void
@@ -651,8 +653,6 @@ pgm_gles_image_update_projection (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesImage *glesimage = PGM_GLES_IMAGE (glesdrawable);
 
-  GST_LOG_OBJECT (glesdrawable, "update_projection");
-
   update_image_ratio (glesimage);
   update_layout (glesimage);
 }
@@ -662,8 +662,6 @@ pgm_gles_image_set_size (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesImage *glesimage = PGM_GLES_IMAGE (glesdrawable);
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_size");
-
   update_drawable_ratio (glesimage);
   update_layout (glesimage);
 }
@@ -673,8 +671,6 @@ pgm_gles_image_set_position (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesImage *glesimage = PGM_GLES_IMAGE (glesdrawable);
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_position");
-
   update_vertices_position (glesimage);
 }
 
@@ -682,11 +678,10 @@ static void
 pgm_gles_image_set_fg_color (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesImage *glesimage = PGM_GLES_IMAGE (glesdrawable);
+  PgmGlesFloat *fg_color = glesimage->fg_color;
   PgmDrawable *drawable;
   PgmGlesFloat color[4];
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_fg_color");
-
   drawable = glesdrawable->drawable;
 
   color[0] = drawable->fg_r * INV_255;
@@ -695,22 +690,22 @@ pgm_gles_image_set_fg_color (PgmGlesDrawable *glesdrawable)
   color[3] = drawable->fg_a * drawable->opacity * INV_255;
 
   GST_OBJECT_LOCK (drawable);
-  glesimage->fg_color[0] = color[0];
-  glesimage->fg_color[1] = color[1];
-  glesimage->fg_color[2] = color[2];
-  glesimage->fg_color[3] = color[3];
-  glesimage->fg_color[4] = color[0];
-  glesimage->fg_color[5] = color[1];
-  glesimage->fg_color[6] = color[2];
-  glesimage->fg_color[7] = color[3];
-  glesimage->fg_color[8] = color[0];
-  glesimage->fg_color[9] = color[1];
-  glesimage->fg_color[10] = color[2];
-  glesimage->fg_color[11] = color[3];
-  glesimage->fg_color[12] = color[0];
-  glesimage->fg_color[13] = color[1];
-  glesimage->fg_color[14] = color[2];
-  glesimage->fg_color[15] = color[3];
+  fg_color[0] = color[0];
+  fg_color[1] = color[1];
+  fg_color[2] = color[2];
+  fg_color[3] = color[3];
+  fg_color[4] = color[0];
+  fg_color[5] = color[1];
+  fg_color[6] = color[2];
+  fg_color[7] = color[3];
+  fg_color[8] = color[0];
+  fg_color[9] = color[1];
+  fg_color[10] = color[2];
+  fg_color[11] = color[3];
+  fg_color[12] = color[0];
+  fg_color[13] = color[1];
+  fg_color[14] = color[2];
+  fg_color[15] = color[3];
   GST_OBJECT_UNLOCK (drawable);
 }
 
@@ -724,8 +719,6 @@ pgm_gles_image_set_opacity (PgmGlesDrawable *glesdrawable)
   gfloat fg_opacity, border_opacity;
   guint i;
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_opacity");
-
   /* Get the opacities */
   GST_OBJECT_LOCK (drawable);
   fg_opacity = drawable->fg_a * drawable->opacity * SQR_INV_255;
@@ -750,8 +743,6 @@ pgm_gles_image_sync (PgmGlesDrawable *glesdrawable)
   PgmImage *image = PGM_IMAGE (glesdrawable->drawable);
   PgmImageStorageType type;
 
-  GST_LOG_OBJECT (glesdrawable, "sync");
-
   /* Synchronize various properties */
   pgm_gles_image_set_fg_color (glesdrawable);
   update_interp (glesimage);
@@ -796,7 +787,7 @@ pgm_gles_image_sync (PgmGlesDrawable *glesdrawable)
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlesImage, pgm_gles_image, PGM_TYPE_GLES_DRAWABLE);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlesImage, pgm_gles_image, PGM_TYPE_GLES_DRAWABLE)
 
 void
 pgm_gles_image_register (GTypeModule *module)
@@ -817,8 +808,6 @@ pgm_gles_image_dispose (GObject *object)
 
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "dispose");
-
   /* Clean up the slaves */
   GST_OBJECT_LOCK (master);
   walk = master->slaves;
@@ -893,10 +882,8 @@ pgm_gles_image_class_finalize (PgmGlesImageClass *klass)
 static void
 pgm_gles_image_init (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "init");
-
   glesimage->empty = TRUE;
-  glesimage->border_width = 0.0f;
+  /* glesimage->border_width = 0.0f; */
 }
 
 /* Public methods */
@@ -954,8 +941,6 @@ pgm_gles_image_clear (PgmGlesImage *glesimage)
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glesimage);
   PgmGlesContextTask *task = NULL;
 
-  GST_DEBUG_OBJECT (glesimage, "clear");
-
   if (glesimage->empty)
     return;
 
@@ -985,8 +970,6 @@ pgm_gles_image_set_from_file (PgmGlesImage *glesimage)
   PgmImage *image = PGM_IMAGE (glesdrawable->drawable);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "set_from_file");
-
   GST_OBJECT_LOCK (image);
 
   if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_FILE
@@ -1032,8 +1015,6 @@ pgm_gles_image_set_from_buffer (PgmGlesImage *glesimage)
   PgmImage *image = PGM_IMAGE (glesdrawable->drawable);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "set_from_buffer");
-
   GST_OBJECT_LOCK (image);
 
   if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_BUFFER
@@ -1048,7 +1029,7 @@ pgm_gles_image_set_from_buffer (PgmGlesImage *glesimage)
                                image->data.buffer.width,
                                image->data.buffer.height,
                                image->data.buffer.size,
-                               image->data.buffer.stride);
+                               image->data.buffer.stride, FALSE);
   pgm_gles_texture_set_matrix (glesimage->texture, image->mapping_matrix);
 
   GST_OBJECT_UNLOCK (image);
@@ -1076,8 +1057,6 @@ pgm_gles_image_set_from_gst_buffer (PgmGlesImage *glesimage)
   PgmImage *image = PGM_IMAGE (glesdrawable->drawable);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "set_from_gst_buffer");
-
   /* It's the first time the glesimage receives a GstBuffer, let's set the
    * texture buffer with its different informations */
   if (G_UNLIKELY (glesimage->empty))
@@ -1142,8 +1121,6 @@ pgm_gles_image_set_from_pixbuf (PgmGlesImage *glesimage)
   PgmImage *image = PGM_IMAGE (glesdrawable->drawable);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "set_from_pixbuf");
-
   GST_OBJECT_LOCK (image);
 
   if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_PIXBUF
@@ -1181,8 +1158,6 @@ pgm_gles_image_set_from_image (PgmGlesImage *glesimage)
   PgmGlesViewport *glesviewport = glesdrawable->glesviewport;
   PgmGlesImage *master;
 
-  GST_DEBUG_OBJECT (glesimage, "set_from_image");
-
   /* Retrieve the master glesimage */
   GST_OBJECT_LOCK (image);
 
@@ -1210,16 +1185,12 @@ pgm_gles_image_set_from_image (PgmGlesImage *glesimage)
 void
 pgm_gles_image_set_mapping_matrix (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_mapping_matrix");
-
   update_mapping_matrix (glesimage);
 }
 
 void
 pgm_gles_image_set_alignment (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_alignment");
-
   update_alignment (glesimage);
   update_layout (glesimage);
 }
@@ -1227,8 +1198,6 @@ pgm_gles_image_set_alignment (PgmGlesImage *glesimage)
 void
 pgm_gles_image_set_layout (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_layout");
-
   update_layout (glesimage);
 }
 
@@ -1238,8 +1207,6 @@ pgm_gles_image_set_interp (PgmGlesImage *glesimage)
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glesimage);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "set_interp");
-
   update_interp (glesimage);
 
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_UPDATE_TEXTURE,
@@ -1254,8 +1221,6 @@ pgm_gles_image_set_wrapping (PgmGlesImage *glesimage)
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glesimage);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesimage, "set_wrapping");
-
   update_wrapping (glesimage);
 
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_UPDATE_TEXTURE,
@@ -1267,8 +1232,6 @@ pgm_gles_image_set_wrapping (PgmGlesImage *glesimage)
 void
 pgm_gles_image_set_aspect_ratio (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_aspect_ratio");
-
   update_image_ratio (glesimage);
   update_layout (glesimage);
 }
@@ -1276,8 +1239,6 @@ pgm_gles_image_set_aspect_ratio (PgmGlesImage *glesimage)
 void
 pgm_gles_image_set_border_width (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_border_width");
-
   update_border_width (glesimage);
   update_layout (glesimage);
 }
@@ -1285,15 +1246,11 @@ pgm_gles_image_set_border_width (PgmGlesImage *glesimage)
 void
 pgm_gles_image_set_border_inner_color (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_border_inner_color");
-
   update_border_inner_color (glesimage);
 }
 
 void
 pgm_gles_image_set_border_outer_color (PgmGlesImage *glesimage)
 {
-  GST_DEBUG_OBJECT (glesimage, "set_border_outer_color");
-
   update_border_outer_color (glesimage);
 }
