@@ -66,10 +66,10 @@
 
 /* These are the A,R,G,B component byte masks for a native endian ARGB value
  * (such as what is given by Cairo) */
-#define ARGB_A_MASK 0xFF << ARGB_A_OFFSET
-#define ARGB_R_MASK 0xFF << ARGB_R_OFFSET
-#define ARGB_G_MASK 0xFF << ARGB_G_OFFSET
-#define ARGB_B_MASK 0xFF << ARGB_B_OFFSET
+#define ARGB_A_MASK 0xFF000000
+#define ARGB_R_MASK 0x00FF0000
+#define ARGB_G_MASK 0x0000FF00
+#define ARGB_B_MASK 0x000000FF
 
 GST_DEBUG_CATEGORY_STATIC (pgm_gles_text_debug);
 #define GST_CAT_DEFAULT pgm_gles_text_debug
@@ -78,56 +78,24 @@ static PgmGlesDrawableClass *parent_class = NULL;
 
 /* Private methods */
 
-/* Gets the projected height of the drawable on the viewport */
+/* Gets the projected height of the drawable on the viewport at z=0 */
 static gfloat
 get_projected_height (PgmGlesDrawable *glesdrawable,
                       gfloat canvas_height)
 {
   PgmGlesViewport *glesviewport = glesdrawable->glesviewport;
-  PgmViewport *viewport = PGM_VIEWPORT (glesviewport);
-  PgmCanvas *canvas = viewport->canvas;
-  gfloat height;
 
-  /* FIXME: That's just the projected height on z=0 */
-  GST_OBJECT_LOCK (canvas);
-  GST_OBJECT_LOCK (viewport);
-
-  if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
-      || viewport->rotation == PGM_VIEWPORT_ROTATION_180)
-    height = (canvas_height * viewport->projected_height) / canvas->height;
-  else
-    height = (canvas_height * viewport->projected_height) / canvas->width;
-
-  GST_OBJECT_UNLOCK (viewport);
-  GST_OBJECT_UNLOCK (canvas);
-
-  return height;
+  return (canvas_height * glesviewport->projected_h) / glesviewport->canvas_h;
 }
 
-/* Gets the projected width of the drawable on the viewport */
+/* Gets the projected width of the drawable on the viewport at z=0 */
 static gfloat
 get_projected_width (PgmGlesDrawable *glesdrawable,
                      gfloat canvas_width)
 {
   PgmGlesViewport *glesviewport = glesdrawable->glesviewport;
-  PgmViewport *viewport = PGM_VIEWPORT (glesviewport);
-  PgmCanvas *canvas = viewport->canvas;
-  gfloat width;
 
-  /* FIXME: That's just the projected width on z=0 */
-  GST_OBJECT_LOCK (canvas);
-  GST_OBJECT_LOCK (viewport);
-
-  if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
-      || viewport->rotation == PGM_VIEWPORT_ROTATION_180)
-    width = (canvas_width * viewport->projected_width) / canvas->width;
-  else
-    width = (canvas_width * viewport->projected_width) / canvas->height;
-
-  GST_OBJECT_UNLOCK (viewport);
-  GST_OBJECT_UNLOCK (canvas);
-
-  return width;
+  return (canvas_width * glesviewport->projected_w) / glesviewport->canvas_w;
 }
 
 /* Defines texture coordinates */
@@ -135,15 +103,40 @@ static void
 set_coordinates (PgmGlesText *glestext)
 {
   PgmGlesTexture *glestexture = glestext->texture;
+  PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
+  PgmText *text = PGM_TEXT (glesdrawable->drawable);
+  gfloat ratio = ((gfloat) glestexture->height / glestexture->height_pot);
+  gfloat offset;
 
-  glestext->coord[2] = (PgmGlesFloat) glestexture->width
-    / glestexture->width_pot;
+  if (glestext->multiline == TRUE)
+    offset = 0.0f;
+  else
+    offset = (1.0f - (text->height / glesdrawable->height)) * ratio * -0.5f;
+
+  glestext->coord[2] = (PgmGlesFloat) glestexture->width / glestexture->width_pot;
   glestext->coord[4] = glestext->coord[2];
-  glestext->coord[5] = (PgmGlesFloat) glestexture->height
-    / glestexture->height_pot;
+  glestext->coord[1] = offset;
+  glestext->coord[3] = glestext->coord[1];
+  glestext->coord[5] = (PgmGlesFloat) ratio + offset;
   glestext->coord[7] = glestext->coord[5];
 }
 
+/* Stores color depending on PgmText color normalizing components to the range
+ * [0.0; 1.0] */
+static void
+set_color (PgmGlesText *glestext)
+{
+  PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
+  PgmText *text = PGM_TEXT (glesdrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+  glestext->color[0] = text->r * INV_255;
+  glestext->color[1] = text->g * INV_255;
+  glestext->color[2] = text->b * INV_255;
+  glestext->color[3] = text->a * INV_255;
+  GST_OBJECT_UNLOCK (text);
+}
+
 /* Stores outline width depending on PgmText outline width converting it
  * from canvas coordinates to viewport coordinates */
 static void
@@ -152,8 +145,7 @@ set_outline_width (PgmGlesText *glestext)
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
   PgmText *text = PGM_TEXT (glesdrawable->drawable);
 
-  glestext->outline_width = get_projected_height (glesdrawable,
-                                                  text->outline_width);
+  glestext->outline_width = get_projected_height (glesdrawable, text->outline_width);
 }
 
 /* Stores outline color depending on PgmText color normalizing components to
@@ -172,6 +164,60 @@ set_outline_color (PgmGlesText *glestext)
   GST_OBJECT_UNLOCK (text);
 }
 
+/* Stores the shadow casting state */
+static void
+set_cast_shadow (PgmGlesText *glestext)
+{
+  PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
+  PgmText *text = PGM_TEXT (glesdrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+  glestext->cast_shadow = text->cast_shadow;
+  GST_OBJECT_UNLOCK (text);
+}
+
+/* Stores the shadow position */
+static void
+set_shadow_position (PgmGlesText *glestext)
+{
+  PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
+  PgmText *text = PGM_TEXT (glesdrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+
+  if (text->shadow_position & PGM_TEXT_SHADOW_NORTH)
+    glestext->shadow_longitude_factor = -1.0f;
+  else if (text->shadow_position & PGM_TEXT_SHADOW_SOUTH)
+    glestext->shadow_longitude_factor = 1.0f;
+  else
+    glestext->shadow_longitude_factor = 0.0f;
+
+  if (text->shadow_position & PGM_TEXT_SHADOW_WEST)
+    glestext->shadow_latitude_factor = -1.0f;
+  else if (text->shadow_position & PGM_TEXT_SHADOW_EAST)
+    glestext->shadow_latitude_factor = 1.0f;
+  else
+    glestext->shadow_latitude_factor = 0.0f;
+
+  GST_OBJECT_UNLOCK (text);
+}
+
+/* Stores shadow color depending on PgmText color normalizing components to the
+ * range [0.0; 1.0] */
+static void
+set_shadow_color (PgmGlesText *glestext)
+{
+  PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
+  PgmText *text = PGM_TEXT (glesdrawable->drawable);
+
+  GST_OBJECT_LOCK (text);
+  glestext->shadow_color[0] = text->shadow_r * INV_255;
+  glestext->shadow_color[1] = text->shadow_g * INV_255;
+  glestext->shadow_color[2] = text->shadow_b * INV_255;
+  glestext->shadow_color[3] = text->shadow_a * INV_255;
+  GST_OBJECT_UNLOCK (text);
+}
+
 /* Defines Pango spacing depending on PgmText line spacing converting it
  * from canvas coordinates to viewport coordinates */
 static void
@@ -182,8 +228,8 @@ set_line_spacing (PgmGlesText *glestext)
   gfloat line_spacing;
 
   line_spacing = get_projected_height (glesdrawable, text->line_spacing);
-  pango_layout_set_spacing (glestext->layout, (gint)
-                            (line_spacing * PANGO_SCALE));
+  pango_layout_set_spacing
+    (glestext->layout, pgm_lrintf (line_spacing * PANGO_SCALE));
 }
 
 /* Calculates the maximum total height of a line of text for the current font
@@ -234,8 +280,8 @@ set_font_height (PgmGlesText *glestext)
   /* All the heights here are in device unit. */
   gfloat requested_height, wanted_height, max_height, font_ratio;
 
+  /* text->height is a value in canvas space */
   requested_height = get_projected_height (glesdrawable, text->height);
-  requested_height *= glesdrawable->height;
 
   /* The "pango height" is something like the height of an M, but we want to set
    * the maximum height of a line, so we need to calculate the ratio first. */
@@ -248,6 +294,10 @@ set_font_height (PgmGlesText *glestext)
   pango_font_description_set_absolute_size (glestext->desc,
                                             wanted_height * PANGO_SCALE);
   pango_layout_set_font_description (glestext->layout, glestext->desc);
+
+  /* Precompute and store the offset to apply to the shadow based on the font
+   * wanted height, the idea is to increase the offset with the font height. */
+  glestext->shadow_offset = sqrtf (wanted_height / 16.0f);
 }
 
 /* Defines Pango font family depending on PgmText font family */
@@ -285,18 +335,64 @@ static void
 set_width (PgmGlesText *glestext)
 {
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
-  PgmViewport *viewport = PGM_VIEWPORT (glesdrawable->glesviewport);
-  gfloat non_square_ratio;
-  gfloat width;
 
-  width = get_projected_width (glesdrawable, glesdrawable->width);
+  /* Mutliline mode */
+  if (glestext->multiline == TRUE)
+    {
+      PgmViewport *viewport = PGM_VIEWPORT (glesdrawable->glesviewport);
+      gfloat non_square_ratio, size;
 
-  GST_OBJECT_LOCK (viewport);
-  non_square_ratio = 1.0f / viewport->pixel_aspect_ratio;
-  GST_OBJECT_UNLOCK (viewport);
+      /* Set the correct width */
+      size = get_projected_width (glesdrawable, glesdrawable->width);
+      GST_OBJECT_LOCK (viewport);
+      non_square_ratio = 1.0f / viewport->pixel_aspect_ratio;
+      GST_OBJECT_UNLOCK (viewport);
+      pango_layout_set_width
+        (glestext->layout, pgm_lrintf (size * PANGO_SCALE * non_square_ratio));
 
-  pango_layout_set_width (glestext->layout, (guint)
-                          (width * PANGO_SCALE * non_square_ratio));
+      /* Set the correct height */
+      size = get_projected_height (glesdrawable, glesdrawable->height);
+      pango_layout_set_height
+        (glestext->layout, pgm_lrintf (size * PANGO_SCALE));
+    }
+
+  /* Monoline mode */
+  else
+    {
+      PgmText *text = PGM_TEXT (glesdrawable->drawable);
+
+      /* If the text needs to be ellipsized, set the correct width */
+      if (text->ellipsize != PGM_TEXT_ELLIPSIZE_NONE)
+        {
+          PgmViewport *viewport = PGM_VIEWPORT (glesdrawable->glesviewport);
+          gfloat non_square_ratio, width;
+
+          /* Set the correct width */
+          width = get_projected_width (glesdrawable, glesdrawable->width);
+          GST_OBJECT_LOCK (viewport);
+          non_square_ratio = 1.0f / viewport->pixel_aspect_ratio;
+          GST_OBJECT_UNLOCK (viewport);
+          pango_layout_set_width
+            (glestext->layout,
+             pgm_lrintf (width * PANGO_SCALE * non_square_ratio));
+        }
+      /* Otherwise disable the width */
+      else
+        pango_layout_set_width (glestext->layout, -1);
+
+      /* Disable the height */
+      pango_layout_set_height (glestext->layout, -1);
+    }
+}
+
+/* Stores the multiline state */
+static void
+set_multiline (PgmGlesText *glestext)
+{
+  PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
+  PgmText *text = PGM_TEXT (glesdrawable->drawable);
+
+  glestext->multiline = text->multiline;
 }
 
 /* Defines Pango justification depending on PgmText justification */
@@ -554,27 +650,33 @@ update_properties (PgmGlesText *glestext)
   set_alignment (glestext);
   set_ellipsize (glestext);
   set_justify (glestext);
+  set_multiline (glestext);
   set_line_spacing (glestext);
   set_width (glestext);
+  set_coordinates (glestext);
   set_wrap (glestext);
   set_gravity (glestext);
   set_label (glestext);
+  set_color (glestext);
   set_outline_width (glestext);
   set_outline_color (glestext);
+  set_cast_shadow (glestext);
+  set_shadow_position (glestext);
+  set_shadow_color (glestext);
   set_font_height (glestext);
 }
 
-/* Cairo uses premultiplied-alpha to represent pixels (mostly for compositing).
- * This function unpremultiplies the alpha of cairo's native endian ARGB buffer
- * and converts it to BGRA in memory order. */
+/* Cairo uses a premultiplied alpha blending function, pigment's not. This
+ * function unpremultiplies the alpha of cairo's native endian ARGB buffer and
+ * converts it to BGRA in memory order. */
 static void
 unpremultiply_alpha (guint32 *buffer,
-                     guint width,
-                     guint height)
+                     guint32 width,
+                     guint32 height)
 {
-  guint size = width * height;
-  guint8 a, r, g, b;
-  guint i;
+  guint32 size = width * height;
+  guint32 a, r, g, b;
+  guint32 i;
 
   for (i = 0; i < size; i++)
     {
@@ -616,19 +718,40 @@ create_pixmap (PgmGlesText *glestext)
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
   PgmViewport *viewport = PGM_VIEWPORT (glesdrawable->glesviewport);
   PgmGlesContextTask *task;
-  gfloat pixel_aspect_ratio;
+  gfloat sx, sy;
 
-  glestext->width = (guint) get_projected_width (glesdrawable,
-                                                 glesdrawable->width);
-  glestext->height = (guint) get_projected_height (glesdrawable,
-                                                   glesdrawable->height);
+  /* It's important here to round the floating-point value returned by
+   * get_projected_* using the default "to nearest" rounding mode of the FPU and
+   * not the "truncate" mode implied by the float-to-int cast. It's faster and
+   * allows to get the exact rasterized size required to have a direct texel to
+   * pixel mapping. */
+  glestext->width = MAX
+    (1, pgm_lrintf (get_projected_width (glesdrawable, glesdrawable->width)));
+  glestext->height = MAX
+    (1, pgm_lrintf (get_projected_height (glesdrawable, glesdrawable->height)));
 
   GST_OBJECT_LOCK (viewport);
-  pixel_aspect_ratio = viewport->pixel_aspect_ratio;
+  if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
+      || viewport->rotation == PGM_VIEWPORT_ROTATION_180)
+    {
+      sx = viewport->pixel_aspect_ratio;
+      sy = 1.0f;
+    }
+  else
+    {
+      sx = 1.0f;
+      sy = viewport->pixel_aspect_ratio;
+    }
   GST_OBJECT_UNLOCK (viewport);
 
   glestext->size = glestext->width * glestext->height * 4;
   glestext->buffer = (guchar *) g_slice_alloc0 (glestext->size);
+  if (glestext->buffer == NULL)
+    {
+      GST_WARNING_OBJECT (glestext, "Unable to allocate %d bytes",
+                          glestext->size);
+      return;
+    }
   glestext->surface = cairo_image_surface_create_for_data (glestext->buffer,
                                                            CAIRO_FORMAT_ARGB32,
                                                            glestext->width,
@@ -636,14 +759,17 @@ create_pixmap (PgmGlesText *glestext)
                                                            glestext->width * 4);
 
   glestext->cairo_ctx = cairo_create (glestext->surface);
+  g_assert (glestext->cairo_ctx != NULL);
   cairo_set_font_options (glestext->cairo_ctx, glestext->font_options);
-  cairo_scale (glestext->cairo_ctx, pixel_aspect_ratio, 1.0f);
+  cairo_scale (glestext->cairo_ctx, sx, sy);
   glestext->layout = pango_cairo_create_layout (glestext->cairo_ctx);
+  g_assert (glestext->layout != NULL);
   glestext->pango_ctx = pango_layout_get_context (glestext->layout);
+  g_assert (glestext->pango_ctx != NULL);
 
   pgm_gles_texture_set_buffer (glestext->texture, glestext->buffer,
-                               PGM_IMAGE_RGBA, glestext->width, glestext->height,
-                               glestext->size, 0);
+                               PGM_IMAGE_RGBA, glestext->width,
+                               glestext->height, glestext->size, 0, TRUE);
   set_coordinates (glestext);
 
   /* And request a texture generation */
@@ -686,14 +812,39 @@ update_pixmap (PgmGlesText *glestext)
   PgmGlesDrawable *glesdrawable = PGM_GLES_DRAWABLE (glestext);
   PgmGlesContextTask *task;
 
-  /* Make context fully transparent */
+  /* Clear the surface */
+  g_assert (glestext->cairo_ctx != NULL);
   cairo_set_operator (glestext->cairo_ctx, CAIRO_OPERATOR_SOURCE);
   cairo_set_source_rgba (glestext->cairo_ctx, 0.0f, 0.0f, 0.0f, 0.0f);
   cairo_paint (glestext->cairo_ctx);
 
-  /* Then draw the text */
-  cairo_set_source_rgba (glestext->cairo_ctx, 1.0f, 1.0f, 1.0f, 1.0f);
-  pango_cairo_show_layout (glestext->cairo_ctx, glestext->layout);
+  /* Draw the text without shadow */
+  if (!glestext->cast_shadow)
+    {
+      cairo_set_source_rgba (glestext->cairo_ctx, glestext->color[0],
+                             glestext->color[1], glestext->color[2],
+                             glestext->color[3]);
+      pango_cairo_show_layout (glestext->cairo_ctx, glestext->layout);
+    }
+  /* Draw the text with a shadow */
+  else
+    {
+      gdouble tx = glestext->shadow_offset * glestext->shadow_latitude_factor;
+      gdouble ty = glestext->shadow_offset * glestext->shadow_longitude_factor;
+
+      cairo_set_source_rgba (glestext->cairo_ctx, glestext->shadow_color[0],
+                             glestext->shadow_color[1],
+                             glestext->shadow_color[2],
+                             glestext->shadow_color[3]);
+      cairo_save (glestext->cairo_ctx);
+      cairo_translate (glestext->cairo_ctx, tx, ty);
+      pango_cairo_show_layout (glestext->cairo_ctx, glestext->layout);
+      cairo_translate (glestext->cairo_ctx, -tx, -ty);
+      cairo_set_source_rgba (glestext->cairo_ctx, glestext->color[0],
+                             glestext->color[1], glestext->color[2],
+                             glestext->color[3]);
+      pango_cairo_show_layout (glestext->cairo_ctx, glestext->layout);
+    }
 
   /* And the stroke */
   if (glestext->outline_width > 0.0f)
@@ -724,7 +875,7 @@ update_pixmap (PgmGlesText *glestext)
   /* Update texture */
  pgm_gles_texture_set_buffer (glestext->texture, glestext->buffer,
                               PGM_IMAGE_BGRA, glestext->width, glestext->height,
-                              glestext->size, 0);
+                              glestext->size, 0, TRUE);
 
   /* Request texture upload */
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_UPLOAD_TEXTURE,
@@ -766,8 +917,6 @@ pgm_gles_text_draw (PgmGlesDrawable *glesdrawable)
   static PgmGlesUshort indices[] = { 0, 1, 2, 2, 3, 0 };
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
 
-  GST_LOG_OBJECT (glesdrawable, "draw");
-
   /* Only draw if it's visible */
   if (glestext->fg_color[3] != 0.0f)
     {
@@ -795,8 +944,6 @@ pgm_gles_text_regenerate (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
 
-  GST_DEBUG_OBJECT (glesdrawable, "regenerate");
-
   if (glestext->size_updated)
     {
       regenerate_text (glestext);
@@ -809,8 +956,6 @@ pgm_gles_text_update_projection (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
 
-  GST_DEBUG_OBJECT (glesdrawable, "update_projection");
-
   glestext->size_updated = TRUE;
 }
 
@@ -818,9 +963,7 @@ static void
 pgm_gles_text_set_size (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
-  guint i;
-
-  GST_DEBUG_OBJECT (glesdrawable, "set_size");
+  guint32 i;
 
   for (i = 0; i < 12; i++)
     glestext->vertex[i] = glesdrawable->bg_vertex[i];
@@ -834,8 +977,6 @@ pgm_gles_text_set_position (PgmGlesDrawable *glesdrawable)
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
   guint i;
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_position");
-
   for (i = 0; i < 12; i++)
     glestext->vertex[i] = glesdrawable->bg_vertex[i];
 }
@@ -844,11 +985,10 @@ static void
 pgm_gles_text_set_fg_color (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
+  PgmGlesFloat *fg_color = glestext->fg_color;
   PgmDrawable *drawable;
   PgmGlesFloat color[4];
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_fg_color");
-
   drawable = glesdrawable->drawable;
 
   color[0] = drawable->fg_r * INV_255;
@@ -857,22 +997,22 @@ pgm_gles_text_set_fg_color (PgmGlesDrawable *glesdrawable)
   color[3] = drawable->fg_a * drawable->opacity * INV_255;
 
   GST_OBJECT_LOCK (drawable);
-  glestext->fg_color[0] = color[0];
-  glestext->fg_color[1] = color[1];
-  glestext->fg_color[2] = color[2];
-  glestext->fg_color[3] = color[3];
-  glestext->fg_color[4] = color[0];
-  glestext->fg_color[5] = color[1];
-  glestext->fg_color[6] = color[2];
-  glestext->fg_color[7] = color[3];
-  glestext->fg_color[8] = color[0];
-  glestext->fg_color[9] = color[1];
-  glestext->fg_color[10] = color[2];
-  glestext->fg_color[11] = color[3];
-  glestext->fg_color[12] = color[0];
-  glestext->fg_color[13] = color[1];
-  glestext->fg_color[14] = color[2];
-  glestext->fg_color[15] = color[3];
+  fg_color[0] = color[0];
+  fg_color[1] = color[1];
+  fg_color[2] = color[2];
+  fg_color[3] = color[3];
+  fg_color[4] = color[0];
+  fg_color[5] = color[1];
+  fg_color[6] = color[2];
+  fg_color[7] = color[3];
+  fg_color[8] = color[0];
+  fg_color[9] = color[1];
+  fg_color[10] = color[2];
+  fg_color[11] = color[3];
+  fg_color[12] = color[0];
+  fg_color[13] = color[1];
+  fg_color[14] = color[2];
+  fg_color[15] = color[3];
   GST_OBJECT_UNLOCK (drawable);
 }
 
@@ -883,8 +1023,6 @@ pgm_gles_text_set_opacity (PgmGlesDrawable *glesdrawable)
   PgmDrawable *drawable;
   PgmGlesFloat alpha;
 
-  GST_DEBUG_OBJECT (glesdrawable, "set_opacity");
-
   drawable = glesdrawable->drawable;
   alpha = drawable->fg_a * drawable->opacity * SQR_INV_255;
 
@@ -901,8 +1039,6 @@ pgm_gles_text_sync (PgmGlesDrawable *glesdrawable)
 {
   PgmGlesText *glestext = PGM_GLES_TEXT (glesdrawable);
 
-  GST_DEBUG_OBJECT (glesdrawable, "sync");
-
   create_pixmap (glestext);
   update_properties (glestext);
 
@@ -915,7 +1051,7 @@ pgm_gles_text_sync (PgmGlesDrawable *glesdrawable)
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlesText, pgm_gles_text, PGM_TYPE_GLES_DRAWABLE);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlesText, pgm_gles_text, PGM_TYPE_GLES_DRAWABLE)
 
 void
 pgm_gles_text_register (GTypeModule *module)
@@ -930,8 +1066,6 @@ pgm_gles_text_dispose (GObject *object)
   PgmGlesText *glestext = PGM_GLES_TEXT (object);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glestext, "dispose");
-
   free_pixmap (glestext);
 
   pango_font_description_free (glestext->desc);
@@ -993,15 +1127,14 @@ pgm_gles_text_class_finalize (PgmGlesTextClass *klass)
 static void
 pgm_gles_text_init (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "init");
-
   glestext->font_options = cairo_font_options_create ();
   cairo_font_options_set_antialias (glestext->font_options,
                                     CAIRO_ANTIALIAS_SUBPIXEL);
 
+  glestext->multiline = TRUE;
   glestext->desc = pango_font_description_new ();
 
-  glestext->size_updated = FALSE;
+  /* glestext->size_updated = FALSE; */
 }
 
 /* Public methods */
@@ -1018,6 +1151,7 @@ pgm_gles_text_new (PgmDrawable *drawable,
   GST_DEBUG_OBJECT (glestext, "created new glestext");
 
   glestext->texture = pgm_gles_texture_new (glesviewport->context);
+  glestext->texture->filter = PGM_GLES_NEAREST;
 
   glesdrawable = PGM_GLES_DRAWABLE (glestext);
   glesdrawable->drawable = gst_object_ref (drawable);
@@ -1031,8 +1165,6 @@ pgm_gles_text_new (PgmDrawable *drawable,
 void
 pgm_gles_text_set_label (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_label");
-
   set_label (glestext);
   update_text (glestext);
 }
@@ -1040,8 +1172,6 @@ pgm_gles_text_set_label (PgmGlesText *glestext)
 void
 pgm_gles_text_set_markup (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_markup");
-
   set_label (glestext);
   update_text (glestext);
 }
@@ -1049,8 +1179,6 @@ pgm_gles_text_set_markup (PgmGlesText *glestext)
 void
 pgm_gles_text_set_font_family (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_font_family");
-
   set_font_family (glestext);
   update_text (glestext);
 }
@@ -1058,8 +1186,6 @@ pgm_gles_text_set_font_family (PgmGlesText *glestext)
 void
 pgm_gles_text_set_font_height (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_font_height");
-
   set_font_height (glestext);
   update_text (glestext);
 }
@@ -1067,17 +1193,15 @@ pgm_gles_text_set_font_height (PgmGlesText *glestext)
 void
 pgm_gles_text_set_ellipsize (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_ellipsize");
-
   set_ellipsize (glestext);
+  set_width (glestext);
+  set_coordinates (glestext);
   update_text (glestext);
 }
 
 void
 pgm_gles_text_set_justify (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_justify");
-
   set_justify (glestext);
   update_text (glestext);
 }
@@ -1085,8 +1209,6 @@ pgm_gles_text_set_justify (PgmGlesText *glestext)
 void
 pgm_gles_text_set_alignment (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_alignment");
-
   set_alignment (glestext);
   update_text (glestext);
 }
@@ -1094,8 +1216,6 @@ pgm_gles_text_set_alignment (PgmGlesText *glestext)
 void
 pgm_gles_text_set_wrap (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_wrap");
-
   set_wrap (glestext);
   update_text (glestext);
 }
@@ -1103,8 +1223,6 @@ pgm_gles_text_set_wrap (PgmGlesText *glestext)
 void
 pgm_gles_text_set_gravity (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_gravity");
-
   set_gravity (glestext);
   update_text (glestext);
 }
@@ -1112,8 +1230,6 @@ pgm_gles_text_set_gravity (PgmGlesText *glestext)
 void
 pgm_gles_text_set_stretch (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_stretch");
-
   set_stretch (glestext);
   update_text (glestext);
 }
@@ -1121,8 +1237,6 @@ pgm_gles_text_set_stretch (PgmGlesText *glestext)
 void
 pgm_gles_text_set_style (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_style");
-
   set_style (glestext);
   update_text (glestext);
 }
@@ -1130,8 +1244,6 @@ pgm_gles_text_set_style (PgmGlesText *glestext)
 void
 pgm_gles_text_set_variant (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_variant");
-
   set_variant (glestext);
   update_text (glestext);
 }
@@ -1139,26 +1251,36 @@ pgm_gles_text_set_variant (PgmGlesText *glestext)
 void
 pgm_gles_text_set_weight (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_weight");
-
   set_weight (glestext);
   update_text (glestext);
 }
 
+void
+pgm_gles_text_set_multiline (PgmGlesText *glestext)
+{
+  set_multiline (glestext);
+  set_width (glestext);
+  set_coordinates (glestext);
+  update_text (glestext);
+}
+
 void
 pgm_gles_text_set_line_spacing (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_line_spacing");
-
   set_line_spacing (glestext);
   update_text (glestext);
 }
 
+void
+pgm_gles_text_set_color (PgmGlesText *glestext)
+{
+  set_color (glestext);
+  update_text (glestext);
+}
+
 void
 pgm_gles_text_set_outline_color (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_outline_color");
-
   set_outline_color (glestext);
   update_text (glestext);
 }
@@ -1166,8 +1288,27 @@ pgm_gles_text_set_outline_color (PgmGlesText *glestext)
 void
 pgm_gles_text_set_outline_width (PgmGlesText *glestext)
 {
-  GST_DEBUG_OBJECT (glestext, "set_outline_width");
-
   set_outline_width (glestext);
   update_text (glestext);
 }
+
+void
+pgm_gles_text_set_cast_shadow (PgmGlesText *glestext)
+{
+  set_cast_shadow (glestext);
+  update_text (glestext);
+}
+
+void
+pgm_gles_text_set_shadow_position (PgmGlesText *glestext)
+{
+  set_shadow_position (glestext);
+  update_text (glestext);
+}
+
+void
+pgm_gles_text_set_shadow_color (PgmGlesText *glestext)
+{
+  set_shadow_color (glestext);
+  update_text (glestext);
+}
