@@ -154,7 +154,11 @@ free_buffer (PgmGlesTexture *glestexture)
 
     case PGM_GLES_TEXTURE_BUFFER:
       if (glestexture->data.buffer)
-        glestexture->data.buffer = NULL;
+        {
+          if (!glestexture->shared)
+            g_free (glestexture->data.buffer);
+          glestexture->data.buffer = NULL;
+        }
       break;
 
     default:
@@ -194,6 +198,8 @@ init_texture (PgmGlesTexture *glestexture)
 
   /* Indentifiants */
   glestexture->id = NULL;
+
+  glestexture->shared = 0;
 }
 
 /* Destructor */
@@ -252,12 +258,22 @@ pgm_gles_texture_set_buffer (PgmGlesTexture *glestexture,
                              guint width,
                              guint height,
                              guint size,
-                             guint stride)
+                             guint stride,
+                             gboolean share)
 {
   free_buffer (glestexture);
 
   glestexture->storage = PGM_GLES_TEXTURE_BUFFER;
-  glestexture->data.buffer = buffer;
+  if (share)
+    {
+      glestexture->data.buffer = buffer;
+      glestexture->shared = 1;
+    }
+  else
+    {
+      glestexture->data.buffer = g_memdup (buffer, size);
+      glestexture->shared = 0;
+    }
   glestexture->width = width;
   glestexture->height = height;
   glestexture->stride = stride;
