@@ -95,6 +95,9 @@ struct _PgmGlesTexture {
 
   /* Identifiant(s) */
   guint *id;
+
+  /* Whether the buffer is shared */
+  guint8 shared;
 };
 
 /* create a new texture */
@@ -110,7 +113,8 @@ void            pgm_gles_texture_set_buffer        (PgmGlesTexture *glestexture,
                                                     guint width,
                                                     guint height,
                                                     guint size,
-                                                    guint stride);
+                                                    guint stride,
+                                                    gboolean share);
 
 /* Set a GdkPixbuf */
 void            pgm_gles_texture_set_pixbuf        (PgmGlesTexture *glestexture,
