@@ -133,6 +133,37 @@ static PgmViewportClass *parent_class = NULL;
 
 /* Private functions */
 
+/* Copy a layer taking a ref on each drawable */
+static GList*
+copy_layer (GList *layer)
+{
+  GList *new_layer = NULL;
+
+  if (layer)
+    {
+      GList *last;
+
+      new_layer = g_slice_new (GList);
+      new_layer->data = g_object_ref (layer->data);
+      new_layer->prev = NULL;
+      last = new_layer;
+      layer = layer->next;
+
+      while (layer)
+        {
+          last->next = g_slice_new (GList);
+          last->next->prev = last;
+          last = last->next;
+          last->data = g_object_ref (layer->data);
+          layer = layer->next;
+        }
+
+      last->next = NULL;
+    }
+
+  return new_layer;
+}
+
 /* Create a change task */
 static Task*
 task_change_new (PgmGlesDrawable *glesdrawable,
@@ -329,6 +360,9 @@ gles_drawable_free (PgmGlesViewport *glesviewport,
   GSList *walk, *tmp;
   Task *task;
 
+  if (glesdrawable == NULL)
+    return;
+
   /* Disconnect from the 'changed' signal and remove it from the hash */
   GST_OBJECT_LOCK (glesviewport);
   GST_OBJECT_LOCK (glesdrawable->drawable);
@@ -513,22 +547,42 @@ do_task_remove (PgmGlesViewport *glesviewport,
   task_remove_free (task);
 }
 
+/* Regenerate a layer */
+static void
+regenerate_layer (GList *layer)
+{
+  GList *walk = layer;
+
+  while (walk)
+    {
+      PgmGlesDrawable *glesdrawable = (PgmGlesDrawable*) walk->data;
+      pgm_gles_drawable_regenerate (glesdrawable);
+      g_object_unref (glesdrawable);
+      walk = walk->next;
+    }
+
+  g_list_free (layer);
+}
+
 /* Complete a regenerate task and free it */
 static void
 do_task_regenerate (PgmGlesViewport *glesviewport,
                     Task *task)
 {
+  GList *near_layer, *middle_layer, *far_layer;
+
+  /* Get copies of the layers */
   g_mutex_lock (glesviewport->layer_lock);
-
-  g_list_foreach (glesviewport->near_layer,
-                  (GFunc) pgm_gles_drawable_regenerate, NULL);
-  g_list_foreach (glesviewport->middle_layer,
-                  (GFunc) pgm_gles_drawable_regenerate, NULL);
-  g_list_foreach (glesviewport->far_layer,
-                  (GFunc) pgm_gles_drawable_regenerate, NULL);
-
+  near_layer = copy_layer (glesviewport->near_layer);
+  middle_layer = copy_layer (glesviewport->middle_layer);
+  far_layer = copy_layer (glesviewport->far_layer);
   g_mutex_unlock (glesviewport->layer_lock);
 
+  /* Then regenerate each drawable of each layer */
+  regenerate_layer (far_layer);
+  regenerate_layer (middle_layer);
+  regenerate_layer (near_layer);
+
   task_any_free (task);
 }
 
@@ -570,8 +624,6 @@ drawable_added_cb (PgmCanvas *canvas,
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "drawable_added_cb");
-
   /* Add the add task in the update queue */
   task = task_add_new (drawable, layer, order);
   g_mutex_lock (glesviewport->update_lock);
@@ -593,8 +645,6 @@ drawable_removed_cb (PgmCanvas *canvas,
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "drawable_removed_cb");
-
   /* Add the remove task in the update queue */
   task = task_remove_new (drawable, layer);
   g_mutex_lock (glesviewport->update_lock);
@@ -617,8 +667,6 @@ drawable_reordered_cb (PgmCanvas *canvas,
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "drawable_reordered_cb");
-
   /* Add the reorder task in the update queue */
   task = task_reorder_new (drawable, layer, order);
   g_mutex_lock (glesviewport->update_lock);
@@ -638,8 +686,6 @@ regenerated_cb (PgmCanvas *canvas,
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (data);
   Task *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "drawable_regenerated_cb");
-
   /* Add the regenerate task in the update queue */
   task = task_any_new (TASK_REGENERATE);
   g_mutex_lock (glesviewport->update_lock);
@@ -718,16 +764,29 @@ sync_gles_drawable_from_canvas (PgmGlesViewport *glesviewport,
     }
 }
 
+/* Update a layer projection */
+static void
+update_layer_projection (GList *layer)
+{
+  GList *walk = layer;
+
+  while (walk)
+    {
+      PgmGlesDrawable *glesdrawable = (PgmGlesDrawable*) walk->data;
+      pgm_gles_drawable_update_projection (glesdrawable);
+      g_object_unref (glesdrawable);
+      walk = walk->next;
+    }
+
+  g_list_free (layer);
+}
+
 /* PgmViewport methods */
 
 static PgmError
 pgm_gles_viewport_set_title (PgmViewport *viewport,
                              const gchar *title)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_title");
-
   return PGM_ERROR_X;
 }
 
@@ -737,8 +796,6 @@ pgm_gles_viewport_show (PgmViewport *viewport)
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "show");
-
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_VISIBILITY, NULL);
   pgm_gles_context_push_immediate_task (glesviewport->context, task);
 
@@ -751,8 +808,6 @@ pgm_gles_viewport_hide (PgmViewport *viewport)
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "hide");
-
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_VISIBILITY, NULL);
   pgm_gles_context_push_immediate_task (glesviewport->context, task);
 
@@ -763,10 +818,6 @@ static PgmError
 pgm_gles_viewport_set_decorated (PgmViewport *viewport,
                                  gboolean decorated)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_decorated");
-
   return PGM_ERROR_X;
 }
 
@@ -774,10 +825,6 @@ static PgmError
 pgm_gles_viewport_set_cursor (PgmViewport *viewport,
                               PgmViewportCursor cursor)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_cursor");
-
   return PGM_ERROR_X;
 }
 
@@ -785,10 +832,6 @@ static PgmError
 pgm_gles_viewport_set_icon (PgmViewport *viewport,
                             GdkPixbuf *icon)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_icon");
-
   return PGM_ERROR_X;
 }
 
@@ -796,10 +839,6 @@ static PgmError
 pgm_gles_viewport_set_drag_status (PgmViewport *viewport,
                                    gboolean accept)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_drag_status");
-
   return PGM_ERROR_X;
 }
 
@@ -808,11 +847,21 @@ pgm_gles_viewport_set_size (PgmViewport *viewport,
                             gint width,
                             gint height)
 {
+  return PGM_ERROR_X;
+}
+
+static PgmError
+pgm_gles_viewport_set_screen_resolution (PgmViewport *viewport,
+                            gint width,
+                            gint height)
+{
+  PgmGlesContextTask *task;
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_DEBUG_OBJECT (glesviewport, "set_size");
+  task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_RESOLUTION, NULL);
+  pgm_gles_context_push_immediate_task (glesviewport->context, task);
 
-  return PGM_ERROR_X;
+  return PGM_ERROR_OK;
 }
 
 static PgmError
@@ -822,8 +871,6 @@ pgm_gles_viewport_set_alpha_blending (PgmViewport *viewport,
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "set_alpha_blending");
-
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_ALPHA_BLENDING, NULL);
   pgm_gles_context_push_immediate_task (glesviewport->context, task);
 
@@ -834,10 +881,6 @@ static PgmError
 pgm_gles_viewport_set_opacity (PgmViewport *viewport,
                                guchar opacity)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_opacity");
-
   return PGM_ERROR_X;
 }
 
@@ -845,10 +888,6 @@ static PgmError
 pgm_gles_viewport_set_fullscreen (PgmViewport *viewport,
                                   gboolean fullscreen)
 {
-  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
-
-  GST_DEBUG_OBJECT (glesviewport, "set_fullscreen");
-
   return PGM_ERROR_X;
 }
 
@@ -859,8 +898,6 @@ pgm_gles_viewport_get_screen_resolution (PgmViewport *viewport,
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glesviewport, "get_screen_resolution");
-
   pgm_gles_backend_get_screen_resolution (glesviewport->context->backend, width,
                                           height);
 
@@ -874,8 +911,6 @@ pgm_gles_viewport_get_screen_size_mm (PgmViewport *viewport,
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glesviewport, "get_screen_size_mm");
-
   pgm_gles_backend_get_screen_size_mm (glesviewport->context->backend, width,
                                        height);
 
@@ -888,8 +923,6 @@ pgm_gles_viewport_set_canvas (PgmViewport *viewport,
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_DEBUG_OBJECT (glesviewport, "set_canvas");
-
   /* There's a canvas bound, let's clear associated data */
   if (glesviewport->canvas)
     {
@@ -963,8 +996,33 @@ pgm_gles_viewport_update_projection (PgmViewport *viewport)
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
   PgmGlesContextTask *task;
 
-  GST_DEBUG_OBJECT (glesviewport, "update_projection");
+  GST_OBJECT_LOCK (viewport);
 
+  /* Store the projected size  */
+  if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
+      || viewport->rotation == PGM_VIEWPORT_ROTATION_180)
+    {
+      glesviewport->projected_w = (gfloat) viewport->projected_width;
+      glesviewport->projected_h = (gfloat) viewport->projected_height;
+    }
+  else
+    {
+      glesviewport->projected_w = (gfloat) viewport->projected_height;
+      glesviewport->projected_h = (gfloat) viewport->projected_width;
+    }
+
+  /* Store the canvas size */
+  if (viewport->canvas)
+    {
+      GST_OBJECT_LOCK (viewport->canvas);
+      glesviewport->canvas_w = viewport->canvas->width;
+      glesviewport->canvas_h = viewport->canvas->height;
+      GST_OBJECT_UNLOCK (viewport->canvas);
+    }
+
+  GST_OBJECT_UNLOCK (viewport);
+
+  /* And queue the projection update task */
   task = pgm_gles_context_task_new (PGM_GLES_CONTEXT_PROJECTION, NULL);
   pgm_gles_context_push_immediate_task (glesviewport->context, task);
 
@@ -984,8 +1042,6 @@ pgm_gles_viewport_get_pixel_formats (PgmViewport *viewport,
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glesviewport, "get_pixel_formats");
-
   GST_OBJECT_LOCK (glesviewport);
 
   /* The classic formats supported by all hardware */
@@ -1007,8 +1063,6 @@ pgm_gles_viewport_get_caps_mask (PgmViewport *viewport,
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glesviewport, "get_caps_mask");
-
   GST_OBJECT_LOCK (glesviewport);
   *caps_mask = glesviewport->capacities;
   GST_OBJECT_UNLOCK (glesviewport);
@@ -1016,14 +1070,25 @@ pgm_gles_viewport_get_caps_mask (PgmViewport *viewport,
   return PGM_ERROR_OK;
 }
 
+static PgmError
+pgm_gles_viewport_get_max_texture_size (PgmViewport *viewport,
+                                        guint32 *max_texture_size)
+{
+  PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
+
+  GST_OBJECT_LOCK (glesviewport);
+  *max_texture_size = glesviewport->context->max_texture_size;
+  GST_OBJECT_UNLOCK (glesviewport);
+
+  return PGM_ERROR_OK;
+}
+
 static PgmError
 pgm_gles_viewport_get_frame_rate (PgmViewport *viewport,
                                   guint *frame_rate)
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (viewport);
 
-  GST_LOG_OBJECT (glesviewport, "get_frame_rate");
-
   GST_OBJECT_LOCK (glesviewport);
   *frame_rate = glesviewport->context->fps;
   GST_OBJECT_UNLOCK (glesviewport);
@@ -1043,8 +1108,6 @@ pgm_gles_viewport_read_pixels (PgmViewport *viewport,
   PgmGlesViewportPixelRectangle *rectangle = NULL;
   PgmGlesContextTask *task = NULL;
 
-  GST_DEBUG_OBJECT (glesviewport, "read_pixels");
-
   rectangle = g_slice_new (PgmGlesViewportPixelRectangle);
   if (!rectangle)
     return PGM_ERROR_X;
@@ -1063,7 +1126,7 @@ pgm_gles_viewport_read_pixels (PgmViewport *viewport,
 
 /* GObject stuff */
 
-PGM_DEFINE_DYNAMIC_TYPE (PgmGlesViewport, pgm_gles_viewport, PGM_TYPE_VIEWPORT);
+PGM_DEFINE_DYNAMIC_TYPE (PgmGlesViewport, pgm_gles_viewport, PGM_TYPE_VIEWPORT)
 
 void
 pgm_gles_viewport_register (GTypeModule *module)
@@ -1076,8 +1139,6 @@ pgm_gles_viewport_dispose (GObject *object)
 {
   PgmGlesViewport *glesviewport = PGM_GLES_VIEWPORT (object);
 
-  GST_DEBUG_OBJECT (glesviewport, "dispose");
-
   if (glesviewport->canvas)
     {
       /* Disconnect handlers */
@@ -1151,6 +1212,8 @@ pgm_gles_viewport_class_init (PgmGlesViewportClass *klass)
     GST_DEBUG_FUNCPTR (pgm_gles_viewport_set_fullscreen);
   viewport_class->get_screen_resolution =
     GST_DEBUG_FUNCPTR (pgm_gles_viewport_get_screen_resolution);
+  viewport_class->set_screen_resolution =
+    GST_DEBUG_FUNCPTR (pgm_gles_viewport_set_screen_resolution);
   viewport_class->get_screen_size_mm =
     GST_DEBUG_FUNCPTR (pgm_gles_viewport_get_screen_size_mm);
   viewport_class->set_canvas = GST_DEBUG_FUNCPTR (pgm_gles_viewport_set_canvas);
@@ -1162,6 +1225,8 @@ pgm_gles_viewport_class_init (PgmGlesViewportClass *klass)
     GST_DEBUG_FUNCPTR (pgm_gles_viewport_get_pixel_formats);
   viewport_class->get_caps_mask =
     GST_DEBUG_FUNCPTR (pgm_gles_viewport_get_caps_mask);
+  viewport_class->get_max_texture_size =
+    GST_DEBUG_FUNCPTR (pgm_gles_viewport_get_max_texture_size);
   viewport_class->get_frame_rate =
     GST_DEBUG_FUNCPTR (pgm_gles_viewport_get_frame_rate);
   viewport_class->read_pixels =
@@ -1244,12 +1309,22 @@ pgm_gles_viewport_class_init (PgmGlesViewportClass *klass)
     GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_variant);
   klass->changed_func[PGM_TEXT_WEIGHT] =
     GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_weight);
+  klass->changed_func[PGM_TEXT_MULTILINE] =
+    GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_multiline);
   klass->changed_func[PGM_TEXT_LINE_SPACING] =
     GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_line_spacing);
+  klass->changed_func[PGM_TEXT_COLOR] =
+    GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_color);
   klass->changed_func[PGM_TEXT_OUTLINE_COLOR] =
     GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_outline_color);
   klass->changed_func[PGM_TEXT_OUTLINE_WIDTH] =
     GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_outline_width);
+  klass->changed_func[PGM_TEXT_CAST_SHADOW] =
+    GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_cast_shadow);
+  klass->changed_func[PGM_TEXT_SHADOW_POSITION] =
+    GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_shadow_position);
+  klass->changed_func[PGM_TEXT_SHADOW_COLOR] =
+    GST_DEBUG_FUNCPTR ((PgmGlesDrawableChangedFunc) pgm_gles_text_set_shadow_color);
 }
 
 static void
@@ -1261,19 +1336,17 @@ pgm_gles_viewport_class_finalize (PgmGlesViewportClass *klass)
 static void
 pgm_gles_viewport_init (PgmGlesViewport *glesviewport)
 {
-  GST_DEBUG_OBJECT (glesviewport, "init");
-
   /* Attached canvas */
-  glesviewport->canvas = NULL;
+  /* glesviewport->canvas = NULL; */
 
   /* Empty lists */
-  glesviewport->far_layer = NULL;
-  glesviewport->middle_layer = NULL;
-  glesviewport->near_layer = NULL;
+  /* glesviewport->far_layer = NULL; */
+  /* glesviewport->middle_layer = NULL; */
+  /* glesviewport->near_layer = NULL; */
   glesviewport->layer_lock = g_mutex_new ();
 
   /* Update task queue */
-  glesviewport->update_queue = NULL;
+  /* glesviewport->update_queue = NULL; */
   glesviewport->update_lock = g_mutex_new ();
 
   /* Create the drawable hash using pointers as keys */
@@ -1282,6 +1355,18 @@ pgm_gles_viewport_init (PgmGlesViewport *glesviewport)
   /* Create context */
   glesviewport->context = pgm_gles_context_new (glesviewport);
 
+  /* Signal handlers id */
+  /* glesviewport->add_handler = 0; */
+  /* glesviewport->remove_handler = 0; */
+  /* glesviewport->reorder_handler = 0; */
+  /* glesviewport->regenerated_handler = 0; */
+
+  /* Projection informations */
+  glesviewport->projected_w = 800;
+  glesviewport->projected_h = 600;
+  glesviewport->canvas_w = 4.0f;
+  glesviewport->canvas_h = 3.0f;
+
   /* Fill capacities */
   glesviewport->capacities = PGM_VIEWPORT_HARDWARE_ACCELERATION;
 #ifdef TSLIB
@@ -1289,7 +1374,7 @@ pgm_gles_viewport_init (PgmGlesViewport *glesviewport)
 #endif /* TSLIB */
 
   /* Drag status */
-  glesviewport->drag_status = FALSE;
+  /* glesviewport->drag_status = FALSE; */
 }
 
 /* Public methods */
@@ -1346,6 +1431,24 @@ pgm_gles_viewport_update_drawable_projection (PgmGlesViewport *glesviewport)
   g_mutex_unlock (glesviewport->layer_lock);
 }
 
+void
+pgm_gles_viewport_update_drawable_projection (PgmGlesViewport *glesviewport)
+{
+  GList *near_layer, *middle_layer, *far_layer;
+
+  /* Get copies of the layers */
+  g_mutex_lock (glesviewport->layer_lock);
+  near_layer = copy_layer (glesviewport->near_layer);
+  middle_layer = copy_layer (glesviewport->middle_layer);
+  far_layer = copy_layer (glesviewport->far_layer);
+  g_mutex_unlock (glesviewport->layer_lock);
+
+  /* Then update the projection of each drawable of each layer */
+  update_layer_projection (far_layer);
+  update_layer_projection (middle_layer);
+  update_layer_projection (near_layer);
+}
+
 /* Connects the GLES drawable to the "changed" signal. Should be called at the
  * creation of a new GLES drawable to be notified of any change. */
 void
