@@ -47,7 +47,7 @@ G_BEGIN_DECLS
   (G_TYPE_INSTANCE_GET_CLASS((obj), PGM_TYPE_GLES_VIEWPORT, PgmGlesViewportClass))
 
 /* FIXME: Hardly maintainable */
-#define PROPERTY_CHANGED_COUNT 41
+#define PROPERTY_CHANGED_COUNT 46
 
 typedef void (*PgmGlesDrawableChangedFunc) (PgmGlesDrawable *glesdrawable);
 
@@ -57,27 +57,33 @@ struct _PgmGlesViewport {
   /* Canvas bound */
   PgmCanvas *canvas;
 
-  /* Signal handler IDs */
-  gulong add_handler;
-  gulong remove_handler;
-  gulong reorder_handler;
-  gulong regenerated_handler;
-
   /* PgmGlDrawable layer lists */
   GList  *far_layer;
   GList  *middle_layer;
   GList  *near_layer;
   GMutex *layer_lock;
 
+  /* Update task queue */
+  GSList *update_queue;
+  GMutex *update_lock;
+
   /* PgmDrawable/PgmGlesDrawable lookup hash */
   GHashTable *drawable_hash;
 
   /* OpenGL ES context */
   PgmGlesContext *context;
 
-  /* Update task queue */
-  GSList *update_queue;
-  GMutex *update_lock;
+  /* Signal handler IDs */
+  gulong add_handler;
+  gulong remove_handler;
+  gulong reorder_handler;
+  gulong regenerated_handler;
+
+  /* Projection informations */
+  gfloat projected_w;
+  gfloat projected_h;
+  gfloat canvas_w;
+  gfloat canvas_h;
 
   /* Capacities mask */
   gulong capacities;
