@@ -342,8 +342,6 @@ dispatch_x_events (PgmGlesBackend *glesbackend)
   PgmGlesViewport *glesviewport = glesbackend->context->glesviewport;
   PgmViewport *viewport = PGM_VIEWPORT (glesviewport);
 
-  GST_LOG_OBJECT (glesxbackend, "dispatch_x_events");
-
   if (!XPending (glesxbackend->xdisplay))
     return TRUE;
 
@@ -720,8 +718,6 @@ pgm_gles_x_backend_set_visibility (PgmGlesBackend *glesbackend,
 {
   PgmGlesXBackend *glesxbackend = PGM_GLES_X_BACKEND (glesbackend);
 
-  GST_DEBUG_OBJECT (glesxbackend, "set_visibility");
-
   if (visible)
     XMapWindow (glesxbackend->xdisplay, glesxbackend->win);
   else
@@ -737,8 +733,6 @@ pgm_gles_x_backend_swap_buffers (PgmGlesBackend *glesbackend)
 {
   PgmGlesXBackend *glesxbackend = PGM_GLES_X_BACKEND (glesbackend);
 
-  GST_LOG_OBJECT (glesxbackend, "swap_buffers");
-
   eglSwapBuffers (glesxbackend->egldisplay, glesxbackend->surface);
 }
 
@@ -756,8 +750,6 @@ pgm_gles_x_backend_get_screen_size_mm (PgmGlesBackend *glesbackend,
 {
   PgmGlesXBackend *glesxbackend = PGM_GLES_X_BACKEND (glesbackend);
 
-  GST_LOG_OBJECT (glesxbackend, "get_screen_size_mm");
-
   *width = glesxbackend->size_mm_width;
   *height = glesxbackend->size_mm_height;
 }
@@ -769,8 +761,6 @@ pgm_gles_x_backend_get_screen_resolution (PgmGlesBackend *glesbackend,
 {
   PgmGlesXBackend *glesxbackend = PGM_GLES_X_BACKEND (glesbackend);
 
-  GST_LOG_OBJECT (glesxbackend, "get_screen_resolution");
-
   *width = glesxbackend->resolution_width;
   *height = glesxbackend->resolution_height;
 }
@@ -778,15 +768,13 @@ pgm_gles_x_backend_get_screen_resolution (PgmGlesBackend *glesbackend,
 /* GObject stuff */
 
 PGM_DEFINE_DYNAMIC_TYPE (PgmGlesXBackend, pgm_gles_x_backend,
-                         PGM_TYPE_GLES_BACKEND);
+                         PGM_TYPE_GLES_BACKEND)
 
 static void
 pgm_gles_x_backend_dispose (GObject *object)
 {
   PgmGlesXBackend *glesxbackend = PGM_GLES_X_BACKEND (object);
 
-  GST_DEBUG_OBJECT (glesxbackend, "dispose");
-
   if (glesxbackend->created)
     pgm_gles_x_backend_destroy_window (PGM_GLES_BACKEND (glesxbackend));
 
@@ -836,8 +824,6 @@ pgm_gles_x_backend_class_finalize (PgmGlesXBackendClass *klass)
 static void
 pgm_gles_x_backend_init (PgmGlesXBackend *glesxbackend)
 {
-  GST_DEBUG_OBJECT (glesxbackend, "init");
-
   glesxbackend->atom = NULL;
   glesxbackend->created = FALSE;
 }
