@@ -2,7 +2,7 @@
  *
  * Pigment unit test for pgmcanvas.c
  *
- * Copyright © 2006, 2007 Fluendo Embedded S.L.
+ * Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,22 +18,20 @@
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * FIXME: pgm_canvas_regenerate
  */
 
-#include <gst/check/gstcheck.h>
+#include <tests/check/common/pgmcheck.h>
 #include <pgm/pgm.h>
 
 /* test drawable floating reference on canvas_add/del_* */
-GST_START_TEST (test_canvas_add_del_drawable)
+PGM_START_TEST (test_canvas_add_del_drawable)
 {
   PgmError ret;
-  gboolean success;
   PgmCanvas *canvas;
   PgmDrawable *drawable;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
 
   /* create a new floating drawable with refcount 1 */
@@ -61,21 +59,79 @@ GST_START_TEST (test_canvas_add_del_drawable)
   /* clean up our own reference */
   gst_object_unref (GST_OBJECT_CAST (drawable));
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
+
+PGM_START_TEST (test_canvas_add_del_many)
+{
+  PgmCanvas *canvas = NULL;
+  PgmDrawable *drawables[10] = {0};
+  guint i = 0;
+
+  canvas = pgm_canvas_new ();
+  fail_if (NULL == canvas, "canvas creation failed");
+
+  for (i=0; i<10; i++)
+    {
+      drawables[i] = pgm_image_new ();
+    }
+
+  ASSERT_PGM_ERROR_WITH_CRITICAL
+    (pgm_canvas_add_many (NULL, PGM_DRAWABLE_MIDDLE,
+                         drawables[0],
+                         drawables[1],
+                         drawables[2],
+                         NULL));
+  ASSERT_PGM_SUCCESS (pgm_canvas_add_many (canvas,
+                                           PGM_DRAWABLE_MIDDLE,
+                                           NULL));
+  ASSERT_PGM_SUCCESS (pgm_canvas_add_many (canvas,
+                                           PGM_DRAWABLE_MIDDLE,
+                                           drawables[0],
+                                           drawables[1],
+                                           drawables[2],
+                                           drawables[3],
+                                           drawables[4],
+                                           drawables[5],
+                                           drawables[6],
+                                           drawables[7],
+                                           drawables[8],
+                                           drawables[9],
+                                           NULL));
+
+  ASSERT_PGM_ERROR_WITH_CRITICAL
+    (pgm_canvas_remove_many (NULL, drawables[0], NULL));
+  ASSERT_PGM_SUCCESS (pgm_canvas_remove_many (canvas, NULL));
+  /* check that the preceding call to remove_many did not remove anything */
+  for (i=0; i<10; i++)
+    {
+      ASSERT_OBJECT_REFCOUNT (drawables[i], "drawables", 1);
+    }
+
+  ASSERT_PGM_SUCCESS (pgm_canvas_remove_many (canvas,
+                                           drawables[0],
+                                           drawables[1],
+                                           drawables[2],
+                                           drawables[3],
+                                           drawables[4],
+                                           drawables[5],
+                                           drawables[6],
+                                           drawables[7],
+                                           drawables[8],
+                                           drawables[9],
+                                           NULL));
+
+  gst_object_unref (canvas);
+}
+PGM_END_TEST
 
 /* test drawable floating reference on canvas unref */
-GST_START_TEST (test_canvas_add_drawable_unref_object)
+PGM_START_TEST (test_canvas_add_drawable_unref_object)
 {
   PgmError ret;
-  gboolean success;
   PgmCanvas *canvas;
   PgmDrawable *drawable;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
 
   /* create a new floating drawable with refcount 1 */
@@ -101,12 +157,11 @@ GST_START_TEST (test_canvas_add_drawable_unref_object)
 
   /* clean up our own reference */
   gst_object_unref (GST_OBJECT_CAST (drawable));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 /* test insertion of z-ordered drawables */
-GST_START_TEST (test_canvas_z_ordered_insertion)
+PGM_START_TEST (test_canvas_z_ordered_insertion)
 {
   PgmCanvas *canvas = NULL;
   PgmDrawable *img1, *img2, *img3, *img4, *img5;
@@ -116,9 +171,6 @@ GST_START_TEST (test_canvas_z_ordered_insertion)
   gint order;
   PgmError ret;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
   img1 = pgm_image_new ();
   pgm_drawable_set_position (img1, 0.0f, 0.0f, 0.0f);
@@ -216,12 +268,11 @@ GST_START_TEST (test_canvas_z_ordered_insertion)
   fail_if (order != 4, "bad drawable order");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 /* test insertion of z-misordered drawables */
-GST_START_TEST (test_canvas_z_misordered_insertion)
+PGM_START_TEST (test_canvas_z_misordered_insertion)
 {
   PgmCanvas *canvas = NULL;
   PgmDrawable *img1, *img2, *img3, *img4, *img5;
@@ -231,9 +282,6 @@ GST_START_TEST (test_canvas_z_misordered_insertion)
   gint order;
   PgmError ret;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
   img1 = pgm_image_new ();
   pgm_drawable_set_position (img1, 0.0f, 0.0f, 1.0f);
@@ -331,13 +379,12 @@ GST_START_TEST (test_canvas_z_misordered_insertion)
   fail_if (order != 4, "bad drawable order");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 /* test insertion of a drawable in a layer having drawables with the
  * same z-position */
-GST_START_TEST (test_canvas_add_with_same_z_position)
+PGM_START_TEST (test_canvas_add_with_same_z_position)
 {
   PgmCanvas *canvas = NULL;
   PgmDrawable *img1, *img2, *img3, *img4, *img5;
@@ -345,9 +392,6 @@ GST_START_TEST (test_canvas_add_with_same_z_position)
   PgmDrawableLayer layer;
   gint order;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
   img1 = pgm_image_new ();
   pgm_drawable_set_position (img1, 0.0f, 0.0f, 0.0f);
@@ -372,12 +416,11 @@ GST_START_TEST (test_canvas_add_with_same_z_position)
   fail_if (order != 3, "bad drawable order");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 /* test canvas reordering */
-GST_START_TEST (test_canvas_reordering)
+PGM_START_TEST (test_canvas_reordering)
 {
   PgmCanvas *canvas = NULL;
   PgmDrawable *img1, *img2, *img3, *img4, *img5, *img6, *img7, *img8;
@@ -386,9 +429,6 @@ GST_START_TEST (test_canvas_reordering)
   PgmDrawableLayer layer;
   gint order;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
   img1 = pgm_image_new ();
   pgm_drawable_set_position (img1, 0.0f, 0.0f, 0.0f);
@@ -489,12 +529,11 @@ GST_START_TEST (test_canvas_reordering)
   fail_if (order != 1, "drawable order should be 1");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 /* test canvas reordering */
-GST_START_TEST (test_canvas_z_reordering)
+PGM_START_TEST (test_canvas_z_reordering)
 {
   PgmCanvas *canvas = NULL;
   PgmDrawable *img1, *img2, *img3, *img4, *img5;
@@ -503,9 +542,6 @@ GST_START_TEST (test_canvas_z_reordering)
   PgmDrawableLayer layer;
   gint order;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
   img1 = pgm_image_new ();
   pgm_drawable_set_position (img1, 0.0f, 0.0f, 0.0f);
@@ -564,21 +600,17 @@ GST_START_TEST (test_canvas_z_reordering)
   fail_if (order != 4, "drawable order should be 4");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 /* test default values */
-GST_START_TEST (test_canvas_default_values)
+PGM_START_TEST (test_canvas_default_values)
 {
   PgmCanvas *canvas;
   PgmError ret;
   gboolean success;
   gfloat width, height;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
 
   /* size */
@@ -587,21 +619,17 @@ GST_START_TEST (test_canvas_default_values)
   fail_if (width != 4.0f || height != 3.0f, "bad default size");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST;
+PGM_END_TEST;
 
 /* test set/get correctness */
-GST_START_TEST (test_canvas_set_get_correctness)
+PGM_START_TEST (test_canvas_set_get_correctness)
 {
   PgmCanvas *canvas;
   PgmError ret;
   gboolean success;
   gfloat width, height;
 
-  success = pgm_init_check (NULL, NULL);
-  fail_unless (success, "init_check returned FALSE");
-
   canvas = pgm_canvas_new ();
 
   /* size */
@@ -612,9 +640,8 @@ GST_START_TEST (test_canvas_set_get_correctness)
   fail_if (width != 16.0f || height != 9.0f, "size not set");
 
   gst_object_unref (GST_OBJECT_CAST (canvas));
-  pgm_deinit ();
 }
-GST_END_TEST
+PGM_END_TEST
 
 Suite *
 pgm_canvas_suite (void)
@@ -622,8 +649,11 @@ pgm_canvas_suite (void)
   Suite *s = suite_create ("PgmCanvas");
   TCase *tc_chain = tcase_create ("pgmcanvas tests");
 
+  PGM_USE_STANDARD_FIXTURES (tc_chain);
+
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_canvas_add_del_drawable);
+  tcase_add_test (tc_chain, test_canvas_add_del_many);
   tcase_add_test (tc_chain, test_canvas_add_drawable_unref_object);
   tcase_add_test (tc_chain, test_canvas_z_ordered_insertion);
   tcase_add_test (tc_chain, test_canvas_z_misordered_insertion);
