@@ -262,9 +262,6 @@ PGM_START_TEST (test_image_from_file)
   image = PGM_IMAGE (pgm_image_new_from_file (NULL, 0));
   fail_if (NULL != image, "created an image for NULL filename");
 
-  image = PGM_IMAGE (pgm_image_new_from_file (UNEXISTING_FILE, 0));
-  fail_if (NULL != image, "created an image for unexisting file");
-
   image = PGM_IMAGE (pgm_image_new_from_file (TEST_IMAGE_FILE, 0));
   fail_if (NULL == image, "image creation failed with right params");
   ASSERT_PGM_SUCCESS (pgm_image_get_storage_type (image, &storage));
@@ -285,11 +282,6 @@ PGM_START_TEST (test_image_from_file)
   fail_unless (PGM_IMAGE_EMPTY == storage,
                "storage type should be PGM_IMAGE_EMPTY");
 
-  ASSERT_PGM_ERROR (pgm_image_set_from_file (image, UNEXISTING_FILE, 0));
-  ASSERT_PGM_SUCCESS (pgm_image_get_storage_type (image, &storage));
-  fail_unless (PGM_IMAGE_EMPTY == storage,
-               "storage type should be PGM_IMAGE_EMPTY");
-
   ASSERT_PGM_SUCCESS (pgm_image_set_from_file (image, TEST_IMAGE_FILE, 0));
 
   gst_object_unref (image);
@@ -348,7 +340,7 @@ PGM_START_TEST (test_image_small_image)
   image = pgm_image_new ();
   fail_unless (NULL != image, "image creation");
 
-  g_signal_connect (image, "pixbuf-loaded", G_CALLBACK (image_loaded_cb),
+  g_signal_connect (image, "file-loaded", G_CALLBACK (image_loaded_cb),
                     &image_loaded);
 
   ASSERT_PGM_SUCCESS (pgm_image_set_from_file (PGM_IMAGE (image),
